\name{DynamicGraph-class}

\docType{class}

\alias{DynamicGraph-class}
\alias{Str}
\alias{Str,DynamicGraph-method}
\alias{Str,NULL-method}
\alias{Str,integer-method}
\alias{Str,numeric-method}
% \alias{Str,list-method}
% \alias{Str,dg.list-method}
\alias{Str,DynamicGraph-method}
\alias{show,DynamicGraph-method}

\title{Class "DynamicGraph" }

\description{ The class for the dynamicGraph. }

\section{Objects from the Class}{
 An object of this class \code{DynamicGraph-class}
 is the returned value from \code{\link{dynamicGraphMain}}.

 The object has the lists of vertices and blocks (block trees)
 of the dynamicGraph, and a list of models, each (of class 
 \code{\link{DynamicGraphModel-class}}) with the views of the model.
 
 Each view (of class \code{\link{DynamicGraphView-class}}) of a model
 will hold the edges (edges between vertices, factors and blocks)
 and factor- and extra-vertices of the view, together with 
 vectors of indices of vertices and blocks visible in the view.

 Objects of the class \code{\link{DynamicGraph-class}} is the
 outer frame holding the "data" and the models, each model is of the class 
 \code{\link{DynamicGraphModel-class}} with one or more views of the class 
 \code{\link{DynamicGraphView-class}} in a sub-frame for the model.

 If the argument \code{returnLink} is TRUE for \code{\link{dynamicGraphMain}}
 then the object returned from \code{\link{dynamicGraphMain}} will have
 the function \code{\link{drawModel}} for drawing other models and views
 associated with the returned object of \code{\link{DynamicGraph-class}}.

 % Objects of class \code{DynamicGraph-class} can be created 
 % by calls of the form \code{new("DynamicGraph", ...)}.
}

\section{Slots}{
  \describe{
    \item{\code{title}:}{Object of class \code{"character"}:
        The value of the argument \code{title} of 
        \code{\link{dynamicGraphMain}}, 
        the header label, \code{tktitle}, of the window,
	also printed by \code{Str}. }
    \item{\code{vertices}:}{Object of class \code{"dg.VertexList"}:
        The \code{vertexList} of the graph window. }
    \item{\code{blocks}:}{Object of class \code{"dg.BlockList"}:
        The \code{blockList} of the graph window. }
    \item{\code{blockTree}:}{Object of class \code{"list"}:
        The \code{blockTree} of the graph window. }
    % \item{\code{propagateAll}:}{Object of class \code{"logical"}: 
    %     }
    \item{\code{drawModel}:}{Object of class \code{"function"}: 
        Function for adding models to the returned \code{dynamicGraph}. }
    \item{\code{models}:}{Object of class \code{"list"}:
        A list of objects, 
        each of class \code{\link{DynamicGraphModel-class}}. }
  }
}

\section{Methods}{
  \describe{
    \item{Str}{\code{signature(object = "DynamicGraph")}:
      Compactly display the internal *str*ucture of a dynamicGraph object,
      using \code{\link{asDataFrame}} on each list of node objects. }
    \item{show}{\code{signature(object = "DynamicGraph")}: calls the method
      \code{\link{Str}}. }
  }
}

\author{Jens Henrik Badsberg}

\seealso{
\code{\link{dynamicGraphMain}},
\code{\link{DynamicGraphModel-class}}, and
\code{\link{DynamicGraphView-class}}.
}

\keyword{classes}

\eof
\name{DynamicGraph}

\alias{DynamicGraph}

\title{Simple interface to dynamicGraph}

\description{
    A simple interface to \code{dynamicGraph} in the
    sense that the graph should not be given as list of objects
    as to \code{\link{dynamicGraphMain}}. Here vertices can be 
    specified by a vector of text strings with names,
    and/or edges by pairs of the indices of the vertices.

    The function is also used to add models and views to
    an existing \code{dynamicGraph}.
}

\usage{
DynamicGraph(names = NULL, types = NULL, from = NULL, to = NULL, 
    edge.types = NULL, edge.list = NULL, labels = names,  
    blocks = NULL, block.tree = NULL, oriented = FALSE, factors = NULL, 
    texts = NULL, extra.from = NULL, extra.to = NULL, extra.edge.list = NULL, 
    object = NULL, viewType = "Simple", 
    vertexClasses = validVertexClasses(), factorClasses = validFactorClasses(),
    edgeClasses = validEdgeClasses(), viewClasses = validViewClasses(), 
    N = 3, drawblocks = TRUE, right.to.left = FALSE, nested.blocks = FALSE, 
    overlaying = TRUE, vertexColor = "red", extraVertexColor = "white", 
    edgeColor = "black", factorVertexColor = "default", factorEdgeColor = "brown", 
    blockEdgeColor = "default", blockColors = NULL, extraEdgeColor = "peru", 
    frameModels = NULL, frameViews = NULL, graphWindow = NULL, addModel = FALSE, 
    addView = FALSE, overwrite = FALSE, returnNewMaster = FALSE, 
    redraw = FALSE, ...) 
}

\arguments{
  \item{names}{ A vector with text strings for the \code{names} 
        of the vertices. }
  \item{types}{ A vector with text strings for the \code{types},
        labels of \code{\link{dg.Vertex}}, of the vertices. }
  \item{from}{ If not \code{edge.list} is given:
        The indices of the first endpoints of the edges. }
  \item{to}{ If not \code{edge.list} is given:
        The indices of the second endpoints of the edges. }
  \item{edge.types}{ A vector of text strings giving the \code{types} of the
     edges, identify which classes the edges should be of,
     containing the \code{dg.VertexEdge}.
     } 
  \item{edge.list}{ If not \code{from} and \code{to} are given:
        A list where each item specifies an edge by a vector
        of the indices or names of the vertices. }
  \item{labels}{ A vector with text strings for the \code{labels} 
        of the vertices. }
  \item{blocks}{ 
        A list defining the \code{blocks}:
        Each item is the vector of the indices of the vertices of the
	block, or the vector with text strings for the names of the vertices
	of the block. 
        The arguments \code{right.to.left}, \code{nested.blocks} 
        and \code{blockColors} are here used in \code{\link{setBlocks}}
        to control the layout of the blocks.
    }
  \item{block.tree}{ If not the argument \code{blocks} is used:
	A structure with the blocks in a \code{block.tree}.
        The arguments \code{overlaying} and \code{blockColors} are here used
        in \code{\link{setTreeBlocks}} to control the layout of the blocks.
        }
  \item{oriented}{ Logical. If \code{oriented} is set to TRUE
        then the edges are oriented, also when no block structure is given. }
  \item{factors}{A list defining the \code{factor vertices}:
        Each item is the vector of the indices of the vertices of a factor.
        }
  \item{texts}{ A vector of text strings, for additional labels. 
		These labels will be set by \code{'ExtraVertices'}
                of class \code{\link{dg.TextVertex-class}}. }
  \item{extra.from}{ If not \code{extra.edge.list} is given:
        The indices of the first endpoints of the extra edges,
        negative for extra vertices. }
  \item{extra.to}{ If not \code{extra.edge.list} is given:
        The indices of the second endpoints of the extra edges,
        negative for extra vertices. }
  \item{extra.edge.list}{ If not \code{extra.from} and \code{extra.to}
        are given: A list where each item specifies an extra edge by a vector
        of the indices or names of the vertices or extra vertices,
        negative indices for extra vertices. }
  \item{object}{ The model \code{object}, or NULL,
        see \code{\link{dg.Model-class}}. }
  \item{vertexClasses}{ % The valid \code{vertexClasses}. 
      Returned value from \code{\link{validVertexClasses}},
      or extension of this matrix. Used when creating
      new vertices in \code{\link{dynamicGraphMain}}. }
  \item{factorClasses}{ % The valid \code{factorClasses}. 
      Returned value from \code{\link{validFactorClasses}},
      or extension of this matrix. Used when creating
      new factor vertices in \code{\link{dynamicGraphMain}}. }
  \item{edgeClasses}{ Returned value from \code{\link{validEdgeClasses}},
      or extension of this matrix. Used when creating
      new edges in \code{\link{dynamicGraphMain}}. }
  \item{viewClasses}{ Returned value from \code{\link{validViewClasses}},
      or extension of this matrix. Used when creating
      new views in \code{\link{dynamicGraphMain}}. }
  \item{viewType}{ A text string with the type of view. }
  \item{N}{ The number, \code{N} > 1, of coordinates for the positions of the
        vertices and block corners. }
  \item{drawblocks}{ Logical. If \code{drawblocks} is set to FALSE,
        then the blocks are not drawn. The strata of the vertices are
        then not updated when the vertices are moved. }
  \item{right.to.left}{ Logical. If \code{right.to.left} is set to TRUE
        then the explanatory blocks are drawn to the right.
        See \code{\link{setBlocks}}. }
  \item{nested.blocks}{ Logical. If \code{nested.blocks} is set to TRUE
        then the blocks are drawn nested. See \code{\link{setBlocks}}. }
  \item{overlaying}{ Logical. If \code{overlaying} is set to FALSE
        then children of a block are not drawn inside the block.
        See \code{\link{setTreeBlocks}}. }
  \item{vertexColor}{ Single text string with the \code{vertexColor}
        of the vertices. }
  \item{extraVertexColor}{ Single text string with the \code{extraVertexColor}
        of the nodes for text labels. }
  \item{edgeColor}{ Single text string with the \code{edgeColor}
        of the edges. }
  \item{factorVertexColor}{ Single text string with the
        \code{factorVertexColor} of the factor vertices. 
        If \code{factorVertexColor} is "default" then
        the color of a factor vertex will depend on the
        type of the generator of the factor.}
  \item{factorEdgeColor}{ Single text string with the \code{factorEdgeColor}
        of the factor edges. }
  \item{blockEdgeColor}{ "default", or list with two text strings for colors.
        % \code{blockEdgeColor} is similar to \code{edgeColor}.
        The two colors are used for respectively edges
        between blocks and for edges between blocks 
        and vertices.}
  \item{blockColors}{ Vector of text string with the \code{blockColors}
        of the blocks.
        See \code{\link{setBlocks}} and \code{\link{setTreeBlocks}}.  }
  \item{extraEdgeColor}{ Single text string with the \code{extraEdgeColor}
        of the extra edges. }
  \item{frameModels}{ An object of class \code{\link{DynamicGraph-class}}.
		      \code{frameModels} is the object for a dataset
		      and the models on that dataset. }
  \item{frameViews}{ An object of class \code{\link{DynamicGraphModel-class}}.
		     \code{frameViews} is the object for a model
		     and the views of that model. }
  \item{graphWindow}{ An object of class \code{\link{DynamicGraphView-class}}.
		      \code{graphWindow} is the object for a view of a model. }
  \item{addModel}{ Logical, if \code{addModel} then a model is added
		   to the argument \code{frameModels}, and a view of
		   the model is drawn. 
		   If the argument \code{overwrite} is TRUE and
                   the argument \code{graphWindow} is given 
		   then the model of \code{graphWindow} is replaced
		   by the model argument \code{object}.
		   If the argument \code{overwrite} is TRUE and
                   the argument \code{frameViews} is given 
		   then the model of \code{frame}\-{Views} is replaced
		   by the model argument \code{object}.
		   }
  \item{addView}{ Logical, if \code{addView} then a view of type set
		  by the argument \code{viewType}
		  for the model of the argument \code{frameViews} is added. }
  \item{overwrite}{ Logical, see the argument \code{addModel}. 
                    The argument \code{returnLink} must be set to TRUE
		    to overwrite a view. }
  \item{redraw}{ Logical. If TRUE then the dynamicGraph of the
                 arguments \code{frameModels} is 'redrawn'.
                 New instances of the windows are made. }
  \item{returnNewMaster}{ Logical. Alternative implementation of 
                          \code{addModel}, using the code of \code{redraw}.
                          As \code{redraw}, but the windows of 
			  \code{frameModels} exists, 
                          and a new model is added. }
  \item{\dots}{ Additional arguments to \code{\link{dynamicGraphMain}}. }
}

\details{
    After converting the first arguments to lists of objects
    the \code{\link{dynamicGraphMain}} does all the work.

    You can look into the source code of \code{\link{DynamicGraph}} about
    how to create these lists of objects for building your applications.
    The list of objects can be exported from
    \code{\link{dynamicGraphMain}}, also after modifying the graph.

   If the argument \code{returnLink} is set to TRUE then the returned
   value can be use in subsequential call to \code{\link{DynamicGraph}}
   to add models and views to the created graph window.

}

\value{
  The returned value from \code{\link{dynamicGraphMain}}.
}

\author{Jens Henrik Badsberg}

\examples{
require(tcltk); require(dynamicGraph)

# Example 1:

Z <- DynamicGraph(paste("", 1:5), title = "Very simple")


# Example 2:

Z <- DynamicGraph(from = 1:4, to = c(2:4, 1), title = "Simply edges")


# Example 3:

V.Types <- c("Discrete", "Ordinal", "Discrete",
             "Continuous", "Discrete", "Continuous")

V.Names  <- c("Sex", "Age", "Eye", "FEV", "Hair", "Shosize")
V.Labels <- paste(V.Names, 1:6, sep ="/")

From <- c(1, 2, 3, 4, 5, 6)
To   <- c(2, 3, 4, 5, 6, 1)

Z <- DynamicGraph(V.Names, V.Types, From, To, texts = c("Gryf", "Gaf"),
                  labels = V.Labels, title = "With labels (extraVertices)")


# Example 4: Oriented (cyclic) edges, without causal structure:

Z <- DynamicGraph(V.Names, V.Types, From, To, oriented = TRUE,
                  labels = V.Labels, title = "Oriented edges")


# Example 5: A factor graph:

Factors <- list(c(1, 2, 3, 4), c(3, 4, 5), c(4, 5, 6))

Z <- DynamicGraph(V.Names, V.Types, from = NULL, to = NULL, factors = Factors,
                  title = "Factorgraph", namesOnEdges = FALSE)


# Example 6: Edges with more than two vertices:

EdgeList <- list(c(1, 2, 3, 4), c(3, 4, 5), c(4, 5, 6))

Z <- DynamicGraph(V.Names, V.Types, edge.list = EdgeList, 
                  title = "Multiple edges", namesOnEdges = FALSE)

Z
}

\keyword{ dplot }
\keyword{ hplot }
\keyword{ iplot }
\keyword{ dynamic }
\keyword{ graphs }
\keyword{ methods }
\keyword{ multivariate }

\eof
\name{DynamicGraphModel-class}

\docType{class}

\alias{DynamicGraphModel-class}
\alias{Str,DynamicGraphModel-method}
\alias{show,DynamicGraphModel-method}

% \newpage

\title{Class "DynamicGraphModel" }

\description{  The class for the models of the dynamicGraph. }

\section{Objects from the Class}{
Objects of this class is created in the \code{drawModel} function
of \code{\link{dynamicGraphMain}}.

Objects of class \code{DynamicGraphModel-class} will be a part of the returned
value of the function \code{\link{dynamicGraphMain}}, and will store the model,
the views of the model, each of class \code{\link{DynamicGraphView-class}},
and the function \code{redrawView} for drawing views of the model.

% Objects of class \code{DynamicGraphModel-class} can be created by
% calls of the form \code{new("DynamicGraphModel", ...)}.
}

\section{Slots}{
  \describe{
    \item{\code{title}:}{Object of class \code{"character"}:
        The value of the argument \code{title} of 
        \code{\link{dynamicGraphMain}}, combined with the slot \code{name}
        of the model object, printed by \code{Str}. }
    \item{\code{index}:}{Object of class \code{"numeric"}:
        The \code{index} of the model. }
    \item{\code{redrawView}:}{Object of class \code{"function"}: 
        For drawing views of the model. }
    \item{\code{model}:}{Object of class \code{"list"}:
        The \code{model} of the graph windows. }
    \item{\code{graphs}:}{Object of class \code{"list"}:
        A list of view objects,
        each of class \code{\link{DynamicGraphView-class}}. }
  }
}

\section{Methods}{
  \describe{
    \item{Str}{\code{signature(object = "DynamicGraphModel")}:
      Compactly display the internal *str*ucture of a dynamicGraph model object. }
    \item{show}{\code{signature(object = "DynamicGraph")}: calls the method
      \code{\link{Str}}. }
  }
}

\author{Jens Henrik Badsberg}

\seealso{ 
\code{\link{dynamicGraphMain}},
\code{\link{DynamicGraph-class}}, and 
\code{\link{DynamicGraphView-class}}.
}

\keyword{classes}

\eof
\name{DynamicGraphView-class}

\docType{class}

\alias{DynamicGraphView-class}
\alias{Str,DynamicGraphView-method}
\alias{show,DynamicGraphView-method}
\alias{EssentialDynamicGraphView-class}
\alias{FactorDynamicGraphView-class}
\alias{MoralDynamicGraphView-class}
\alias{SimpleDynamicGraphView-class}

\title{Class "DynamicGraphView" }

\description{ The class for views of the models of the dynamicGraph. }

\section{Objects from the Class}{
Objects of the class \code{DynamicGraphView-class} is created
in the \code{redrawView} function in the \code{drawModel}
function of \code{\link{dynamicGraphMain}}.

Objects of this class will be a part of each model in the model list
of the returned value from \code{\link{dynamicGraphMain}},
and will store the view specific items of a model,
plus the identification of the Tcl/tk window.

% Objects of class \code{DynamicGraphView-class} can be created
% by calls of the form \code{new("DynamicGraphView", ...)}.
}

\section{Slots}{
  \describe{
    \item{\code{title}:}{Object of class \code{"character"}:
        The value of the argument \code{title} of 
        \code{\link{dynamicGraphMain}}, combined with the
        \code{viewType} of the model object, printed by \code{Str}. }
    \item{\code{index}:}{Object of class \code{"numeric"}:
       The \code{index} of the view. }
    \item{\code{top}:}{Object of class \code{"tkwin"}:
       The \code{tkwin}-top of the graph window. }
    \item{\code{canvas}:}{Object of class \code{"tkwin"}:
       The \code{canvas}-top of the graph window. }
    \item{\code{tags}:}{Object of class \code{"list"}:
       The tags of the objects drawn in the graph window. }
    \item{\code{update}:}{Object of class \code{"function"}: 
        For updating the view and slots of the view. }
    \item{\code{id}:}{Object of class \code{"numeric"}:
       Integer, increased for each redraw of the graph window. }
    \item{\code{viewLabel}:}{Text string with the type of the view. }
    % \item{\code{setVertexLabels}:}{Object of class \code{"logical"}: ... }
    % \item{\code{object}:}{Object of class \code{"list"}:
    %    The object of the graph. }
    \item{\code{visibleVertices}:}{Object of class \code{"numeric"}:
       Numeric vector of the indices of the vertices of
       \code{vertexList} plotted in the view. }
    \item{\code{visibleBlocks}:}{Object of class \code{"numeric"}:
       Numeric vector of the indices of the blocks of
       \code{blockList} plotted in the view. }
    \item{\code{extraVertices}:}{Object of class \code{"dg.Vertexlist"}:
       The \code{extraList} of the view. }
    \item{\code{vertexEdges}:}{Object of class \code{"dg.VertexEdgeList"}:
       The \code{edgeList} of the view. }
    \item{\code{blockEdges}:}{Object of class \code{"dg.BlockEdgeList"}:
       The \code{blockEdgeList} of the view. }
    \item{\code{factorVertices}:}{Object of class \code{"dg.FactorVertexList"}:
       The \code{factorVertexList} of the view. }
    \item{\code{factorEdges}:}{Object of class \code{"dg.FactorEdgeList"}:
       The \code{factorEdgeList} of the view. }
    \item{\code{extraEdges}:}{Object of class \code{"dg.ExtraEdgeList"}:
       The \code{extraEdgeList} of the view. }
  }
}

\section{Methods}{
  \describe{
    \item{Str}{\code{signature(object = "DynamicGraphView")}:
      Compactly display the internal *str*ucture of a dynamicGraph view object. }
    \item{show}{\code{signature(object = "DynamicGraph")}: calls the method
      \code{\link{Str}}. }
  }
}

\author{Jens Henrik Badsberg}

\seealso{
\code{\link{dynamicGraphMain}},
\code{\link{DynamicGraph-class}}, and
\code{\link{DynamicGraphModel-class}}.
}

\keyword{classes}

\eof
\name{blockTreeToList}

\alias{blockTreeToList}

\title{Extract the list of blocks from a block tree}

\description{Returns the list of blocks from a block tree.}

\usage{blockTreeToList(tree)}

\arguments{
  \item{tree}{ The \code{BlockTree} part of the result of
               \code{\link{setTreeBlocks}}. }
}

\value{A list of blocks, each block of class \code{dg.Block}.}

\author{Jens Henrik Badsberg}

\examples{
Block.tree <- list(label = "W", Vertices = c("contry"),
                   X = list(Vertices = c("sex", "race"),
                            A = list(Vertices = c("hair", "eye"),
                                     horizontal = FALSE),
                            B = list(Vertices = c("age"),
                                     C = list(Vertices = c("education")))))
Names <- unlist(Block.tree)
Names <- Names[grep("Vertices", names(Names))]
Types <- rep("Discrete", length(Names))
vertices <- returnVertexList(Names, types = Types)
blocktree <- setTreeBlocks(Block.tree, vertices)
Labels(blockTreeToList(blocktree$BlockTree))
}

\keyword{ methods }
\keyword{ dynamic }
\keyword{ graphs }

\eof
\name{dg.Block-class}

\docType{class}

\alias{dg.Block-class}
\alias{ancestors<-,dg.Block-method}
\alias{ancestors,dg.Block-method}
\alias{closed<-,dg.Block-method}
\alias{closed,dg.Block-method}
\alias{descendants<-,dg.Block-method}
\alias{descendants,dg.Block-method}
\alias{draw,dg.Block-method}
\alias{index<-,dg.Block-method}
\alias{index,dg.Block-method}
\alias{name,dg.Block-method}
\alias{position<-,dg.Block-method}
\alias{position,dg.Block-method}
\alias{stratum<-,dg.Block-method}
\alias{stratum,dg.Block-method}
\alias{visible<-,dg.Block-method}
\alias{visible,dg.Block-method}

\alias{newBlock}

\alias{newBlock}
\alias{dg.Block}
\alias{closed}
\alias{closed<-}
\alias{ancestors}
\alias{ancestors<-}
\alias{descendants}
\alias{descendants<-}

\title{Class "dg.Block" }

\description{  The class for the blocks. }

% \section{Objects from the Class}{
% Objects can be created by calls of the form \code{new("dg.Block", ...)}.
% }

\usage{
newBlock(stratum = 0, index = 0, position = matrix(rep(0, 6), ncol = 3),
         closed = FALSE, visible = TRUE, color = "Grey", label = "Label",
         label.position = c(0, 0, 0), ancestors = NULL)
}

\arguments{
  \item{stratum}{ Integer for the \code{stratum} of the block. 
	% The block is identified by this stratum, and thus the
        % strata of the blocks are different and unique.
        % Each vertex can only belong to one block,
	% and thus has assigned one stratum.
        }
  \item{index}{ Integer for the \code{index} of the block. }
  \item{position}{ Numeric vector for the \code{position} of the block,
      the two diagonal opposite corners. }
  \item{closed}{ Logical, if TRUE then the block is drawn as a "vertex". }
  \item{visible}{ Logical, if TRUE then the block is drawn. }
  \item{color}{ Text string for the \code{color} of the block. }
  \item{label}{ Text string for the \code{label} of the block. }
  \item{label.position}{ Numeric vector for the \code{label.position} 
                         of the block label. }
  \item{ancestors}{ Integer vector for the \code{ancestor} blocks of the
                    block. 
                    The slot with the \code{descendant} blocks is set
                    after the creation of the block. }
}

\section{Slots}{
  \describe{
    \item{\code{stratum}:}{Object of class \code{"numeric"},
       the \code{stratum} of the block. }
    \item{\code{index}:}{Object of class \code{"numeric"} 
       with (minus) the \code{index} of the block,
       the position of the block in a block list. }
    \item{\code{ancestors}:}{Object of class \code{"numeric"}.
       Integer vector for the \code{ancestor} blocks of the block. }
    \item{\code{descendants}:}{Object of class \code{"numeric"}.
       Integer vector for the \code{descendants} blocks of the block. 
       The slot with the \code{descendant} blocks is set after the creation of the block. }
    \item{\code{position}:}{Object of class \code{"matrix"},
       the \code{position} of the block, the two diagonal opposite corners. }
    \item{\code{closed}:}{Object of class \code{"logical"},
       if TRUE then the block is closed, e.i. drawn as a "vertex". }
    \item{\code{visible}:}{Object of class \code{"logical"},
       if TRUE then the block is drawn, else it is hidden in a closed block. }
    \item{\code{color}:}{Object of class \code{"character"}, see \code{"dg.Node"}. }
    \item{\code{label}:}{Object of class \code{"character"}, see \code{"dg.Node"}. }
    \item{\code{label.position}:}{Object of class \code{"numeric"}, see \code{"dg.Node"}. } 
  }
}

\section{Extends}{
Class \code{"dg.Node"}, directly.
}

\section{Methods}{
  \describe{
    \item{ancestors<-}{\code{signature(x = "dg.Block")}: ... }
    \item{ancestors}{\code{signature(object = "dg.Block")}: ... }
    \item{closed<-}{\code{signature(x = "dg.Block")}: ... }
    \item{closed}{\code{signature(object = "dg.Block")}: ... }
    \item{descendants<-}{\code{signature(x = "dg.Block")}: ... }
    \item{descendants}{\code{signature(object = "dg.Block")}: ... }
    \item{draw}{\code{signature(object = "dg.Block")}:
       Method for drawing the closed block. }
    \item{index<-}{\code{signature(x = "dg.Block")}: ... }
    \item{index}{\code{signature(object = "dg.Block")}: ... }
    \item{name}{\code{signature(object = "dg.Block")}: 
       Extract the label of the block. }
    \item{position<-}{\code{signature(x = "dg.Block")}: ... }
    \item{position}{\code{signature(object = "dg.Block")}: ... }
    \item{stratum<-}{\code{signature(x = "dg.Block")}: ... }
    \item{stratum}{\code{signature(object = "dg.Block")}: ... }
    \item{visible<-}{\code{signature(x = "dg.Block")}: ... }
    \item{visible}{\code{signature(object = "dg.Block")}: ... }
    % \item{propertyDialog}{\code{signature(object = "dg.Block")}: ... }
    % \item{addToPopups}{\code{signature(object = "dg.Block")}: ... }
  }
}

\author{Jens Henrik Badsberg}

\seealso{
% \code{\link{newBlock}},
\code{\link{setBlocks}},
\code{\link{setTreeBlocks}},
\code{\link{dg.Node-class}}.
}

\note{
    The \code{dg.Block} class has the methods
    \code{\link{name}}, \code{\link{label}}, \code{\link{labelPosition}},
    \code{\link{position}}, \code{\link{color}}, \code{\link{stratum}},
    \code{\link{index}}, \code{\link{visible}},
    \code{\link{ancestors}}, and \code{\link{descendants}},
    for extracting values
    and the replacement methods
    % \code{\link{name<-}}, 
    \code{\link{label<-}}, \code{\link{labelPosition<-}},
    \code{\link{position<-}}, \code{\link{color<-}}, \code{\link{stratum<-}},
    \code{\link{index<-}}, \code{\link{visible<-}},
    \code{\link{ancestors<-}}, and \code{\link{descendants<-}}.
    Items are added to the pop up menu of a block
    by the method \code{\link{addToPopups}}.

    The method \code{\link{propertyDialog}} is also avaliable.

    I have not yet had any use of a block name.
    Thus this slot is not available for the block,
    as neither for edges.
}

% \seealso{\code{\link{newVertex}}}

\examples{
b <- newBlock()
str(b)

color(b)
label(b)
labelPosition(b)
name(b)
index(b)
position(b)
stratum(b)
ancestors(b)
descendants(b)
visible(b)

color(b) <- "grey"
label(b) <- "NameAndLabel"
labelPosition(b) <- c(1, 2, 3)
# name(b) <- "NameAndLabel" # Not possible!!!
index(b) <- 3
position(b) <- matrix(c( 10,  20,  30,  40, 
                        110, 120, 130, 140), byrow = TRUE, ncol = 4)
stratum(b) <- 1
ancestors(b) <- c(1, 2)
descendants(b) <- c(4, 5)
visible(b) <- FALSE

str(b)
}

\keyword{classes}

\eof
\name{dg.BlockEdge-class}

\docType{class}

\alias{dg.BlockEdge-class}
\alias{nodeTypesOfEdge,dg.BlockEdge-method}
\alias{oriented<-,dg.BlockEdge-method}
\alias{oriented,dg.BlockEdge-method}

\alias{newBlockEdge}

\alias{newBlockEdge}
\alias{dg.BlockEdge}.


\title{Class "dg.BlockEdge" }

\description{  The class for edges between blocks and for
               edges between vertices and blocks. }

% \section{Objects from the Class}{
% Objects can be created by calls of the form \code{new("dg.BlockEdge", ...)}.
% }

\usage{
newBlockEdge(node.indices, nodes = NULL, width = 2,
             color = "DarkOliveGreen", oriented = TRUE, type = NULL,
             label = ifelse(is.null(nodes), "",
                            paste(Labels(nodes), collapse = "~")),
             dash = "")
}

\arguments{
  \item{node.indices}{ Vector with \code{abs(node.indices)}
                       the indices of the nodes of the block edge.
                       If the index is positiv then the node is a vertex,
                       else it is a block. }
  \item{nodes}{ List with the \code{nodes} of the block edge. Used for
      setting the default label. }
  \item{width}{ Numeric for the \code{width} of the block edge. }
  \item{color}{ Text string with the \code{color} of the block edge. }
  \item{oriented}{ Logical. If TRUE then the block edge is \code{oriented}. }
  \item{type}{ Text string with the \code{type} of the block edge. }
  \item{label}{ Text string with the \code{label} of the block edge. }
  \item{dash}{ Text string for the \code{color} pattern of the edge. }
}

\details{
The function is used in \code{\link{returnBlockEdgeList}}.
\code{\link{dynamicGraphMain}} will automatic update block edges
when vertices are moved between blocks.
}

\section{Slots}{
  \describe{
    \item{\code{oriented}:}{Object of class \code{"logical"}, see \code{"dg.VertexEdge"}. }
    \item{\code{vertex.indices}:}{Object of class \code{"numeric"}, see \code{"dg.Edge"}. }
    \item{\code{width}:}{Object of class \code{"numeric"}, see \code{"dg.Edge"}. }
    \item{\code{dash}:}{Object of class \code{"character"}, see \code{"dg.Edge"}. }
    \item{\code{color}:}{Object of class \code{"character"}, see \code{"dg.Edge"}. }
    \item{\code{label}:}{Object of class \code{"character"}, see \code{"dg.Edge"}. }
    \item{\code{label.position}:}{Object of class \code{"numeric"}, see \code{"dg.Edge"}. }
  }
}

\section{Extends}{
Class \code{"dg.Edge"}, directly.
Class \code{"dg.Node"}, by class \code{"dg.Edge"}.
}

\section{Methods}{
  \describe{
    \item{nodeTypesOfEdge}{\code{signature(object = "dg.BlockEdge")}:
       Extract the types ("super classes": \code{"Vertex"} or \code{"Block"})
       of the vertices (nodes) of the edge. }
    \item{oriented<-}{\code{signature(x = "dg.BlockEdge")}: ... }
    \item{oriented}{\code{signature(object = "dg.BlockEdge")}: ... }
    % \item{propertyDialog}{\code{signature(object = "dg.BlockEdge")}: ... }
    % \item{propertyDialog}{\code{signature(object = "dg.BlockEdge")}: ... }
  }
}

\author{Jens Henrik Badsberg}

\seealso{
% \code{\link{newBlockEdge}},
\code{\link{returnBlockEdgeList}},
\code{\link{dg.Edge-class}}.
}

\note{ 
   The methods of \code{\link{dg.Edge}}
   also applies for \code{dg.BlockEdge}.
}

% \seealso{\code{\link{newVertexEdge}}}

\examples{ 
vertices <- returnVertexList(paste("V", 1:4, sep = ""))
block <- newBlock(stratum = 1)
blockedge <- newBlockEdge(c(4, -1), 
                          new("dg.VertexList", list(vertices[[1]], block)))

str(blockedge)

color(blockedge)
label(blockedge)
labelPosition(blockedge)
width(blockedge)
nodeIndicesOfEdge(blockedge)
nodeTypesOfEdge(blockedge)

color(blockedge) <- "Black"
label(blockedge) <- "V1~1"
labelPosition(blockedge) <- c(0, 1, 2)
width(blockedge) <- 1
nodeIndicesOfEdge(blockedge) <- c(1, -1)

str(blockedge)
}

\keyword{classes}

\eof
\name{dg.DiscreteVertex-class}

\docType{class}

\alias{dg.DiscreteVertex-class}
\alias{draw,dg.DiscreteVertex-method}
\alias{dg.ContinuousVertex-class}
\alias{draw,dg.ContinuousVertex-method}
\alias{dg.OrdinalVertex-class}
\alias{draw,dg.OrdinalVertex-method}

\title{Classses "dg.DiscreteVertex", "dg.ContinuousVertex", and "dg.OrdinalVertex" }

\description{ The class for vertices for discrete variables, 
 continuous variables, and ordinal variables. }

\section{Objects from the Class}{
Objects has the methods for extracting and setting the slots for vertices,
and the method for drawing the vertex.

% Objects can be created by calls of the form
%  \code{new("dg.DiscreteVertex", ...)},
%  \code{new("dg.ContinuousVertex", ...)},
%  and \code{new("dg.OrdinalVertex", ...)}.
}

\section{Slots}{
  \describe{
    \item{\code{name}:}{Object of class \code{"character"}, see \code{"dg.Vertex"}. }
    \item{\code{index}:}{Object of class \code{"numeric"}, see \code{"dg.Vertex"}. }
    \item{\code{position}:}{Object of class \code{"numeric"}, see \code{"dg.Vertex"}. }
    \item{\code{blockindex}:}{Object of class \code{"numeric"}, see \code{"dg.Vertex"}. }
    \item{\code{stratum}:}{Object of class \code{"numeric"}, see \code{"dg.Vertex"}. }
    \item{\code{color}:}{Object of class \code{"character"}, see \code{"dg.Vertex"}. }
    \item{\code{label}:}{Object of class \code{"character"}, see \code{"dg.Vertex"}. }
    \item{\code{label.position}:}{Object of class \code{"numeric"}, see \code{"dg.Vertex"}. }
  }
}

\section{Extends}{
Class \code{"dg.Vertex"}, directly.
Class \code{"dg.Node"}, by class \code{"dg.Vertex"}.
}

\section{Methods}{
  \describe{
    \item{draw}{\code{signature(object = "dg.DiscreteVertex")}: Method for drawing the vertex.
    The symbol will be a \code{'d'ot} for \code{'d'iscrete} variable. }
    \item{draw}{\code{signature(object = "dg.OrdinalVertex")}: Method for drawing the vertex.
    The symbol will be a \code{filled square} for \code{ordinal} variable. }
    \item{draw}{\code{signature(object = "dg.ContinuousVertex")}: Method for drawing the vertex.
    The symbol will be a \code{'c'circle} for \code{'c'ontinuous} variable. }
  }
}

\author{Jens Henrik Badsberg}

\seealso{
\code{\link{newVertex}},
\code{\link{returnVertexList}},
\code{\link{dg.Vertex-class}}.
}

\keyword{classes}

\eof
\name{dg.Edge-class}

\docType{class}

\alias{dg.Edge-class}
\alias{draw,dg.Edge-method}
\alias{name,dg.Edge-method}
\alias{nodeIndicesOfEdge<-,dg.Edge-method}
\alias{nodeIndicesOfEdge,dg.Edge-method}
\alias{label<-,dg.Edge-method}
\alias{label,dg.Edge-method}
\alias{width<-,dg.Edge-method}
\alias{width,dg.Edge-method}
\alias{dash,dg.Edge-method}
\alias{dash<-,dg.Edge-method}
\alias{nodeIndices,dg.Edge-method}
\alias{propertyDialog,dg.Edge-method}

\title{Class "dg.Edge" }

\description{  A skeleton class for the classes of edges. }

\section{Objects from the Class}{
Objects has the methods for extracting and setting the slots for edges,
and the method for drawing the edge.

% Objects can be created by calls of the form \code{new("dg.Edge", ...)}.
}

\section{Slots}{
  \describe{
    \item{\code{vertex.indices}:}{Object of class \code{"numeric"}:
       The \code{vertex.indices} of the vertices of the edge. }
    \item{\code{width}:}{Object of class \code{"numeric"}:
       The \code{width} of the edge. }
    \item{\code{dash}:}{Object of class \code{"character"}:
       The \code{dash} pattern of the edge.

From the Tcl/tk Reference Manual:

% \begin{verbatim}

"DASH PATTERNS

Many items support the notion of an dash pattern for outlines. 

The first possible syntax is a list of integers. Each element represents the
number of pixels of a line segment. Only the odd segments are drawn using the
"outline" color. The other segments are drawn transparant. 

The second possible syntax is a character list containing only 5 possible
characters $[.,-\_ ]$. The space can be used to enlarge the space between other
line elements, and can not occur as the first position in the string. Some
examples:    
 -dash . = -dash \{2 4\}
 -dash - = -dash \{6 4\}
 -dash -. = -dash \{6 4 2 4\}
 -dash -.. = -dash \{6 4 2 4 2 4\}
 -dash \{. \} = -dash \{2 8\}
 -dash , = -dash \{4 4\}

The main difference of this syntax with the previous is that it it
shape-conserving. This means that all values in the dash list will be
multiplied by the line width before display. This assures that "." will always
be displayed as a dot and "-" always as a dash regardless of the line width. 

On systems which support only a limited set of dash patterns, the dash pattern
will be displayed as the closest dash pattern that is available. For example,
on Windows only the first 4 of the above examples are available.  The last 2
examples will be displayed identically to the first one. "
% \end{verbatim}

 }
    \item{\code{color}:}{Object of class \code{"character"}:
       The \code{color} of the edge. }
    \item{\code{label}:}{Object of class \code{"character"}:
       The \code{label} of the edge. }
    \item{\code{label.position}:}{Object of class \code{"numeric"}:
       The \code{label.position} of the edge. }
  }
}

\section{Extends}{
Class \code{"dg.Node"}, directly.
}

\section{Methods}{
  \describe{
    \item{dash<-}{\code{signature(x = "dg.Edge")} :
       Set the \code{dash} pattern of the edge. }
    \item{dash}{\code{signature(object = "dg.Edge")}:
       Return the \code{dash} pattern of the edge. }
    \item{draw}{\code{signature(object = "dg.Edge")}: ... }
    \item{label<-}{\code{signature(x = "dg.Edge")}:
       Set the \code{label} of the edge. }
    \item{label}{\code{signature(object = "dg.Edge")}:
       Return the \code{label} of the edge. }
    \item{name}{\code{signature(object = "dg.Edge")}: 
       Return the \code{name}, equal to the \code{label}, of the edge. }
    \item{nodeIndicesOfEdge<-}{\code{signature(x = "dg.Edge")}:
       Set the \code{indices} of the vertices of the edge. }
    \item{nodeIndicesOfEdge}{\code{signature(object = "dg.Edge")}:
       Return the \code{indices} of the vertices of the edge. }
    \item{width<-}{\code{signature(x = "dg.Edge")}:
       Set the \code{width} of the edge. }
    \item{width}{\code{signature(object = "dg.Edge")}:
       Return the \code{width} of the edge. }
    % \item{propertyDialog}{\code{signature(object = "dg.Edge")}: ... }
    % \item{addToPopups}{\code{signature(object = "dg.Edge")}: ... }
  }
}

\author{Jens Henrik Badsberg}

\seealso{
\code{\link{dg.VertexEdge-class}},
\code{\link{dg.BlockEdge-class}},
\code{\link{dg.FactorEdge-class}}.
}

\keyword{classes}

\eof
\name{dg.ExtraEdge-class}

\docType{class}

\alias{dg.ExtraEdge-class}
\alias{nodeTypesOfEdge,dg.ExtraEdge-method}

\alias{newExtraEdge}

\alias{newExtraEdge}
\alias{dg.ExtraEdge}

\title{Class "dg.ExtraEdge" }

\description{  The class for the edges between vertices and extra vertices. }

% \section{Objects from the Class}{
% Objects can be created by calls of the form \code{new("dg.ExtraEdge", ...)}.
% }

\usage{
newExtraEdge(node.indices, nodes = NULL, width = 2,
             color = "DarkOliveGreen", type = NULL,
             label = ifelse(is.null(nodes), "",
                            paste(Labels(nodes), collapse = "~")),
             dash = "")
}

\arguments{
  \item{node.indices}{ Vector with \code{abs(node.indices)}
                       the indices of the nodes of the extra edge.
                       If the index is positiv then the node is a vertex,
                       else it is the extra vertex. }
  \item{nodes}{ List with the \code{nodes} of the extra edge. 
      Used for setting the default label. }
  \item{width}{ Numeric for the \code{width} of the extra edge. }
  \item{color}{ Text string with the \code{color} of the extra edge. }
  \item{type}{ Text string with the \code{type} of the extra edge. }
  \item{label}{ Text string with the \code{label} of the extra edge. }
  \item{dash}{ Text string for the \code{dash} pattern of the edge. }
}

\section{Slots}{
  \describe{
    \item{\code{vertex.indices}:}{Object of class \code{"numeric"},
          see \code{"dg.Edge"}. }
    \item{\code{width}:}{Object of class \code{"numeric"},
          see \code{"dg.Edge"}. }
    \item{\code{dash}:}{Object of class \code{"character"},
          see \code{"dg.Edge"}. }
    \item{\code{color}:}{Object of class \code{"character"},
          see \code{"dg.Edge"}. }
    \item{\code{label}:}{Object of class \code{"character"},
          see \code{"dg.Edge"}. }
    \item{\code{label.position}:}{Object of class \code{"numeric"},
          see \code{"dg.Edge"}. }
  }
}

\section{Extends}{
Class \code{"dg.Edge"}, directly.
Class \code{"dg.Node"}, by class \code{"dg.Edge"}.
}

\section{Methods}{
  \describe{
    \item{nodeTypesOfEdge}{\code{signature(object = "dg.ExtraEdge")}:
      Extract the types ("super classes": \code{"Vertex"} or \code{"Extra"})
      of the vertices (nodes) of the edge. }
    % \item{propertyDialog}{\code{signature(object = "dg.ExtraEdge")}: ... }
    % \item{addToPopups}{\code{signature(object = "dg.ExtraEdge")}: ... }
  }
}

\author{Jens Henrik Badsberg}

\seealso{
% \code{\link{newExtraEdge}},
\code{\link{returnExtraEdgeList}},
\code{\link{dg.Edge-class}}, and
\code{\link{dg.TextVertex-class}}.
}

% \seealso{\code{\link{newVertexEdge}}}

\note{ 
   The methods (except \code{\link{oriented}}) of \code{\link{dg.Edge}}
   also applies for \code{dg.ExtraEdge}.

   Extra vertices and nodes are used in \code{demo(dg.USArrests)}
   to display the loadings in a biplot.
 }

\examples{
vertices  <- returnVertexList(paste("V", 1:4, sep = ""))
extra     <- returnVertexList(paste("E", 1:4, sep = ""))
extraedge <- newExtraEdge(c(3, -2), 
                          new("dg.VertexList", c(vertices[3], extra[2])))
str(extraedge)

color(extraedge)
label(extraedge)
labelPosition(extraedge)
width(extraedge)
nodeIndicesOfEdge(extraedge)
nodeTypesOfEdge(extraedge)

color(extraedge) <- "Black"
label(extraedge) <- "Gryf"
labelPosition(extraedge) <- c(0, 1, 2)
width(extraedge) <- 1
nodeIndicesOfEdge(extraedge) <- c(1, -1)
str(extraedge)
}

\keyword{classes}

\eof
\name{dg.FactorEdge-class}

\docType{class}

\alias{dg.FactorEdge-class}
\alias{nodeTypesOfEdge,dg.FactorEdge-method}

\alias{newFactorEdge}

\alias{newFactorEdge}
\alias{dg.FactorEdge}

\title{Class "dg.FactorEdge" }

\description{  The class for the bipartite graph edges between vertices and factors. }

% \section{Objects from the Class}{
% Objects can be created by calls of the form \code{new("dg.FactorEdge", ...)}.
% }

\usage{
newFactorEdge(node.indices, nodes = NULL, width = 2,
              color = "DarkOliveGreen", type = NULL,
              label = ifelse(is.null(nodes), "",
                             paste(Labels(nodes), collapse = "~")),
              dash = "")
}

\arguments{
  \item{node.indices}{ Vector with \code{abs(node.indices)}
                       the indices of the nodes of the factor edge.
                       If the index is positiv then the node is a vertex,
                       else it is the factor vertex. }
  \item{nodes}{ List with the \code{nodes} of the factor edge. Used for
      setting the default label. }
  \item{width}{ Numeric for the \code{width} of the factor edge. }
  \item{color}{ Text string with the \code{color} of the factor edge. }
  \item{type}{ Text string with the \code{type} of the factor edge. }
  \item{label}{ Text string with the \code{label} of the factor edge. }
  \item{dash}{ Text string for the \code{dash} pattern of the edge. }
}

\section{Slots}{
  \describe{
    \item{\code{vertex.indices}:}{Object of class \code{"numeric"},
          see \code{"dg.Edge"}. }
    \item{\code{width}:}{Object of class \code{"numeric"},
          see \code{"dg.Edge"}. }
    \item{\code{dash}:}{Object of class \code{"character"},
          see \code{"dg.Edge"}. }
    \item{\code{color}:}{Object of class \code{"character"},
          see \code{"dg.Edge"}. }
    \item{\code{label}:}{Object of class \code{"character"},
          see \code{"dg.Edge"}. }
    \item{\code{label.position}:}{Object of class \code{"numeric"},
          see \code{"dg.Edge"}. }
  }
}

\section{Extends}{
Class \code{"dg.Edge"}, directly.
Class \code{"dg.Node"}, by class \code{"dg.Edge"}.
}

\section{Methods}{
  \describe{
    \item{nodeTypesOfEdge}{\code{signature(object = "dg.FactorEdge")}:
      Extract the types ("super classes": \code{"Vertex"} or \code{"Factor"})
      of the vertices (nodes) of the edge. }
    % \item{propertyDialog}{\code{signature(object = "dg.FactorEdge")}: ... }
    % \item{addToPopups}{\code{signature(object = "dg.FactorEdge")}: ... }
  }
}

\author{Jens Henrik Badsberg}

\seealso{
% \code{\link{newFactorEdge}},
\code{\link{returnFactorEdgeList}},
\code{\link{dg.Edge-class}}, and
\code{\link{dg.FactorVertex-class}}.
}

% \seealso{\code{\link{newVertexEdge}}}

\note{ 
   The methods (except \code{\link{oriented}})
   of \code{\link{dg.Edge}}
   also applies for \code{dg.FactorEdge}.
 }

\examples{
vertices <- returnVertexList(paste("V", 1:4, sep = ""))
factor <- newFactor(1:3, new("dg.VertexList", vertices[1:3]),
                    type = "Generator")
factoredge <- newFactorEdge(c(1, -1), 
                            new("dg.VertexList", list(vertices[[1]], factor)))
str(factoredge)

color(factoredge)
label(factoredge)
labelPosition(factoredge)
width(factoredge)
nodeIndicesOfEdge(factoredge)
nodeTypesOfEdge(factoredge)

color(factoredge) <- "Black"
label(factoredge) <- "V1~V1:2:3"
labelPosition(factoredge) <- c(0, 1, 2)
width(factoredge) <- 1
nodeIndicesOfEdge(factoredge) <- c(1, -1)
str(factoredge)
}

\keyword{classes}

\eof
\name{dg.FactorVertex-class}

\docType{class}

\alias{dg.FactorVertex-class}
\alias{nodeIndices<-,dg.FactorVertex-method}
\alias{nodeIndices,dg.FactorVertex-method}
\alias{index<-,dg.FactorVertex-method}
\alias{index,dg.FactorVertex-method}

\alias{newFactor}

\alias{dg.FactorVertex}
\alias{newFactor}
\alias{nodeIndices}
\alias{nodeIndices<-}  

\title{Class "dg.FactorVertex" }

\description{ A skeleton class for the classes of factor vertices. }

% \section{Objects from the Class}{
% Objects can be created by calls of 
% the form \code{new("dg.FactorVertex", ...)}.
% }

\usage{
newFactor(vertex.indices, vertices = NULL, 
          name = ifelse(is.null(vertices), "", 
                        paste(Labels(vertices), collapse = ":")),
          label = name, type = "Generator",
          index = 0, width = 2, color = "default",
          factorClasses = validFactorClasses())
}

\arguments{
  \item{vertex.indices}{ Integer vector of the \code{vertex.indices} 
                         of the vertices of the factor. }
  \item{vertices}{ The \code{vertices} of the factor. Used for setting
      the default label. }
  \item{name}{ Text string for the \code{name} of the factor vertex. }
  \item{label}{ Text string for the \code{label} of the factor vertex. }
  \item{type}{ Text string for the \code{type}, e.i. the class,
      of the factor vertex. Should be in \code{factorClasses}. }
  \item{index}{ Integer for the \code{index} of the factor vertex. }
  \item{width}{ Numeric, for the \code{width} of the factor vertex. }
  \item{color}{ Text string for the \code{color} of the factor vertex. }
  \item{factorClasses}{ The valid \code{factorClasses}. }
}

\section{Slots}{
  \describe{
%    \item{\code{test.position}:}{Object of class \code{"numeric"}:
%       The \code{position} of a test statistic for the factor. }
    \item{\code{vertex.indices}:}{Object of class \code{"numeric"}:
       The \code{vertex.indices} of the vertices of the factor. }
    \item{\code{name}:}{Object of class \code{"character"},
       see \code{"dg.Vertex"}. }
    \item{\code{index}:}{Object of class \code{"numeric"},
       see \code{"dg.Vertex"}. }
    \item{\code{position}:}{Object of class \code{"numeric"},
       see \code{"dg.Vertex"}. }
    \item{\code{blockindex}:}{Object of class \code{"numeric"},
       see \code{"dg.Vertex"}. }
    \item{\code{stratum}:}{Object of class \code{"numeric"},
       see \code{"dg.Vertex"}. }
    \item{\code{color}:}{Object of class \code{"character"},
       see \code{"dg.Vertex"}. }
    \item{\code{label}:}{Object of class \code{"character"},
       see \code{"dg.Vertex"}. }
    \item{\code{label.position}:}{Object of class \code{"numeric"},
       see \code{"dg.Vertex"}. }
  }
}

\section{Extends}{
Class \code{"dg.Vertex"}, directly.
Class \code{"dg.Node"}, by class \code{"dg.Vertex"}.
}

\section{Methods}{
  \describe{
    \item{nodeIndices<-}{\code{signature(x = "dg.FactorVertex")}: ... }
    \item{nodeIndices}{\code{signature(object = "dg.FactorVertex")}: ... }
    \item{index<-}{\code{signature(x = "dg.FactorVertex")}: ... }
    \item{index}{\code{signature(object = "dg.FactorVertex")}: ... }
    % \item{propertyDialog}{\code{signature(object = "dg.FactorVertex")}: ... }
    % \item{addToPopups}{\code{signature(object = "dg.FactorVertex")}: ... }
  }
}

\author{Jens Henrik Badsberg}

\seealso{
% \code{\link{newFactor}},
\code{\link{returnFactorVerticesAndEdges}},
\code{\link{dg.FactorEdge-class}},
\code{\link{dg.Vertex-class}},
\code{\link{dg.Node-class}}, and
\code{\link{validFactorClasses}}.
}

\note{ 
   The methods (except \code{\link{stratum}})
   of \code{\link{dg.Vertex}}
   also applies for \code{dg.FactorVertex}.
 }

% \seealso{\code{\link{newVertex}}, \code{\link{validVertexClasses}}}

\examples{
vertices <- returnVertexList(paste("V", 1:4, sep = ""), 
                             types = rep("Discrete", 4))
factor <- newFactor(c(1, 2, 3), new("dg.VertexList", vertices[1:3]),
                    type = "Generator")

str(factor)

color(factor)
label(factor)
labelPosition(factor)
name(factor)
index(factor)
position(factor)
nodeIndices(factor)

color(factor) <- "green"
label(factor) <- "v-1-2-3"
labelPosition(factor) <- c(1, 2, 3)
name(factor) <- "V-123" 
index(factor) <- 3
position(factor) <- c( 10,  20,  30,  40)

str(factor)
}

\keyword{classes}

\eof
\name{dg.Generator-class}

\docType{class}

\alias{dg.Generator-class}
\alias{draw,dg.Generator-method}
\alias{dg.DiscreteGenerator-class}
\alias{draw,dg.DiscreteGenerator-method}
\alias{dg.LinearGenerator-class}
\alias{draw,dg.LinearGenerator-method}
\alias{dg.QuadraticGenerator-class}
\alias{draw,dg.QuadraticGenerator-method}

\title{Classses "dg.Generator", "dg.DiscreteGenerator", "dg.Linear"\-"Generator",
and "dg.QuadraticGenerator" }

\description{ The class for factor vertices for general terms,
 discrete terms, linear terms, and quadratic terms. 

 The class adds the \code{draw} method to the 
 \code{\link{dg.FactorVertex-class}}.

 The objects of the classes are usually created by the function
 \code{\link{returnFactorVerticesAndEdges}}.
}

% \section{Objects from the Class}{
% Objects can be created by calls of the form
% \code{new("dg.Generator", ...)},
% \code{new("dg.DiscreteGenerator", ...)},
% \code{new("dg.LinearGenerator", ...)}, or
% \code{new("dg.QuadraticGenerator", ...)}.
% }

\section{Slots}{
  \describe{
    % \item{\code{test.position}:}{Object of class \code{"numeric"},
    %      see \code{"dg.FactorVertex"}. }
    \item{\code{vertex.indices}:}{Object of class \code{"numeric"},
          see \code{"dg.FactorVertex"}. }
    \item{\code{name}:}{Object of class \code{"character"},
          see \code{"dg.Vertex"}. }
    \item{\code{index}:}{Object of class \code{"numeric"},
          see \code{"dg.Vertex"}. }
    \item{\code{position}:}{Object of class \code{"numeric"},
          see \code{"dg.Vertex"}. }
    \item{\code{blockindex}:}{Object of class \code{"numeric"},
          see \code{"dg.Vertex"}. }
    \item{\code{stratum}:}{Object of class \code{"numeric"},
          see \code{"dg.Vertex"}. }
    \item{\code{color}:}{Object of class \code{"character"},
          see \code{"dg.Vertex"}.
       Default is \code{yellow} for general, \code{cyan} for discrete,
       \code{magenta} for linear, and \code{blue} for quadratic terms. }
    \item{\code{label}:}{Object of class \code{"character"},
          see \code{"dg.Vertex"}. }
    \item{\code{label.position}:}{Object of class \code{"numeric"},
          see \code{"dg.Vertex"}. }
  }
}

\section{Extends}{
Class \code{"dg.FactorVertex"}, directly.
Class \code{"dg.Vertex"}, by class \code{"dg.FactorVertex"}.
Class \code{"dg.Node"}, by class \code{"dg.FactorVertex"}.
}

\section{Methods}{
  \describe{
    \item{draw}{\code{signature(object = "dg.Generator")}: ... }
    \item{draw}{\code{signature(object = "dg.DiscreteGenerator")}: ... }
    \item{draw}{\code{signature(object = "dg.LinearGenerator")}: ... }
    \item{draw}{\code{signature(object = "dg.QuadraticGenerator")}: ... }
  }
}

\author{Jens Henrik Badsberg}

\seealso{
\code{\link{newFactor}},
\code{\link{returnFactorVerticesAndEdges}},
\code{\link{dg.FactorVertex-class}},
\code{\link{dg.Vertex-class}},
\code{\link{dg.Node-class}}.
}

\keyword{classes}

\eof
\name{dg.Model-class}
\docType{class}

\alias{dg.Model-class}
\alias{modifyModel,dg.Model-method}
\alias{testEdge,dg.Model-method}
\alias{setGraphComponents,dg.Model-method}
\alias{returnGraphComponents,dg.Model-method}
\alias{graphComponents,dg.Model-method}
\alias{vertexEdges,dg.Model-method}
\alias{vertexEdges<-,dg.Model-method}
\alias{Str,dg.Model-method}
\alias{setGraphComponents}
\alias{returnGraphComponents}
\alias{graphComponents}
\alias{vertexEdges}
\alias{vertexEdges<-}

\alias{newDefaultModelObject}

\alias{newDefaultModelObject}
\alias{modifyModel}
\alias{testEdge}

\title{Class "dg.Model" }

\description{  An example class for the model object of dynamicGraph. }

\section{Objects from the Class}{

This is an example of the object for interface between
\code{\link{dynamicGraphMain}} and your models.

The model object of the call of \code{\link{dynamicGraphMain}}
should have the methods \code{modifyModel}, \code{testEdge},
\code{graphComponents} and \code{setGraphComponents}.

When the graph is modified, by adding or dropping vertices or edge,
the method \code{modifyModel} is called on the argument \code{object}
of \code{\link{dynamicGraphMain}}. 

If a value different from NULL is returned from \code{modifyModel} at the 
key event "add edge" then the edge is added to the the view of the model,
the graph window.

If NULL is returned from \code{modifyModel} then \code{\link{drawModel}}
can be used to draw the new graph.
If a value different from NULL is returned from \code{modifyModel} then 
\code{\link{redrawView}} and \code{\link{drawModel}} should not be called
from \code{modifyModel} for the action add edge.

If the edge is to be added in a slave window then the edges to
draw can be returned in a component with name \code{edgeList} (or
\code{newEdges$vertexEdges}) of the returned structure.

If an \code{object} is returned in the list of the returned
value from \code{modifyModel} then \code{object} in 
\code{\link{dynamicGraphMain}} is replaced by this object
(and the object is also assigned in the top level environment,
if \code{objectName} was given to \code{\link{dynamicGraphMain}}).

If a factor edge is added then the component \code{edgeList} with the
edges of the new model should be available in the returned structure.
New factor edges and factor vertices should also be avaliable in the
components \code{FactorEdges} and \code{FactorVertices}.

Similar for the key events "dropEdge", "addVertex" and "dropVertex".

The methods \code{graphComponents} and \code{setGraphComponents}
are used to communicate the graph components between several views
of the same model in \code{dynamicGraphMain}.
The method \code{graphComponents} of the model object is for returning
the objects to draw in the view, depending on the \code{viewType}.

If \code{NULL} is returned for a component in the returned list
from \code{graphComponents} then the corresponding value of
\code{Arguments} is used in \code{\link{redrawView}}.
To force an empty list, return the value \code{list()}
(or \code{numeric(0)} for \code{VisibleVertices}
and \code{VisibleBlocks}). See also the argument
\code{factorVertexList} of \code{\link{drawModel}}.

The method \code{setGraphComponents} of the model object
is called on the model object when the model is modified.

The methods \code{testEdge} of \code{object} should return
an object with the methods \code{label} and \code{width}
for labeling edges, see \code{\link{newDefaultTestObject}}.

% Objects can be created by calls of the form \code{new("dg.Model", ...)}.
}

\usage{newDefaultModelObject(name)}

\arguments{\item{name}{ Text string with the \code{name} of the model object.}}

% \details{ 

% This is an example of the object for interface between
% \code{\link{dynamicGraphMain}} and your models.

% The model object of the call of \code{\link{dynamicGraphMain}}
% should have the methods \code{modifyModel}, \code{testEdge}
% \code{graphComponents}, and \code{setGraphComponents}.

% When the graph is modified, by adding or dropping vertices or edge,
% the method \code{modifyModel} is called on the argument \code{object}
% of \code{\link{dynamicGraphMain}}. 

% If an \code{object} is returned in the list of the returned
% value from \code{modifyModel} then \code{object} in
% \code{\link{dynamicGraphMain}} is replaced by this object,
% and the object is also assigned in the top level environment,
% if \code{objectName} was given to \code{\link{dynamicGraphMain}}.

% The methods \code{graphComponents} and \code{setGraphComponents}
% are used to communicate the graph components between several views
% of the same model.
% The method \code{graphComponents} of the model object is for returning
% the objects to draw in the view, depending on the \code{viewType}.
% The method\code{setGraphComponents} of the model object
% is called on the model object when the model is modified.

% The methods \code{testEdge} of \code{object} should return
% an object with the methods \code{label} and \code{width}
% for labeling edges, see \code{\link{newDefaultTestObject}}.

% }

\value{An object of class \code{dg.Model}.}

\section{Slots}{
  \describe{
    \item{\code{name}:}{Object of class \code{"character"}:
      The \code{name} of the model. }
  }
}

\section{Methods}{
  \describe{
    % \item{vertexEdges<-}{\code{signature(x = "dg.Model")}: ... }
    % \item{vertexEdges}{\code{signature(object = "dg.Model")}: ... }
    \item{modifyModel}{\code{signature(object = "dg.Model")}: ... }
    % \item{returnGraphComponents}{\code{signature(object = "dg.Model")}: ... }
    \item{graphComponents}{\code{signature(object = "dg.Model")}: ... }
    \item{setGraphComponents}{\code{signature(object = "dg.Model")}: ... }
    \item{testEdge}{\code{signature(object = "dg.Model")}: ... }
  }
}

\author{Jens Henrik Badsberg}

% \references{ 
%     CoCo, with a guide at \url{http://www.jstatsoft.org/v06/i04/},
%     avaliable form \url{http://www.math.auc.dk/gr/material/CoCo/} and
%    \url{http://www.jbs.agrsci.dk/Biometry/Software-Datasets/CoCo/CoCo.1.6/}
%    has an interface to \code{\link{dynamicGraph}}. }

\references{ 
    CoCo, with a guide at \url{http://www.jstatsoft.org/v06/i04/},
    has an interface to \code{\link{dynamicGraph}}. }

\seealso{\code{\link{dg.Test-class}}.
% \code{\link{newDefaultModelObject}}, and \code{\link{newDefaultTestObject}}.
}

\examples{
# Part of the example "defaultObjects" of demo:


# Edit the following to meet your needs:
#
# - Change the name "your.Model"
#
# - Work out how the get names, types and edges from the model object.
#
# - At "message", insert the relevant code for testing and modifying the model.
#
# - The slots visibleVertices, visibleBlocks, extraVertices, vertexEdges, 
#   blockEdges, factorVertices, factorEdges, and, extraEdges should be
#   eliminated, and you should in "graphComponents" return relevant lists.
#



setClass("your.Model", 
         representation(name             = "character",
                        visibleVertices  = "numeric",
                        visibleBlocks    = "numeric",
                        extraVertices    = "dg.VertexList",
                        vertexEdges      = "dg.VertexEdgeList",
                        blockEdges       = "dg.BlockEdgeList",
                        factorVertices   = "dg.FactorVertexList",
                        factorEdges      = "dg.FactorEdgeList",
                        extraEdges       = "dg.ExtraEdgeList"))

"newYourModelObject"<- function(name) {
    result <- new("your.Model", name = name,
                  extraVertices  = .emptyDgList("dg.VertexList"),
                  vertexEdges    = .emptyDgList("dg.VertexEdgeList"),
                  blockEdges     = .emptyDgList("dg.BlockEdgeList"),
                  factorVertices = .emptyDgList("dg.FactorVertexList"),
                  factorEdges    = .emptyDgList("dg.FactorEdgeList"),
                  extraEdges     = .emptyDgList("dg.ExtraEdgeList"))
    return(result)
  }

if (!isGeneric("graphComponents")) {
  if (is.function("graphComponents"))
    fun <- graphComponents
  else
    fun <- function(object, viewType = NULL, ...)
    standardGeneric("graphComponents")
  setGeneric("graphComponents", fun)
}

setMethod("graphComponents", "your.Model",
          function(object, viewType = NULL, ...)
          { # print(viewType); print ("graphComponents")
            args <- list(...)
            Args <- args$Arguments
            Edges <- object@vertexEdges
            Vertices <- Args$vertexList
            VisibleVertices <- object@visibleVertices
            if (viewType == "Factor") {
              factors <- .cliquesFromEdges(Edges, Vertices, VisibleVertices)
              if (is.null(factors) || (length(factors) == 0)) {
                FactorVertices  <- .emptyDgList("dg.FactorVertexList")
                FactorEdges     <- .emptyDgList("dg.FactorEdgeList")
              } else {
                result <- returnFactorVerticesAndEdges(Vertices, factors)
                FactorVertices  <- result$FactorVertices
                FactorEdges     <- result$FactorEdges
              }
              list(vertexEdges     = object@vertexEdges, 
                   blockEdges      = object@blockEdges, 
                   factorVertices  = FactorVertices,
                   factorEdges     = FactorEdges,
                   visibleVertices = object@visibleVertices, 
                   visibleBlocks   = object@visibleBlocks, 
                   extraVertices   = object@extraVertices, 
                   extraEdges      = object@extraEdges)
            } else if (viewType == "Moral") {
              message("Moral view not implemented; ")
              list(vertexEdges     = object@vertexEdges, 
                   blockEdges      = .emptyDgList("dg.BlockEdgeList"),
                   factorVertices  = .emptyDgList("dg.FactorVertexList"),
                   factorEdges     = .emptyDgList("dg.FactorEdgeList"),
                   visibleVertices = object@visibleVertices, 
                   visibleBlocks   = numeric(), 
                   extraVertices   = object@extraVertices, 
                   extraEdges      = object@extraEdges)
            } else if (viewType == "Essential") {
              message("Essential view not implemented; ")
              list(vertexEdges      = object@vertexEdges, 
                   blockEdges      = .emptyDgList("dg.BlockEdgeList"),
                   factorVertices  = .emptyDgList("dg.FactorVertexList"),
                   factorEdges     = .emptyDgList("dg.FactorEdgeList"),
                   visibleVertices = object@visibleVertices, 
                   visibleBlocks   = numeric(), 
                   extraVertices   = object@extraVertices, 
                   extraEdges      = object@extraEdges)
            } else if (viewType == "Simple") {
              list(vertexEdges     = object@vertexEdges, 
                   blockEdges      = object@blockEdges, 
                   factorVertices  = .emptyDgList("dg.FactorVertexList"),
                   factorEdges     = .emptyDgList("dg.FactorEdgeList"),
                   visibleVertices = object@visibleVertices, 
                   visibleBlocks   = object@visibleBlocks, 
                   extraVertices   = object@extraVertices, 
                   extraEdges      = object@extraEdges)
            } else 
              message("View type not implemented; ")
          })

if (!isGeneric("setGraphComponents")) {
  if (is.function("setGraphComponents"))
    fun <- setGraphComponents
  else
    fun <- function(object, viewType = NULL,
                    visibleVertices = NULL,
                    extraVertices   = NULL,
                    vertexEdges     = NULL,
                    blockEdges      = NULL,
                    factorVertices  = NULL,
                    factorEdges     = NULL,
                    extraEdges      = NULL, ...)
      standardGeneric("setGraphComponents")
  setGeneric("setGraphComponents", fun)
}

setMethod("setGraphComponents", signature(object = "your.Model"),
          function(object, viewType = NULL,
                   visibleVertices = NULL,
                   visibleBlocks   = NULL,
                   extraVertices   = NULL,
                   vertexEdges     = NULL,
                   blockEdges      = NULL,
                   factorVertices  = NULL,
                   factorEdges     = NULL,
                   extraEdges      = NULL, ...)
 {
    if (!is.null(visibleVertices)) object@visibleVertices <- visibleVertices
    if (!(viewType == "Moral"))
      if (!is.null(visibleBlocks  )) object@visibleBlocks   <- visibleBlocks
    if (!is.null(extraVertices  )) object@extraVertices   <- extraVertices
    if (!is.null(vertexEdges    )) object@vertexEdges     <- vertexEdges
    if (!is.null(blockEdges     )) object@blockEdges      <- blockEdges
    if ((viewType == "Factor")) {
      if (!is.null(factorVertices )) object@factorVertices  <- factorVertices
      if (!is.null(factorEdges    )) object@factorEdges     <- factorEdges
    }
    return(object)
 })


if (!isGeneric("dynamic.Graph")) {
  if (is.function("dynamic.Graph"))
    fun <- dynamic.Graph
  else
    fun <- function(object, ...)
  standardGeneric("dynamic.Graph")
  setGeneric("dynamic.Graph", fun)
}

setMethod("dynamic.Graph", signature(object = "your.Model"),
          function(object, ...)
  {

    Names <- Your.function.for.extracting.variable.names.from.object(
             object = object)
    Types <- Your.function.for.extracting.variable.types.from.object(
             object = object)
    Edges <- Your.function.for.extracting.variable.edges.from.object(
             object = object)

    DynamicGraph(names = Names, types = Types, 
                 from = Edges[,1], to = Edges[,2], 
                 object = object, ...)
 })


if (!isGeneric("testEdge")) {
  if (is.function("testEdge"))
    fun <- testEdge
  else
    fun <- function(object, action, name.1, name.2, ...) 
           standardGeneric("testEdge")
  setGeneric("testEdge", fun)
}

setMethod("testEdge", signature(object = "your.Model"),
          function(object, action, name.1, name.2, ...)
 {
    args <- list(...)
    from.type <- args$from.type
    to.type <- args$to.type
    f <- function(type) if(is.null(type)) "" else paste("(", type, ")")
    message(paste("Should return an object with the edge from",
                  name.1, f(from.type), "to", name.2, f(to.type),
                  "deleted from the argument object"))
    return(newYourTestObject())
 })


if (!isGeneric("modifyModel")) {
  if (is.function("modifyModel"))
    fun <- modifyModel
  else
    fun <- function(object, action, name, name.1, name.2, ...)
                    standardGeneric("modifyModel")
  setGeneric("modifyModel", fun)
}

setMethod("modifyModel", signature(object = "your.Model"),
          function(object, action, name, name.1, name.2, ...)
 {
    args <- list(...)
    Args <- args$Arguments
    Edges <- args$newEdges$vertexEdges
    Vertices <- Args$vertexList

    DoFactors <- FALSE
    if (!is.null(args$Arguments)
        && !is.null(args$Arguments$factorVertexList)
        && (length(args$Arguments$factorVertexList) > 0)
        && !is.null(args$Arguments$vertexList))
      DoFactors <- TRUE

    FactorVertices  <- NULL
    FactorEdges     <- NULL
    BlockEdges      <- NULL
    VisibleVertices <- Args$visibleVertices
    VisibleBlocks   <- Args$visibleBlocks
    ExtraVertices   <- NULL
    ExtraEdges      <- NULL

    f <- function(type) if (is.null(type)) "" else paste("(", type, ")")
    g <- function(type) if (is.null(type)) "" else type
    if (action == "dropEdge") {
      message(paste("Should return an object with the edge from",
                    name.1, f(args$from.type), "to", name.2, f(args$to.type),
                    "deleted from the argument object"))
      if ((g(args$from.type) == "Factor") || (g(args$from.type) == "Factor"))
        return(NULL)
    } else if (action == "addEdge") {
       message(paste("Should return an object with the edge from",
                     name.1, f(args$from.type), "to", name.2, f(args$to.type),
                     "added to the argument object"))
      if ((g(args$from.type) == "Factor") || (g(args$from.type) == "Factor"))
        return(NULL)
    } else if (action == "dropVertex")  {
       message(paste("Should return an object with the vertex", 
                     name, f(args$type),
                     "deleted from the argument object"))
      if ((g(args$type) == "Factor"))
        return(NULL)
      VisibleVertices <- VisibleVertices[VisibleVertices != args$index]
      if (DoFactors && (args$index > 0)) {
        x <- (args$Arguments$factorVertexList)
        factors <- lapply(x, function(i) i@vertex.indices)
        types   <- lapply(x, function(i) class(i))
        factors <- lapply(factors, 
                          function(x) { 
                            y <- x[x != args$index]
                            if (length(y) > 0) return(y) else return(NULL) } )


        if (!is.null(factors)) {
          types   <- types[unlist(lapply(factors, function(i) !is.null(i)))]
          factors <- .removeNull(factors)
        }
        if (!is.null(factors)) {
          subset <- function(x)
            lapply(x, function(a) 
                        any(unlist(lapply(x, 
                                          function(A) 
                                            all(!is.na(match(a, A))) &&
                                            (length(a) < length(A))))))
          s <- subset(factors)
          types   <- types[!unlist(s)]
          factors <- factors[!unlist(s)]
          if (!(is.null(factors))) {
            result <- returnFactorVerticesAndEdges(
                            args$Arguments$vertexList, factors, types, 
                            factorClasses = validFactorClasses())
            FactorVertices <- result$FactorVertices
            FactorEdges <- result$FactorEdges
          }
        } else { 
          DoFactors <- FALSE
          FactorVertices <- .emptyDgList("dg.FactorVertexList")
          FactorEdges    <- .emptyDgList("dg.FactorEdgeList")
        }
      }
    } else if (action == "addVertex") {
      VisibleVertices <- c(VisibleVertices, args$index)
      message(paste("Should return an object with the vertex", 
                    name, f(args$type), args$index, 
                    "added to the argument object"))
      if (DoFactors && (args$index > 0)) {
        x <- (args$Arguments$factorVertexList)
        factors <- lapply(x, function(i) i@vertex.indices)
        types   <- lapply(x, function(i) class(i))
        if (!is.null(factors))
          factors <- .removeNull(factors)
        if (is.null(factors)) {
          factors <- list(args$index)
          types   <- validFactorClasses()[1, 1]
        } else { 
          n <- length(types)
          factors <- append(factors, list(args$index))
          types   <- append(types, types[n])
        }
        if (!(is.null(factors))) {
          result <- returnFactorVerticesAndEdges(
                          args$Arguments$vertexList, factors, types, 
                          factorClasses = validFactorClasses())
          FactorVertices <- result$FactorVertices
          FactorEdges <- result$FactorEdges } }
    }
    if (is.null(FactorVertices) && DoFactors && !is.null(Edges)) {

      factors <- .cliquesFromEdges(Edges, Vertices, VisibleVertices)

      if (is.null(factors) || (length(factors) == 0)) {
        FactorVertices <- .emptyDgList("dg.FactorVertexList")
        FactorEdges    <- .emptyDgList("dg.FactorEdgeList")
      } else {
        result <- returnFactorVerticesAndEdges(Vertices, factors)
        FactorVertices  <- result$FactorVertices
        FactorEdges     <- result$FactorEdges } }
    return(list(object          = object,
                BlockEdges      = BlockEdges, 
                FactorVertices  = FactorVertices,
                FactorEdges     = FactorEdges,
                VisibleVertices = VisibleVertices, 
                VisibleBlocks   = VisibleBlocks, 
                ExtraVertices   = ExtraVertices,
                ExtraEdges      = ExtraEdges))
 })

setMethod("Str", "your.Model",
          function(object, setRowLabels = FALSE, title = "", ...) {
              message(object@name) })

newYourModelObject("ModelObject")
}

\keyword{classes}

\eof
\name{dg.Node-class}

\docType{class}

\alias{dg.Node-class}
\alias{addToPopups,dg.Node-method}
\alias{color<-,dg.Node-method}
\alias{color,dg.Node-method}
\alias{label<-,dg.Node-method}
\alias{label,dg.Node-method}
\alias{labelPosition<-,dg.Node-method}
\alias{labelPosition,dg.Node-method}
\alias{propertyDialog,dg.Node-method}

\title{Class "dg.Node" }

\description{  A skeleton class for the classes of vertices,
 edges and block objects. }

\section{Objects from the Class}{
The objects has the slots and methods relevant to vertices, edges and blocks.

% Objects can be created by calls of the form \code{new("dg.Node", ...)}.
}

\section{Slots}{
  \describe{
    \item{\code{color}:}{Object of class \code{"character"}
        with the \code{color} of the object. }
    \item{\code{label}:}{Object of class \code{"character"}
        with the \code{label} of the object. }
    \item{\code{label.position}:}{Object of class \code{"numeric"}
        with the \code{label.positions} of the object. }
  }
}

\section{Methods}{
  \describe{
    \item{addToPopups}{\code{signature(object = "dg.Node")}: 
        Add items to the pop up menu (\code{nodePop}\-\code{upMenu})
        of the object by \code{tkadd}.
    }
    \item{color<-}{\code{signature(x = "dg.Node")}:
        Set the \code{color} of the object. }
    \item{color}{\code{signature(object = "dg.Node")}:
        Return the \code{color} of the object. }
    \item{label<-}{\code{signature(x = "dg.Node")}:
        Set the \code{label} of the object. }
    \item{label}{\code{signature(object = "dg.Node")}:
        Return the \code{label} of the object. }
    \item{labelPosition<-}{\code{signature(x = "dg.Node")}:
        Set the \code{label.position} of the object. }
    \item{labelPosition}{\code{signature(object = "dg.Node")}:
        Return the \code{label.position} of the object. }
    \item{propertyDialog}{\code{signature(object = "dg.Node")}: 
        Open a Tk/tcl-window with the slots of the node. }
  }
}

\author{Jens Henrik Badsberg}

\seealso{
\code{\link{dg.Vertex-class}},
\code{\link{dg.Edge-class}},
\code{\link{dg.Block-class}}.
}

\keyword{classes}

\eof
\name{dg.Test-class}

\docType{class}

\alias{dg.Test-class}
\alias{label,dg.Test-method}
\alias{width,dg.Test-method}

\alias{newDefaultTestObject}

\title{Class "dg.Test" }

\description{  An example class for the test object 
	       for the model object of dynamicGraph. }

\section{Objects from the Class}{
The methods \code{label} and \code{width} should be 
implemented by you for your test object returned by the method
\code{testEdge} of your model \code{object}. 

% Objects can be created by calls of the form \code{new("dg.Test", ...)}.
}

\usage{newDefaultTestObject(name)}

\arguments{\item{name}{ Text string with the \code{name} of the test object.}}

\value{An object of class \code{dg.Test}.}

\section{Slots}{
  \describe{
    \item{\code{deviance}:}{Object of class \code{"numeric"}: 
        The \code{deviance} of the test. }
    \item{\code{df}:}{Object of class \code{"numeric"}: 
        The \code{df} of the test. }
    \item{\code{p}:}{Object of class \code{"numeric"}: 
        The \code{p}-value of the test. }
  }
}

\section{Methods}{
  \describe{
    \item{label}{\code{signature(object = "dg.Test")}: 
       Return the \code{label} of the test. }
    \item{width}{\code{signature(object = "dg.Test")}:
       Return the \code{width} of the test. }
  }
}

\author{Jens Henrik Badsberg}

\seealso{\code{\link{dg.Model-class}}.
% \code{\link{newDefaultModelObject}}, and \code{\link{newDefaultTestObject}}.
}

\examples{
# Part of the example "defaultObjects" of demo:

setClass("your.Test", 
         representation(deviance = "numeric", df = "numeric", p = "numeric"))

"newYourTestObject" <- function(name) {
    df <- round(runif(1, 1, 25))
    message("Just generating a random test!!!!!")
    deviance <- rchisq(1, df)
    p <- 1 - pchisq(deviance, df)
    result <- new("your.Test", df = df, deviance = deviance, p = p)
    return(result)
  }

if (!isGeneric("label") && !isGeneric("label", where = 2)) {
  if (is.function("label"))
    fun <- label
  else
    fun <- function(object) standardGeneric("label")
  setGeneric("label", fun)
}

setMethod("label", "your.Test",
          function(object) format(object@p, digits = 4))

if (!isGeneric("width") && !isGeneric("width", where = 2)) {
  if (is.function("width"))
    fun <- width
  else
    fun <- function(object) standardGeneric("width")
  setGeneric("width", fun)
}

setMethod("width", "your.Test",
          function(object) round(2 + 5 * (1 - object@p)))

newYourTestObject("TestObject")
}

\keyword{classes}

\eof
\name{dg.TextVertex-class}

\docType{class}

\alias{dg.TextVertex-class}
\alias{draw,dg.TextVertex-method}

\title{Class "dg.TextVertex"}

\description{ The class for vertices for setting text string
	      in the graph window.
              These vertices are given by the argument \code{extraList}
              to \code{\link{dynamicGraphMain}}. }

\section{Objects from the Class}{
The vertices of nodes of this class are drawn with very small symbols.
Objects has the methods for extracting and setting the slots for vertices.

% Objects can be created by calls of the form \code{new("dg.TextVertex", ...)}.
}

\section{Slots}{
  \describe{
    \item{\code{name}:}{Object of class \code{"character"} 
       with the \code{name} of the vertex. }
    \item{\code{index}:}{Object of class \code{"numeric"} 
       with the \code{index} of the vertex,
       the position of the vertex in the extra list. }
    \item{\code{position}:}{Object of class \code{"numeric"} 
       with the \code{position} of the vertex.
       Vertices in the same dynamic graph should
       have the same number of coordinates. 
       A small dot is placed at the position of the vertex. 
       This dot can be move outside the window. }
    \item{\code{blockindex}:}{Object of class \code{"numeric"}
       with the \code{blockindex} of the vertex. }
    \item{\code{stratum}:}{Object of class \code{"numeric"} 
       with the \code{stratum} of the vertex. }
    \item{\code{color}:}{Object of class \code{"character"}
       with the \code{color} of the vertex. }
    \item{\code{label}:}{Object of class \code{"character"}
       with the \code{label} of the vertex. }
    \item{\code{label.position}:}{Object of class \code{"numeric"}
       with the \code{label.position} of the vertex.
       Labels of vertices in the same dynamic graph
       should have the same number of coordinates. }
  }
}

\section{Extends}{
Class \code{"dg.Vertex"}, directly.
Class \code{"dg.Node"}, by class \code{"dg.Vertex"}.
}
\section{Methods}{
  \describe{
    \item{draw}{\code{signature(object = "dg.TextVertex")}: ... }
  }
}

\author{Jens Henrik Badsberg}

\seealso{
\code{\link{newVertex}},
\code{\link{returnVertexList}},
\code{\link{dg.Node-class}}, and
\code{\link{dg.ExtraEdge-class}}.
}

\keyword{classes}

\eof
\name{dg.Vertex-class}

\docType{class}

\alias{dg.Vertex-class}
\alias{ancestors<-,dg.Vertex-method}
\alias{ancestors,dg.Vertex-method}
\alias{blockindex<-,dg.Vertex-method}
\alias{blockindex,dg.Vertex-method}
\alias{descendants<-,dg.Vertex-method}
\alias{descendants,dg.Vertex-method}
\alias{index<-,dg.Vertex-method}
\alias{index,dg.Vertex-method}
\alias{name<-,dg.Vertex-method}
\alias{name,dg.Vertex-method}
\alias{position<-,dg.Vertex-method}
\alias{position,dg.Vertex-method}
\alias{stratum<-,dg.Vertex-method}
\alias{stratum,dg.Vertex-method}
% \alias{vertexIndices<-,dg.Vertex-method}
% \alias{vertexIndices,dg.Vertex-method}
\alias{visible<-,dg.Vertex-method}
\alias{visible,dg.Vertex-method}

\alias{newVertex}

\alias{dg.Node}
\alias{dg.Vertex}
\alias{newVertex}
\alias{draw}
\alias{color}
\alias{color<-}
\alias{label}
\alias{label<-}
\alias{labelPosition}
\alias{labelPosition<-}
\alias{name}
\alias{name<-}
\alias{index}
\alias{index<-}  
\alias{position}
\alias{position<-}
\alias{stratum}
\alias{stratum<-}
\alias{blockindex}
\alias{blockindex<-}
\alias{visible}
\alias{visible<-}
\alias{addToPopups}
\alias{propertyDialog}

\title{Class "dg.Vertex" }

\description{  A skeleton class for the classes of vertices. }

\section{Objects from the Class}{
Objects has the methods for extracting and setting the slots for vertices.

% Objects can be created by calls of the form \code{new("dg.Vertex", ...)}.
}

\usage{
newVertex(name, label = name, type = "dg.Vertex", index = 1,
          position = c(0, 0, 0), blockindex = 0, stratum = 0,
          color = ifelse(type == "TextVertex", "GhostWhite", "SaddleBrown"),
          vertexClasses = validVertexClasses())
}

\arguments{
  \item{name}{ Text string with the \code{name} of the vertex. 
     Should be a valid name on variables for your model object. }
  \item{label}{ Text string with the \code{label} of the vertex. }
  \item{type}{ Text string with the \code{type} of the vertex. 
     This \code{type} is the label of class of the vertex,
     the labels of \code{\link{vertexClasses}}. }
  \item{index}{ Integer with the \code{index} of the vertex,
     the position in a vertex list. }
  \item{position}{ Numeric vector with the \code{position} of the vertex.
     Vertices in the same dynamic graph should
     have the same number of coordinates. }
  \item{blockindex}{ Integer with the \code{blockindex} of the vertex. }
  \item{stratum}{ Integer with the \code{stratum} of the vertex. }
  \item{color}{ Text string with the \code{color} of the vertex. }
  \item{vertexClasses}{ The valid \code{vertexClasses}. }
}

\section{Slots}{
  \describe{
    \item{\code{name}:}{Object of class \code{"character"} 
       with the \code{name} of the vertex. 
       Should be a valid name on variables for your model object.}
    \item{\code{index}:}{Object of class \code{"numeric"} 
       with the \code{index} of the vertex,
       the position of the vertex in a vertex list. }
    \item{\code{position}:}{Object of class \code{"numeric"} 
       with the \code{position} of the vertex.
       Vertices in the same dynamic graph should
       have the same number of coordinates. }
    \item{\code{blockindex}:}{Object of class \code{"numeric"}
       with the \code{blockindex} of the vertex. }
    \item{\code{stratum}:}{Object of class \code{"numeric"} 
       with the \code{stratum} of the vertex. }
    \item{\code{color}:}{Object of class \code{"character"}
       with the \code{color} of the vertex. }
    \item{\code{label}:}{Object of class \code{"character"}
       with the \code{label} of the vertex. }
    \item{\code{label.position}:}{Object of class \code{"numeric"}
       with the \code{label.position} of the vertex.
       Labels of vertices in the same dynamic graph
       should have the same number of coordinates. }
  }
}

\section{Extends}{
Class \code{"dg.Node"}, directly.
}

\section{Methods}{
  \describe{
    \item{ancestors<-}{\code{signature(x = "dg.Vertex")}: ... }
    \item{ancestors}{\code{signature(object = "dg.Vertex")}: ... }
    \item{blockindex<-}{\code{signature(x = "dg.Vertex")}: ... }
    \item{blockindex}{\code{signature(object = "dg.Vertex")}: ... }
    \item{descendants<-}{\code{signature(x = "dg.Vertex")}: ... }
    \item{descendants}{\code{signature(object = "dg.Vertex")}: ... }
    \item{index<-}{\code{signature(x = "dg.Vertex")}: ... }
    \item{index}{\code{signature(object = "dg.Vertex")}: ... }
    \item{name<-}{\code{signature(x = "dg.Vertex")}: ... }
    \item{name}{\code{signature(object = "dg.Vertex")}: ... }
    \item{position<-}{\code{signature(x = "dg.Vertex")}: ... }
    \item{position}{\code{signature(object = "dg.Vertex")}: ... }
    \item{stratum<-}{\code{signature(x = "dg.Vertex")}: ... }
    \item{stratum}{\code{signature(object = "dg.Vertex")}: ... }
%     \item{vertexIndices<-}{\code{signature(x = "dg.Vertex")}: ... }
%     \item{vertexIndices}{\code{signature(object = "dg.Vertex")}: ... }
    \item{visible<-}{\code{signature(x = "dg.Vertex")}: ... }
    \item{visible}{\code{signature(object = "dg.Vertex")}: ... }
    \item{propertyDialog}{\code{signature(object = "dg.Node")}: ... }
  }
}

\author{Jens Henrik Badsberg}

\seealso{
% \code{\link{newVertex}},
\code{\link{returnVertexList}},
\code{\link{dg.Node-class}}.
}

\note{ 
    The \code{dg.Vertex} class has the methods
    \code{\link{name}}, \code{\link{label}}, \code{\link{labelPosition}},
    \code{\link{position}}, \code{\link{stratum}},
    % \code{\link{visible}},
    \code{\link{color}}, and \code{\link{index}}
    for extracting values of the object and the replacement methods
    \code{\link{name<-}}, \code{\link{label<-}}, \code{\link{labelPosition<-}},
    \code{\link{position<-}}, \code{\link{stratum<-}}, 
    % \code{\link{visible<-}},
    \code{\link{color<-}}, and \code{\link{index<-}}.
    The method \code{\link{draw}} is used to draw a vertex,
    and items can be added to the pop up menu of the vertex
    by the method \code{\link{addToPopups}}.
    
    Some of these methods also applies for 
    edges (\code{\link{dg.Edge}}),
    blocks (\code{\link{dg.Block}}), 
%    edges between block and between blocks and vertices
    block edges
    (\code{\link{dg.BlockEdge}}),
    factor vertices (\code{\link{dg.FactorVertex}})
    and edges from vertices to factors (\code{\link{dg.FactorEdge}}).
}

\examples{

a <- newVertex("a", "A", "Discrete", 1, c(0, 0, 0))

str(a)

color(a)
label(a)
labelPosition(a)
name(a)
index(a)
position(a)
stratum(a)

color(a) <- "red"
label(a) <- "A vertex"
labelPosition(a) <- c(1, 2, 3)
name(a) <- "Capital.A"
index(a) <- -1
position(a) <- c(10, 20, 30)
stratum(a) <- 1

str(a)





}

\keyword{classes}

\eof
\name{dg.VertexEdge-class}


\docType{class}

\alias{dg.VertexEdge-class}
\alias{nodeTypesOfEdge,dg.VertexEdge-method}
\alias{oriented<-,dg.VertexEdge-method}
\alias{oriented,dg.VertexEdge-method}

\alias{dg.VertexEdge-class}
\alias{dg.DashedEdge-class}
\alias{dg.DottedEdge-class}
\alias{dg.DoubleArrowEdge-class}
\alias{dg.DoubleConnectedEdge-class}
\alias{dg.TripleConnectedEdge-class}

\alias{newVertexEdge}

\alias{dg.Edge}
\alias{dg.VertexEdge}
\alias{dg.DashedEdge}
\alias{dg.DottedEdge}
\alias{dg.DoubleArrowEdge}
\alias{dg.DoubleConnectedEdge}
\alias{dg.TripleConnectedEdge}
\alias{newVertexEdge}
\alias{nodeIndicesOfEdge}
\alias{nodeIndicesOfEdge<-}
\alias{nodeTypesOfEdge}
\alias{width}
\alias{width<-}
\alias{dash}
\alias{dash<-}
\alias{oriented}
\alias{oriented<-}

\title{Class "dg.VertexEdge" }

\description{  The class for edges between vertices. }

\section{Objects from the Class}{
Objects has the methods for extracting and setting the slots for edges,
and the method for drawing the edge.

% Objects can be created by calls of the form \code{new("dg.VertexEdge", ...)}.
}

\usage{
newVertexEdge(vertex.indices, vertices = NULL, width = 2,
              color = "DarkOliveGreen", oriented = FALSE, type = "VertexEdge",
              label = ifelse(is.null(vertices), "",
                             paste(Labels(vertices), collapse = "~")),
              dash = "", N = 3, edgeClasses = validEdgeClasses())
}

\arguments{
  \item{vertex.indices}{ Integer vector of the \code{vertex.indices} 
      of the vertices of the edge. These are the indices of the
      vertices in the list of vertices. }
  \item{vertices}{ All the \code{vertices} of the edge, and only the
     vertices of the edge. Currently, only used for setting the default label. }
  \item{width}{ Numeric, for the \code{width} of the edge. }
  \item{color}{ Text string for the \code{color} of the edge. }
  \item{oriented}{ Logical, TRUE if the edge is \code{oriented},
      also when no blocks are used in \code{\link{dynamicGraphMain}}. }
  \item{type}{ Text string for the \code{type} of the edge. 
      Currently not used. 
      Intended to set which class the edge should be class of
      containing the  class \code{dg.Edge}.}
  \item{label}{ Text string for the \code{label} of the edge.
                By default set by concatenating the labels of the vertices. }
  \item{dash}{ Text string for the \code{color} pattern of the edge. }
  \item{N}{ Integer, \code{N} is the number of coordinates of the vertices. }
  \item{edgeClasses}{ Returned value from \code{\link{validEdgeClasses}},
                      or extension of this matrix. }
}

\details{
    Edges are not constrained to have two vertices.
}

\section{Slots}{
  \describe{
    \item{\code{oriented}:}{Object of class \code{"logical"}.
       If TRUE, then the edge is \code{oriented},
       and thus drawn by an \code{arrow}, also when no blocks are pressent. }
    \item{\code{vertex.indices}:}{Object of class \code{"numeric"}, see \code{"dg.Edge"}. }
    \item{\code{width}:}{Object of class \code{"numeric"}, see \code{"dg.Edge"}. }
    \item{\code{dash}:}{Object of class \code{"character"}, see \code{"dg.Edge"}. }
    \item{\code{color}:}{Object of class \code{"character"}, see \code{"dg.Edge"}. }
    \item{\code{label}:}{Object of class \code{"character"}, see \code{"dg.Edge"}. }
    \item{\code{label.position}:}{Object of class \code{"numeric"}, see \code{"dg.Edge"}. }
  }
}

\section{Extends}{
Class \code{"dg.Edge"}, directly.
Class \code{"dg.Node"}, by class \code{"dg.Edge"}.
}

\section{Methods}{
  \describe{
    \item{nodeTypesOfEdge}{\code{signature(object = "dg.VertexEdge")}: ... }
    \item{oriented<-}{\code{signature(x = "dg.VertexEdge")}: ... }
    \item{oriented}{\code{signature(object = "dg.VertexEdge")}: ... }
    \item{propertyDialog}{\code{signature(object = "dg.VertexEdge")}: ... }
  }
}

\author{Jens Henrik Badsberg}

\seealso{
\code{\link{newVertexEdge}},
\code{\link{returnEdgeList}},
\code{\link{dg.Edge-class}}.
}

\note{
    The \code{dg.Edge} class has
    beside the methods of \code{\link{dg.Vertex}}
    the methods \code{\link{oriented}} and \code{\link{oriented<-}}.

    The method \code{\link{nodeIndicesOfEdge}} will extract the indices of the
    vertices of the edge, and the method \code{\link{nodeTypesOfEdge}} will
    extract the types
    ("super classes": \code{vertex}, \code{factor} or \code{block})
    of the vertices (nodes) of an edge.
    The method \code{\link{draw}} is used to draw the edge,
    and items are added to the pop up menu of an edge
    by the method \code{\link{addToPopups}}.

    Some of these methods also applies for
    block edges (\code{dg.BlockEdge})
    and factor edges (\code{dg.FactorEdge}).
}

\examples{
vertices <- returnVertexList(paste("V", 1:4, sep = ""))
e <- newVertexEdge(c(1, 2, 3), vertices = new("dg.VertexList", vertices[1:3]))

str(e)

color(e)
label(e)
labelPosition(e)
width(e)
oriented(e)
nodeIndicesOfEdge(e)
nodeTypesOfEdge(e)

color(e) <- "Black"
label(e) <- "1-2"
labelPosition(e) <- c(10, 20, 30)
width(e) <- 1
oriented(e) <- TRUE
nodeIndicesOfEdge(e) <- c(1, 2)

str(e)





}

\keyword{classes}

\eof
\name{dg.list-class}

\docType{class}

\alias{dg.list-class}
\alias{dg.NodeList-class}
\alias{dg.EdgeList-class}

\alias{Names}
\alias{Names<-}
\alias{Colors}
\alias{Colors<-}
\alias{Labels}
\alias{Labels<-}
\alias{LabelPositions}
\alias{LabelPositions<-}
\alias{Positions}
\alias{Positions<-}
\alias{Strata}
\alias{Strata<-}
\alias{Blockindices}
\alias{Blockindices<-}
\alias{Indices}
\alias{asDataFrame}
% \alias{Indices<-}
\alias{Visible}
\alias{Visible<-}

\alias{NodeTypes}
% \alias{NodeTypes<-}
\alias{NodeIndices}
% \alias{NodeIndices<-}
\alias{Widths}
\alias{Widths<-}
\alias{Dashes}
\alias{Dashes<-}
\alias{Oriented}
\alias{Oriented<-}

\alias{asDataFrame,dg.list-method}
\alias{Blockindices,dg.list-method}
\alias{Closed<-,dg.list-method}
\alias{Closed,dg.list-method}
\alias{Colors<-,dg.list-method}
\alias{Colors,dg.list-method}
\alias{Dashes<-,dg.list-method}
\alias{Dashes,dg.list-method}
\alias{Indices,dg.list-method}
\alias{LabelPositions<-,dg.list-method}
\alias{LabelPositions,dg.list-method}
\alias{Labels<-,dg.list-method}
\alias{Labels,dg.list-method}
\alias{Names<-,dg.list-method}
\alias{Names,dg.list-method}
\alias{NodeAncestors<-,dg.list-method}
\alias{NodeAncestors,dg.list-method}
\alias{NodeDescendants<-,dg.list-method}
\alias{NodeDescendants,dg.list-method}
\alias{NodeIndices,dg.list-method}
\alias{NodeTypes,dg.list-method}
\alias{Oriented<-,dg.list-method}
\alias{Oriented,dg.list-method}
\alias{Positions<-,dg.list-method}
\alias{Positions,dg.list-method}
\alias{show,dg.list-method}
\alias{Strata<-,dg.list-method}
\alias{Strata,dg.list-method}
\alias{Visible<-,dg.list-method}
\alias{Visible,dg.list-method}
\alias{Widths<-,dg.list-method}
\alias{Widths,dg.list-method}

\alias{Str,list-method}
\alias{Str,dg.list-method}

\title{Class "dg.list", "dg.NodeList", and "dg.EdgeList" }

\description{ A class for lists of vertices, factors, blocks, and edges. }

\section{Objects from the Class}{
Appropriate slots of \code{\link{DynamicGraph-class}}
and \code{\link{DynamicGraphView-class}} should be of this class.

The lists returned from \code{\link{returnVertexList}},
\code{\link{returnEdgeList}}, \code{\link{returnFactorEdgeList}}, and
\code{\link{returnBlockEdgeList}}, and appropriate components of
the returned values from \code{\link{setBlocks}} and
\code{\link{returnFactorVerticesAndEdges}} are of this class.

Objects can be created by calls of the form \code{new("dg.list", ...)}.
}

\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"list"}: The list. }
  }
}

\section{Extends}{
% Class \code{"list"}, directly.

% Class \code{"dg.list"}, directly.
% Class \code{"list"}, by class \code{"dg.list"}.
Class "dg.EdgeList":

Class \code{"dg.NodeList"}, directly.
Class \code{"dg.list"}, by class \code{"dg.NodeList"}.
Class \code{"list"}, by class \code{"dg.NodeList"}.
}

\section{Methods}{
  \describe{
    \item{asDataFrame}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Blockindices}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Closed<-}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Closed}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Colors<-}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Colors}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Dashes<-}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Dashes}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Indices}{\code{signature(objectlist = "dg.list")}: ... }
    \item{LabelPositions<-}{\code{signature(objectlist = "dg.list")}: ... }
    \item{LabelPositions}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Labels<-}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Labels}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Names<-}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Names}{\code{signature(objects = "dg.list")}: ... }
    \item{NodeAncestors<-}{\code{signature(objectlist = "dg.list")}: ... }
    \item{NodeAncestors}{\code{signature(objectlist = "dg.list")}: ... }
    \item{NodeDescendants<-}{\code{signature(objectlist = "dg.list")}: ... }
    \item{NodeDescendants}{\code{signature(objectlist = "dg.list")}: ... }
    \item{NodeIndices}{\code{signature(objectlist = "dg.list")}: ... }
    \item{NodeTypes}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Oriented<-}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Oriented}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Positions<-}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Positions}{\code{signature(objectlist = "dg.list")}: ... }
    \item{show}{\code{signature(object = "dg.list")}: ... }
    \item{Str}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Strata<-}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Strata}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Visible<-}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Visible}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Widths<-}{\code{signature(objectlist = "dg.list")}: ... }
    \item{Widths}{\code{signature(objectlist = "dg.list")}: ... }
  }
}

\author{Jens Henrik Badsberg}

\seealso{
\code{\link{dynamicGraphMain}}, and
\code{\link{DynamicGraphModel-class}}.
}

% \examples{ }

\keyword{classes}

\eof
\name{drawModel}

\alias{drawModel}
\alias{redrawView}
\alias{redrawGraphWindow}
\alias{menu}

\title{Draw the dynamicGraph window and slaves }

% \newpage

\description{
  The functions \code{drawModel} and \code{redrawView} within
  \code{dynamicGraph} is for adding models to \code{dynamicGraph},
  for adding new views of a model, and for overwriting an existing
  view with an other model.

  The functions can not be found at top level.
}

% \usage{
% redrawGraphWindow(graphLattice = NULL, graphWindow = NULL, 
%                   edgeList = NULL, blockEdgeList = NULL, 
%                   factorVertexList = NULL, factorEdgeList = NULL, 
%                   visibleVertices = NULL, extraList = NULL, 
%                   object = NULL, title = "dynamicGraph", 
%                   transformation = NULL, width = NULL, height = NULL, 
%                   w = NULL, 
%                   vertexColor = NULL, extraVertexColor = NULL, 
%                   edgeColor = NULL, factorVertexColor = NULL, 
%                   factorEdgeColor = NULL, blockEdgeColor = NULL, 
%                   blockColors = NULL, background = NULL, ...)
% 
% drawModel(frameModels = NULL, frameViews = NULL, graphWindow = NULL, 
%           edgeList = NULL, oriented = FALSE, blockEdgeList = NULL, 
%           factorVertexList = NULL, factorEdgeList = NULL, 
%           visibleVertices = NULL, extraList = NULL, 
%           object = NULL, viewType = NULL, title = "dynamicGraph", 
%           transformation = NULL, width = NULL, height = NULL, w = NULL, 
%           vertexColor = NULL, extraVertexColor = NULL, edgeColor = NULL, 
%           factorVertexColor = NULL, factorEdgeColor = NULL, 
%           blockEdgeColor = NULL, blockColors = NULL, 
%           background = NULL, initialWindow = FALSE, 
%           returnLink = FALSE, returnNull = FALSE, ...)
% 
% redrawView(frameModels = NULL, frameViews = NULL, graphWindow = NULL, 
%            edgeList = NULL, oriented = FALSE, blockEdgeList = NULL, 
%            factorVertexList = NULL, factorEdgeList = NULL, 
%            visibleVertices = NULL, extraList = NULL, 
%            viewType = NULL, title = "dynamicGraph", 
%            transformation = NULL, width = NULL, height = NULL, w = NULL, 
%            vertexColor = NULL, extraVertexColor = NULL, edgeColor = NULL, 
%            factorVertexColor = NULL, factorEdgeColor = NULL, 
%            blockEdgeColor = NULL, blockColors = NULL, 
%            background = NULL, initialWindow = FALSE, 
%            returnLink = FALSE, returnNull = FALSE, 
%            setUpdateCountModelMain = FALSE, ...)
% 
% }


\arguments{
  \item{frameModels}{ An object of class \code{\link{DynamicGraph-class}}.
		      NULL,
                      or \code{frameModels} of \code{list(...)$Arguments}. }
  \item{frameViews}{ An object of class \code{\link{DynamicGraphModel-class}}.
                     NULL,
                     or \code{frameViews} of \code{list(...)$Arguments}.
                     If \code{frameViews} is set to \code{NULL}, 
                     the default value, then a new model frame will be created
		     by \code{drawModel}. }
  \item{graphWindow}{ An object of class \code{\link{DynamicGraphView-class}}.
		      If \code{graphWindow} is set to the value of 
                      \code{list(...)$Arguments$graphWindow} then
                      the calling graph window will be redrawn.
                      If \code{graphWindow} is set to \code{NULL}, 
                      the default value,
                      then a new slave graph window will be drawn. }
  \item{graphComponent}{ Returned structure from \code{graphComponent}. }
  \item{edgeList}{ As for \code{\link{dynamicGraphMain}}.
        If \code{edgeList} is given (set to a value different from NULL)
        then this value is used, else the value extracted from
        \code{graphComponent} or \code{list(...)$Arguments} is used.
        If the value is not NULL in \code{graphComponent} then this
	value is used, else the value from \code{list(...)$Arguments}
        is used when \code{edgeList} is NULL. }
  \item{oriented}{ As for \code{\link{dynamicGraphMain}}.
        If \code{oriented} is given then this value is used,
        else the value extracted from \code{list(...)$Arguments} is used. }
  \item{blockEdgeList}{ As for \code{\link{dynamicGraphMain}}.
        If \code{blockEdgeList} ... (as for \code{edgeList}). }
  \item{factorVertexList}{ As for \code{\link{dynamicGraphMain}}.
        If \code{factorVertexList} ... (as for \code{edgeList}). }
  \item{factorEdgeList}{ As for \code{\link{dynamicGraphMain}}.
        If \code{factorEdgeList} ... (as for \code{edgeList}). }
  \item{extraEdgeList}{ As for \code{\link{dynamicGraphMain}}.
        If \code{extraEdgeList} ... (as for \code{edgeList}). }
  \item{visibleVertices}{ As for \code{\link{dynamicGraphMain}}.
        If \code{visibleVertices} ... (as for \code{edgeList}). }
  \item{visibleBlocks}{ As for \code{\link{dynamicGraphMain}}.
        If \code{visibleBlocks} ... (as for \code{edgeList}). }
  \item{extraList}{ As for \code{\link{dynamicGraphMain}}.
        If \code{extraList} ... (as for \code{edgeList}). }
  \item{object}{ As for \code{\link{dynamicGraphMain}}.
        If \code{object} ... (as for \code{oriented}). }
  \item{viewType}{ As for \code{\link{dynamicGraphMain}}.
        If \code{viewType} ... (as for \code{oriented}). }
  \item{title}{ As for \code{\link{dynamicGraphMain}}.
        If \code{title} ... (as for \code{oriented}). }
  \item{transformation}{ As for \code{\link{dynamicGraphMain}}.
        If \code{transformation} ... (as for \code{oriented}). }
  \item{width}{ As for \code{\link{dynamicGraphMain}}.
        If \code{width} ... (as for \code{oriented}). }
  \item{height}{ As for \code{\link{dynamicGraphMain}}.
        If \code{height} ... (as for \code{oriented}). }
  \item{w}{ As for \code{\link{dynamicGraphMain}}.
        If \code{w} ... (as for \code{oriented}). }
  \item{vertexColor}{ As for \code{\link{dynamicGraphMain}}.
        If \code{vertexColor} ... (as for \code{oriented}). }
  \item{extraVertexColor}{ As for \code{\link{dynamicGraphMain}}.
        If \code{extraVertexColor} ... (as for \code{oriented}). }
  \item{edgeColor}{ As for \code{\link{dynamicGraphMain}}.
        If \code{edgeColor} ... (as for \code{oriented}). }
  \item{factorVertexColor}{ As for \code{\link{dynamicGraphMain}}.
        If \code{factorVertexColor} ... (as for \code{oriented}). }
  \item{factorEdgeColor}{ As for \code{\link{dynamicGraphMain}}.
        If \code{factorEdgeColor} ... (as for \code{oriented}). }
  \item{blockEdgeColor}{ As for \code{\link{dynamicGraphMain}}.
        If \code{blockEdgeColor} ... (as for \code{oriented}). }
  \item{blockColors}{ As for \code{\link{dynamicGraphMain}}.
        If \code{blockColors} ... (as for \code{oriented}). }
  \item{extraEdgeColor}{ As for \code{\link{dynamicGraphMain}}.
        If \code{extraEdgeColor} ... (as for \code{oriented}). }
  \item{background}{ As for \code{\link{dynamicGraphMain}}.
        If \code{background} ... (as for \code{oriented}). }
  \item{initialWindow}{ Logical, if \code{initialWindow} is TRUE
			the the labels of the edges are updated. }
  \item{returnLink}{ As for \code{\link{dynamicGraphMain}}.
        If \code{returnLink} ... (as for \code{oriented}). }
  \item{returnNull}{ As for \code{\link{dynamicGraphMain}}.
        If \code{returnNull} ... (as for \code{oriented}). }
  \item{setUpdateCountModelMain}{ Logical. If
                                  \code{setUpdateCountModelMain} is TRUE then
                                  views of the same model will be updated. }
  \item{\dots}{ Used to porting \code{list(...)$Arguments}. }
}

\details{
  The \code{drawModel} and \code{redrawView} functions can be called
  from the functions of menus (main menu and pop up menus) of 
  \code{\link{dynamicGraphMain}}, from .GlobalEnv in 
  \code{\link{DynamicGraph}}) via returned values from
  \code{\link{dynamicGraphMain}} (and from the methods of the model
  object in the scope of the function \code{\link{dynamicGraphMain}}).
  As a result the graph window will be redrawn with an other view of the model,
  possible with, e.g., other edges, an other model is drawn,
  or a new slave graph window will appear.

  If the value of a argument to  \code{drawModel} or \code{redrawView} is set,
  then this value is used, else the value from the calling window is used.
  The value of the calling window is given in the argument
  \code{Arguments} in the call of the function of the menu item.

  Below is an example, where items for labeling all the edges
  of the graph are added to the menu.
  The edges are visited, a test is computed for each edge,
  the label and width of the edge is updated, and the
  graph is drawn with the updated edge list.
}

\value{
  The returned value from \code{\link{dynamicGraphMain}}.
}


\author{Jens Henrik Badsberg}

\note{ The functions can not be called from top level, that is,
       the functions does not exists at .GlobalEnv, but only
       in returned values from \code{\link{dynamicGraphMain}}.

       It is recommended that the functions not are called, but that
       \code{\link{DynamicGraph}} is used with the arguments
       \code{frameModels}, \code{frameViews}, \code{graphWindow},
       \code{addModel}, \code{addView}, and/or \code{overwrite}
       to call the functions.
 }

\seealso{ See also \code{\link{dynamicGraphMain}},
          \code{\link{DynamicGraph}}
	  \code{\link{DynamicGraph-class}}, and
	  \code{\link{DynamicGraphModel-class}}.  }

\examples{
# The use of "drawModel"  and "redrawView" by 
# "DynamicGraph" in the example "usermenus" of demo:

your.DrawModel <- function(object, slave = FALSE, viewType = "Simple", ...) {
    args <- list(...)
    Args <- args$Arguments

    # Here you should make your new model (this is just a copy):

    Object <- object
    title <- Object@name

    # and compute edges (here 'NULL' if the model not has been updated):

    Edges <- graphComponents(Object, viewType, Arguments = Args)
    EdgeList       <- Edges$vertexEdges
    ExtraVertices  <- Edges$extraVertices
    FactorVertices <- Edges$factorVertices
    FactorEdges    <- Edges$factorEdges
    BlockEdges     <- Edges$blockEdges
    ExtraEdges     <- Edges$extraEdges
    visualVertices <- Edges$visualVertices
    visualBlocks   <- Edges$visualBlocks

    if (slave) {
      # Drawing ''an other model'' in a new window:
      DynamicGraph(addModel = TRUE,                   # <-
                   frameModels = Args$frameModels, 
                   frameViews = NULL,                 # <- not used here
                   graphWindow = NULL,                # <- not used here
                   edgeList = EdgeList,
                   object = Object, 
                   extraList = ExtraVertices, 
                   extraEdgeList = ExtraEdges, 
                   factorVertexList = FactorVertices, 
                   factorEdgeList = FactorEdges, 
                   blockEdgeList = BlockEdges, 
                   visualVertices = visualVertices,
                   visualBlocks = visualBlocks,
                   title = title, 
                   Arguments = Args)
    } else {
      # Overwriting with ''an other model'' in same view:
      DynamicGraph(overwrite = TRUE,                 # <-
                   addModel = TRUE,                  # <-
                   frameModels = Args$frameModels, 
                   frameViews = Args$frameViews, 
                   graphWindow = Args$graphWindow,   # <-
                   edgeList = EdgeList, 
                   object = Object, 
                   extraList = ExtraVertices, 
                   extraEdgeList = ExtraEdges, 
                   factorVertexList = FactorVertices, 
                   factorEdgeList = FactorEdges, 
                   blockEdgeList = BlockEdges, 
                   visualVertices = visualVertices,
                   visualBlocks = visualBlocks,
                   title = "Not used!", 
                   width = NULL, height = NULL, 
                   Arguments = Args) }
}

your.LabelAllEdges <- function(object, slave = FALSE, ...) 
 {
  args <- list(...)
  Args <- args$Arguments

  getNodeName <- function(index, type)
    if (type == "Vertex")
      name(Args$vertexList[[index]])
    else if (type == "Factor")
      name(Args$factorVertexList[[abs(index)]])
    else if (type == "Extra")
      name(Args$extraList[[abs(index)]])
    else if (type == "Block")
      label(Args$blockList[[abs(index)]])
    else
      NULL

  visitEdges <- function(edges) {
    for (i in seq(along = edges)) {
      vertices <- nodeIndicesOfEdge(edges[[i]])
      types    <- nodeTypesOfEdge(edges[[i]])

      name.f <- getNodeName(vertices[1], types[1])
      name.t <- getNodeName(vertices[2], types[2])

      R <- testEdge(object, action = "remove",
                    name.1 = name.f, name.2 = name.t,
                    from = vertices[1], to = vertices[2],
                    from.type = types[1], to.type = types[2],
                    edge.index = i, force = force, Arguments = Args)

      if (!is.null(R)) {
        if (TRUE || (hasMethod("label", class(R))))
          label(edges[[i]]) <- label(R)
        if (TRUE || (hasMethod("width", class(R))))
          width(edges[[i]]) <- width(R)
      }
    }
    return(edges)
  }

  edgeList <- visitEdges(Args$edgeList)
  factorEdgeList <- visitEdges(Args$factorEdgeList)
  blockEdgeList <- visitEdges(Args$blockEdgeList)

  if (slave) {
    # Adding an other view of the same model:
    DynamicGraph(addView = TRUE,                  # <-
                 frameModels = Args$frameModels, 
                 frameViews = Args$frameViews, 
                 graphWindow = NULL,              # <- not used here
                 edgeList = edgeList, 
                 factorEdgeList = factorEdgeList, 
                 blockEdgeList = blockEdgeList, 
                 title = "A slave window", 
                 Arguments = Args)
  } else {
    # Overwriting with an other view of the same model:
    DynamicGraph(overwrite = TRUE,                # <-
                 addView = TRUE,                  # <-
                 frameModels = Args$frameModels, 
                 frameViews = Args$frameViews, 
                 graphWindow = Args$graphWindow,  # <-
                 edgeList = edgeList, 
                 factorEdgeList = factorEdgeList, 
                 blockEdgeList = blockEdgeList, 
                 title = "Not used!", 
                 width = NULL, height = NULL, 
                 Arguments = Args) } 
 }

 palle <- function(...) print("Palle")
 palle <- function(...) print(list(...)$Arguments$object@name)

 Menus <- 
 list(MainUser = 
      list(label = "Transformation by 'prcomp' on position of \"vertices\", and redraw",
           command = function(object, ...) {
             Args <- list(...)$Arguments
             transformation <- t(prcomp(Positions(Args$vertexList))$rotation)
             Args$redrawView(graphWindow = Args$graphWindow,
                             transformation = transformation, Arguments = Args)
             }),
      MainUser = 
      list(label = "Position of \"vertices\" by 'cmdscale', and redraw",
           command = function(object, ...) {
             Args <- list(...)$Arguments
             Vertices <- Args$vertexList
             Edges <- Args$edgeList
             positions <- Positions(Args$vertexList)
             N <- dim(positions)[2]
             e <- NodeIndices(Edges)
             n <- Names(Vertices)
             X <- matrix(rep(-1, length(n)^2), ncol = length(n))
             for (i in 1:length(e)) {
               suppressWarnings(w <- as.numeric(names(e)[i]))
               if (is.na(w)) w <- .5
               X[e[[i]][1], e[[i]][2]] <- w
               X[e[[i]][2], e[[i]][1]] <- w
             }
             dimnames(X) <- list(n, n)
             d <- 1.25
             X[X==-1] <- d
             X <- X - d * diag(length(n))
             mdsX <- cmdscale(X, k = N, add = TRUE, eig = TRUE, x.ret = TRUE)
             # mdsX <- isoMDS(X, k = N)
             M <- max(abs(mdsX$points))
             Positions(Args$vertexList) <<- mdsX$points / M * 45
             Args$redrawView(graphWindow = Args$graphWindow, 
                             # Positions = Positions(Args$vertexList), 
                             vertexList = Args$vertexList, Arguments = Args)
             }),
      MainUser = 
      list(label = "Position of \"vertices\"",
           command = function(object, ...) 
             print(Positions(list(...)$Arguments$vertexList))),
      MainUser = 
      list(label = "Label all edges, in this window",
           command = function(object, ...) 
                       your.LabelAllEdges(object, slave = FALSE, ...)),
      MainUser = 
      list(label = "Label all edges, in slave window",
           command = function(object, ...) 
                       your.LabelAllEdges(object, slave = TRUE, ...)),
      MainUser = 
      list(label = "Draw model, in this window",
           command = function(object, ...) 
                       your.DrawModel(object, slave = FALSE, ...)),
      MainUser = 
      list(label = "Draw model, in slave window",
           command = function(object, ...) 
                       your.DrawModel(object, slave = TRUE, ...)),
      MainUser = 
      list(label = "Call of function 'modalDialog', result on 'title' at top",
           command = function(object, ...) {
             Args <- list(...)$Arguments
             ReturnVal <- modalDialog("Test modalDialog Entry",
                                      "Enter name", Args$title,
                                      top = Args$top)
             print(ReturnVal)
             if (ReturnVal == "ID_CANCEL")
               return()
             tktitle(Args$top) <- ReturnVal } ),
      MainUser = 
      list(label = "Call of function 'palle', result on 'viewLabel' at bottom",
           command = function(object, ...) {
             Args <- list(...)$Arguments
             tkconfigure(Args$viewLabel, 
                         text = paste(Args$viewType, " | ", palle(...))) } ),
      Vertex = 
      list(label = "Test of user popup menu for vertices: Label",
           command = function(object, name, ...) {
             # print(name)
             args <- list(...)
             # print(names(args))
             # print(c(args$type))
             # print(c(args$index))
             Args <- args$Arguments
             print(Args$vertexList[[args$index]]@label) } ),
      Edge = 
      list(label = "Test of user popup menu for edges: Class",
           command = function(object, name1, name2, ...) {
             args <- list(...)
             # print(c(name1, name2))
             # print(c(args$edge.index, args$which.edge, args$from, args$to))
             # print(c(args$from.type, args$to.type, args$edge.type))
             Args <- list(...)$Arguments
             ReturnVal <- selectDialog("Test selectDialog Entry",
                                       "Select name", Args$edgeClasses[,1],
                                       top = Args$top)
             print(ReturnVal)
             if (ReturnVal == "ID_CANCEL")
               return()
             if ((args$from > 0) && (args$to > 0)) {
               edgeList <- Args$edgeList
               class(edgeList[[args$edge.index]]) <- 
                                              Args$edgeClasses[ReturnVal, 2]
               # vertexEdges(Args$object) <<- edgeList # Not working !!!
               Args$redrawView(graphWindow = Args$graphWindow,
                               edgeList = edgeList, title = "Not used!", 
                               width = NULL, height = NULL, Arguments = Args)
             } } ),
      ClosedBlock = 
      list(label = "Test of user popup menu for blocks",
           command = function(object, name, ...) {
             print(name)
             print(c(list(...)$index)) } )
     )
}

\keyword{ methods }

\eof
\name{dynamicGraph-internal}
\title{Internal dynamicGraph objects}
\alias{.First.lib}
\alias{.onLoad}
\alias{.onLoad.dynamicGraph}
\alias{.onLoad.dynamicGraphInterface}
\description{Internal dynamicGraph objects.}
\details{These are not to be called by the user.}
\keyword{internal}

\eof
\name{dynamicGraphMain}

\alias{dynamicGraphMain}
\alias{dynamicGraph}

% \newpage

\title{ Dynamic Graph }
\description{
    Interactive plot for manipulating graphs.
}

\usage{
dynamicGraphMain(vertexList, visibleVertices = 1:length(vertexList), 
    visibleBlocks = 1:length(blockList), edgeList = NULL, oriented = FALSE, 
    blockList = NULL, blockEdgeList = NULL, blockTree = NULL, 
    factorVertexList = NULL, factorEdgeList = NULL, 
    extraList = NULL, extraEdgeList = NULL, object = NULL, objectName = NULL, 
    vertexClasses = validVertexClasses(), 
    factorClasses = validFactorClasses(), 
    edgeClasses = validEdgeClasses(), 
    viewClasses = validViewClasses(), viewType = "Simple", 
    title = "dynamicGraph", 
    transformation = NULL, width = 400, height = 400, w = 6, 
    vertexColor = "blue", extraVertexColor = "black", edgeColor = "blue", 
    factorVertexColor = "green", 
    factorEdgeColor = "green", blockEdgeColor = "default", 
    blockColors = NULL, extraEdgeColor = "peru", background = "white", 
    closeenough = 2, setVertexLabels = FALSE, drawBlockFrame = TRUE, 
    drawBlockBackground = FALSE, UserMenus = NULL, hasMethods = TRUE, 
    enterLeaveUpdate = TRUE, updateAllViews = TRUE,
    namesOnEdges = TRUE, updateEdgeLabels = TRUE, useNamesForLabels = TRUE, 
    debug.strata = FALSE, debug.edges = FALSE, debug.position = FALSE, 
    debug.update = FALSE, permitZoom = TRUE, margin = 100, 
    returnLink = FALSE, returnNull = FALSE, ...) 
}

\arguments{
    \item{vertexList}{ List of vertices
        (each of class containing the class \code{dg.Vertex})
        created by \code{\link{returnVertexList}}
        or exported from \code{\link{dynamicGraphMain}}. }
    \item{visibleVertices}{ Numeric vector of the indices of the vertices of
        \code{vertexList} to plot. }
    \item{visibleBlocks}{ Numeric vector of the indices of the blocks of
        \code{blockList} to plot. }
    \item{edgeList}{ List of edges (of class containing \code{dg.Edge})
        created by \code{\link{returnEdgeList}}
        or exported from \code{\link{dynamicGraphMain}}.  }
    \item{blockList}{ List of blocks (each of class \code{dg.Block})
        created by \code{\link{setBlocks}}
        or exported from \code{\link{dynamicGraphMain}}.
        If the \code{blockTree} argument is given, then
        the argument \code{blockList} should not also be given. }
    \item{blockEdgeList}{ List of blockedges
        (of class containing the class \code{dg.BlockEdge})
        created by \code{\link{returnBlockEdgeList}}
        or exported from \code{\link{dynamicGraphMain}}. }
    \item{blockTree}{ Structure with blocks (of class \code{dg.Block})
        created by \code{\link{setTreeBlocks}}
        or exported from \code{\link{dynamicGraphMain}}. }
    \item{oriented}{ Logical.  If TRUE then
        edges are oriented, also when blocks are missing. }
    \item{factorVertexList}{ List of secondary vertices,
        called factor vertices, for, e.g., the generators of the model,
        (each of class containing \code{dg.FactorVertex})
        created by \code{\link{returnFactorVerticesAndEdges}}
        or exported from \code{\link{dynamicGraphMain}}. }
    \item{factorEdgeList}{ List of bipartite graph edges,
        called factor edges,
        (each of class containing \code{dg.FactorEdge})
        created by \code{\link{returnFactorEdgeList}}
        or exported from \code{\link{dynamicGraphMain}}.
        Factor edges are edges between vertices and factor vertices. }
    \item{extraList}{ List of vertices (of class containing the class
	\code{dg.Vertex}) 
        created by the function\code{\link{returnVertexList}}
        or exported from \code{\link{dynamicGraphMain}},
        for, e.g., additional titles in the plot. }
    \item{extraEdgeList}{ List of edges between extra vertices and vertices. }
    \item{object}{ NULL, or object with the methods \code{\link{modifyModel}}
        and \code{\link{testEdge}} - for respectively updating the
        object when the graph is updated, and for computing the
	test statistic when an edge is labeled. The returned object from
        \code{\link{testEdge}} should have the methods 
        \code{label} and \code{width}
        % (was \code{labelOfTest} and \code{widthOfTest})
        for extracting the label of a test for putting the label on the edge
        and for extracting the width for the edge.
        See \code{\link{dg.Model-class}}. }
    \item{objectName}{ If set to a text string then the object is assigned with
	this name in .Global.Env when the object is updated. }
    \item{vertexClasses}{ Returned value from \code{\link{validVertexClasses}},
        or extension of this matrix. Used when creating
        new vertices in \code{\link{dynamicGraphMain}}. }
    \item{factorClasses}{ Returned value from \code{\link{validFactorClasses}},
        or extension of this matrix. Used when creating
        new factor vertices in \code{\link{dynamicGraphMain}}. }
    \item{edgeClasses}{ Returned value from \code{\link{validEdgeClasses}},
        or extension of this matrix. Used when creating
        new edges in \code{\link{dynamicGraphMain}}. }
    \item{viewClasses}{ Returned value from \code{\link{validViewClasses}},
        or extension of this matrix. Used when creating
        new views in \code{\link{dynamicGraphMain}}. }
    \item{viewType}{ A text string with the type of view. }
    \item{title}{ Text string with the title set on the graph window. }
    \item{transformation}{ NULL,
                or rotation matrix for projecting the positions
        of the vertices, blocks, etc. onto the canvas. }
    \item{width}{ Integer with the width of the plot canvas. }
    \item{height}{ Integer with the height of the plot canvas. }
    \item{w}{ The \code{radius} of the vertices. Send as argument to the
        \code{draw} method for vertices, and edges are shortened
        by the quantity in each end. }
    \item{vertexColor}{ Single text string. 
        Colors of new vertices created in
        \code{\link{dynamicGraphMain}}
        (by \code{\link{newVertex}}).
        The colors of the vertices of \code{vertexList} are
        given in this list. \code{vertexColor} is given as
        argument to the \code{draw} method of vertices, but this 
        argument are by default not used in the draw method. }
    \item{extraVertexColor}{ Single text string. 
        As \code{vertexColor}, but for the vertices of
        \code{extraList}. }
    \item{edgeColor}{ Single text string. 
        \code{edgeColor} is similar to \code{vertexColor}. }
    \item{factorVertexColor}{ Single text string. 
        \code{factorVertexColor} is similar to
        \code{vertexColor}.
        If \code{factorVertexColor} is "default" then
        the color of a factor vertex will depend on the
        type of the generator of the factor. }
    \item{factorEdgeColor}{ Single text string. 
        \code{factorEdgeColor} is similar to \code{edgeColor}. }
    \item{blockEdgeColor}{ "default", or list with two text strings for colors.
        \code{blockEdgeColor} is similar to \code{edgeColor}.
        The two colors are used for respectively edges
        between two blocks and for edges between blocks 
        and vertices. }
    \item{blockColors}{ List of colors of blocks. 
        Similar to \code{vertexColor}: Only used when creating
        new blocks, else the colors set in \code{blockList}
        and \code{blockTree} are used. }
  \item{extraEdgeColor}{ Single text string. 
        \code{extraEdgeColor} is similar to \code{edgeColor}. }
    \item{background}{ The color of the canvas of the graph window,
        default: "white". }
    \item{closeenough}{ Parameter for whether the mouse is close enough
        when clicking an object of the graph window,
        see \code{\link[tcltk]{tkcanvas}}. }
    \item{setVertexLabels}{ Logical.
                If TRUE then the vertices are marked by the label,
                else the name. }
    \item{drawBlockFrame}{ Logical.
                If TRUE then frames are drawn around blocks. }
    \item{drawBlockBackground}{ Logical. If TRUE then a \code{block canvas} is
        drawn, with color set by \code{blockColors}. }
    \item{UserMenus}{ List with user defined menu items for main menu
        and pop up menus. See \code{\link{redrawView}}
        for an example of a user specified menu. }
    \item{hasMethods}{ Logical. 
        If TRUE then the \code{object} should have the methods
        \code{\link{modifyModel}} and \code{\link{testEdge}}. 
        (I do not know why the R-function
         \code{\link[methods]{hasMethod}} does not
        work on \code{objects} (defined outside the package
	\code{\link{dynamicGraph}})
	inside \code{\link{dynamicGraphMain}}).}
    \item{enterLeaveUpdate}{ Logical. If FALSE then the graph window
        is not redrawn when the mouse enters and leaves the graph window. }
    \item{updateAllViews}{ Logical.
                If TRUE then all windows are updated when one is changed. }
    \item{namesOnEdges}{ Logical. If FALSE then the names of the vertices 
        are not set on the edge as label. }
    \item{updateEdgeLabels}{ Logical.  If FALSE then the edge labels
        are not cleared when the model is updated. }
    \item{useNamesForLabels}{ Logical. If \code{useNamesForLabels} is TRUE
        then names are sued for labels. }
    \item{debug.strata}{ Logical for tracing strata of vertices
                (also in plot). }
    \item{debug.edges}{ Logical for tracing edges (also by labels in plot). }
    \item{debug.position}{ Logical for tracing positions of vertices. }
    \item{debug.update}{ Logical for tracing redraw of graph window. }
    \item{permitZoom}{ Logical. If FALSE then zooming is disabled,
		       and no margin around the canvas is set. }
    \item{margin}{ Integer, the width of the margin round the canvas. }
    \item{returnLink}{ Logical, if \code{returnLink} is TRUE then a reference
		       to the dynamic graph window is returned, else an object 
		       of class \code{\link{DynamicGraph-class}}
                       is returned (if not \code{returnNull} is TRUE).
    }
    \item{returnNull}{ Logical, if \code{returnNull}
                       is TRUE then NULL is returned. }
    \item{\dots}{ Additional arguments. }
}

\details{

    This is a dynamic plotting tool for handling graphs.
    The lay out of the graph can be edited by moving
    the vertices of the graph by the mouse, and edges can be 
    added by clicking vertices and dropping by clicking the edges.

    The function is incremental in the sense that the user can add a method
    for updating the model object of the window when the graph is updated,
    and a method for computing the test of an edge when the edge is clicked
    by the mouse.
    
    Edges can be oriented, drawn by arrows.
    
    Blocks can be used to define a causal structure of the variables
    represented by vertices, and edges between blocks are then oriented.
    Blocks can be given in a structure such that descendant blocks of a blocks
    also are closed when a block is closed.
    
    A secondary set of vertices, factor vertices, can be used to represent
    hypergraphs.
    
    "Slave graph windows" can be created: The windows will share vertices
    and blocks, thus when a vertex is moved in one window, the position of
    the vertex will also change in all slave windows. The edges are not
    shared among windows, since the individual windows will typical
    represent different models.
    Thus factors (vertices and edges) are not shared between graph windows.

    }

% Rotate:            Right click and drag

% Rock plot:         Double right click (not working!)

\value{
    An object of class \code{\link{DynamicGraph-class}}
    (if not \code{returnNull} is TRUE)
    with the lists of vertices and blocks (block trees) of the dynamicGraph,
    and list of models, each (of class \code{\link{DynamicGraphModel-class}})
    with the views of the model.

    Each view (of class \code{\link{DynamicGraphView-class}}) of a model
    will hold the edges (edges between vertices, factors and blocks)
    and factor- and extra-vertices of the view,
    together with which vertices and blocks are visible in the view.

    If \code{returnLink} is TRUE then the returned object
    will have functions for drawing other models and views
    associated with the returned  \code{\link{DynamicGraph-class}} object.

    The function \code{\link{drawModel}} of the returned object
    is used to add and draw other models, 
    in an existing graph window or in a new slave graphs window,
    see \code{\link{DynamicGraph}}.

    The function \code{\link{redrawView}} of the models of the returned object
    is used to draw other views of the model,
    in an existing graph window or in a new slave graphs window,
    see \code{\link{DynamicGraph}}.

%     \item{...@graphs}{Slot 'graphs':
%         List of class \code{\link{DynamicGraphView}}.
% 	Might, beside slots for accessing the graph window, also hold 
% 	slots for edges, factors, factor edges, block edges, etc.}
%     \item{...@graphs[[i]]@top}{Slot 'top' of each object in the list of the
%         'graphs' slot of the returned object:
%         The 'top' of the canvas of the i-the 
%         graph window.}

}

    \section{All object in the graph window:}{
	\itemize{
	    \item{Left click, hold and drag:}{The object will move.}
	    \item{Left click:}{Action to the object:
                Vertices, edges, and blocks will highlight, 
                at vertices and blocks edges are added after highlighting,
                tests are computed for edge labels.}
	    \item{Left or right click, with SHIFT or/and CONTROL:}{The object
                  will be marked.}
	    \item{Double left click:}{Action to object:
                Vertices and edges are deleted,
                blocks will close,
                closed blocks will open.}
	    \item{Right click:}{The pop up menu of the object will appear.}
        }
    }

    \section{Vertices (vertices and factor vertices):}{
	Right click the vertex to get the pop up menu of the vertex:
	\itemize{
	    \item{Highlight a vertex:}{For adding an edge -
                Left click the vertex.}
	    \item{Highlight a vertex:}{For adding to "selectedNodes" -
                Left or right click the vertex while
                holding SHIFT or/and CONTROL down.}
	    \item{Mark a vertex:}{For adding edges, etc. - 
	        Left or right click the vertex
                while holding SHIFT or/and CONTROL down.}
	    \item{Cancel highlighting:}{Left click (or drag) the vertex.}
	    \item{Add an edge:}{After first highlighting a vertex - 
                Left click the other vertex.}
	    \item{Move a vertex:}{Left click and drag the vertex.}
	    \item{Move a vertex label:}{Left click and
                drag the label of the vertex.}
	    \item{Delete a vertex:}{Double left click the vertex.}
	    \item{Create new graph:}{A slave window with the vertex delete -
		Select "Drop vertex" from pop up menu at the vertex.}
	    \item{Change a vertex label:}{Double left click the label,
		and enter the new label in the appearing dialog window.}
	    \item{Delete a vertex label:}{Select "Delete vertex label" from
		the pop up menu at the vertex or at the vertex label.}
	    \item{Create new vertex:}{At mouse position -
		Middle click the canvas.}
	    \item{Create new vertex:}{At mouse position
                with the edge to last vertex -
		Double middle click the canvas.}
	}
	The main menu "Variables":
	\itemize{
	    \item{Create new variable:}{With setting the class of the new
		vertex and defining an expression for updating the
		\code{object} - Select "Create new variable".} 
	    \item{Display, add, a vertex:}{In the current window -
		Select "Select vertex among variables not displayed (here)". }
	    \item{Display, add, a vertex:}{In a slave window -
		Select "Select vertex among variables not displayed (slave)". }
	    \item{Export the vertex list:}{Select
                "Assign 'vertexList' in .GlobalEnv". }
	    \item{Export the labels:}{Select
                "Assign 'extraList' in .GlobalEnv". }
	}
    }
    
    \section{Edges (edges to/from vertices, blocks and factors):}{
	Right click the edge to get the pop up menu of the edge:
	\itemize{
	    \item{Highlight a edge:}{Left click the edge.}
	    \item{Highlight a edge:}{For adding to "selectedEdges" -
                Left or right click the edge while
                holding SHIFT or/and CONTROL down.}
	    \item{Add an edge:}{Left click first the vertex (or block)
                to highlight, and then left click other vertex (or block).}
	    \item{Delete an edge:}{And update in the current window -
		Double left click the edge.}
	    \item{Delete an edge:}{And create a slave graph with the
		resulting graph
		- Select "Drop edge" from the pop up menu at the edge.}
	    \item{Move an edge (2 vertices):}{Left
                click the edge and drag the edge.}
	    \item{Move an edge label:}{Left click the edge label and
		drag the label.}
	    \item{Set an edge label:}{Select "Set edge label" from the
		edge pop up menu.}
	    \item{Compute an edge label:}{Left click the edge label,
		or select "Compute edge label" from pop up menu at the edge.}
	    \item{Compute an edge label:}{Force computation for
		"harder models" - Double left  click the edge label,
		or select "Force compute edge label" from pop up menu of
		the edge.}
	    \item{Delete an edge label:}{Triple left click the edge label,
		or select "Delete label of edge" from the pop up menu at
		the edge.}
	}
	The main menu "Edges":
	\itemize{
	    \item{Delete all edge labels:}{Select "Delete all edge labels". }
	    \item{Export the edge list:}{Select "Assign 'edgeList' in .GlobalEnv". }
	}
    }

    \section{Blocks, opened:}{
	Right click the block label (or colored  block canvas if
	\code{drawBlockBackground} is set to TRUE) to get the pop up
	menu of opened block:
	\itemize{
	\item{Move a block:}{With its sub blocks and vertices - 
                Left click the block label or colored block canvas and
		drag the block. [ Slow !!!] } 
	\item{Resize a block:}{Left click a block corner or a block
                edge and drag. }
	\item{Minimize a block:}{With its sub blocks and vertices - 
                Double left click the block label or the colored block canvas.
              }
	\item{Maximize a block:}{Zoom to the block - Right click the
                block label or colored the block canvas and select
		"Maximize" in the appering popup menu. }
	\item{Zoom out to the full graph:}{Right click the block label
	    or colored block canvas and select "Redraw (Zoom to) full
	    graph" in the appearing block pop up menu. }
	    }
    }			  			   	

    \section{Blocks, closed:}{
        Right click the block to get the pop up menu of the closed block:
	\itemize{
	    \item{Highlight a block:}{For adding edges from all vertices
		of the block to a vertex or a block -
                Left click the block.}
	    \item{Highlight a block:}{For adding to "selectedNodes" -
                Left or right click the block while
                holding SHIFT or/and CONTROL down.}
	    \item{Mark a block:}{For adding edges, etc. - 
	        Left or right click the block
                while holding SHIFT or/and CONTROL down.}
	    \item{Cancel highlighting:}{Of a block - Left click (or drag)
		the block.}
	    \item{Add edges:}{To all vertices of block
		after highlighting first a block or a vertex - Click
                the other block.}
	    \item{Move a closed block:}{Left click and drag the block.}
	    \item{Move a block label:}{Left click and drag the label
                of the block.}
	    \item{Open a closed block:}{Double left click the block.}
	    \item{Change a block label:}{Double left click the label of
		the closed block, 
		and enter the new label in the appearing dialog window.}
	    \item{Delete a block label:}{Select "Delete label of block" from
		the pop up menu at the block or the block label.}
	}
	The main menu "Blocks":
	\itemize{	    
	    \item{Export the block list:}{Select
                "Assign 'blockList' in .GlobalEnv". }
	    \item{Export the block tree:}{Select
                "Assign 'blockTree' in .GlobalEnv". }
	    \item{Export the block edges:}{Select
                "Assign 'blockEdgeList' in .GlobalEnv". }
	}
    }			  			   	

    \section{Factor vertices:}{
        Right click the factor vertex to get the pop up menu of the
	factor: Actions as for vertices.
% 	\itemize{
% 	    \item{Pop up menu for factor vertex:}{Right click the
%                 factor vertex:
% 		Actions as for vertices.}
% 	}
	The main menu "Generators":
	\itemize{	    
	    \item{Export the factor vertices:}{Select
                "Assign 'factorVertexList' in .GlobalEnv". }
	    \item{Export the factor edges:}{Select
                "Assign 'factorEdgeList' in .GlobalEnv". }
		}			  			   	
	    }  			   	
	    
    \section{Factor edges:}{
        Right click the factor edge to get the pop up menu of the factor
	edge: Actions are as for edges.
%	\itemize{
%	    \item{Pop up menu for factor edges:}{Right click the factor edge:
%		Actions are as for edges.}
%		}			  			   	
	    }

    \section{The graph:}{
	\itemize{
	    \item{Create a slave window:}{Select "Make slave window: Same model".}
	    \item{Create a slave window:}{Select "Make slave window: Copy model".}

	    \item{Switch class of view:}{Select "Set class of graph window".}

	    \item{Refresh:}{Faster fix of "corrupted" window:
                            Select "Refresh view (set positions as 'stored')".}
	    \item{Redraw the view (zoom out):}{Select "Redraw graph window (more refreshing)".}
	    \item{Update model:}{Select "Update model and redraw (total refreshing)".}

	    \item{Enable rotation:}{Select "Rest (enable) rotation".}
	    \item{Disable rotation:}{Select "Disable rotation".}

	    \item{Zoom in:}{Select "Zoom in" from main menu, or hit <F1> in canvas. }
	    \item{Zoom out:}{Select "Zoom out" from main menu, or hit <F2> in canvas.}

	    \item{Export current arguments:}{Select
		"Assign 'Args' in .GlobalEnv".}
	    \item{Export the model lattice:}{Select
		"Assign 'frameModels' in .GlobalEnv".}
	    \item{Export the graph lattice:}{Select
		"Assign 'frameViews' in .GlobalEnv".}
	    \item{Export the graph window:}{Select
		"Assign 'graphWindow' in .GlobalEnv".}
	    \item{Export the object:}{Select
		"Assign 'object' in .GlobalEnv".}
	}			  			   	
    }			  			   	
    
    \section{Rotation:}{
	\itemize{
	    \item{Enable rotation:}{Select
		"Enable (reset) transformation" from the main menu "Graph".}
	    \item{Rotate the graph:}{Middle click the canvas, and drag.}
	    \item{Disable rotation:}{Select
		"Disable transformation" from the main menu "Graph".}
	    \item{Export transformation:}{Export the projection
		matrix - Select
		"Assign 'transformation' in .GlobalEnv"
                from the main menu "Graph".}
	}			  			   	
    }

% {}

% http://www.math.auc.dk/gr/material/CoCo/
% http://www.jbs.agrsci.dk/Biometry/Software-Datasets/CoCo/CoCo.1.6/

\section{Acknowledgments}{
Many thanks to the gR-group for useful discussions, especially to 
Claus Dethlefsen for testing early versions of this package on DEAL.
}

% \references{
%      CoCo, avaliable form \url{http://www.math.auc.dk/gr/material/CoCo} and
%      \url{http://www.jbs.agrsci.dk/}
%      \code{Biometry/Software-Datasets/CoCo/CoCo.1.6/}
%      with a guide at \url{http://www.jstatsoft.org/v06/i04/},
%      has an interface to \code{\link{dynamicGraph}}. }

\references{ 
    CoCo, with a guide at \url{http://www.jstatsoft.org/v06/i04/},
    has an interface to \code{\link{dynamicGraph}}. }

\author{Jens Henrik Badsberg}

\note{
     Vertices, edges, blocks, block edges,
     factors, and factor edges are objects of the following classes:
     \code{\link{dg.Vertex}}, \code{\link{dg.Edge}},
     and \code{\link{dg.Block}} contains \code{\link{dg.Node}},
     \code{\link{dg.FactorVertex}} contains \code{\link{dg.Vertex}}, and
     \code{\link{dg.VertexEdge}}, \code{\link{dg.BlockEdge}}, 
     \code{\link{dg.FactorEdge}} contains \code{\link{dg.Edge}}.

     The methods 
     \code{\link{draw}},
     \code{\link{color}},
     \code{\link{color<-}},
     \code{\link{label}},
     \code{\link{label<-}},
     \code{\link{labelPosition}},
     \code{\link{labelPosition<-}},
     \code{\link{name}},
     \code{\link{name<-}},
     \code{\link{index}},
     \code{\link{index<-}},  
     \code{\link{position}},
     \code{\link{position<-}},
     \code{\link{stratum}},
     \code{\link{stratum<-}},
     \code{\link{visible}},
     \code{\link{visible<-}},
     \code{\link{addToPopups}},
     \code{\link{oriented}},
     \code{\link{oriented<-}},
     \code{\link{width}},
     \code{\link{width<-}},
     \code{\link{nodeIndicesOfEdge}},
     \code{\link{nodeIndicesOfEdge<-}},
     \code{\link{nodeTypesOfEdge}},
     % ( \code{\link{labelOfEdge}},
     %   \code{\link{labelOfEdge<-}},
     %   \code{\link{widthOfEdge}},
     %   \code{\link{widthOfEdge<-}} ),
     \code{\link{ancestors}},
     \code{\link{ancestors<-}},
     \code{\link{descendants}}, and
     \code{\link{descendants<-}},
     are implemented for objects of these classes.

     For lists of vertices, edges, blocks, block edges,
     factors, and factor edges the methods
     \code{\link{Names}},
     \code{\link{Names<-}},
     \code{\link{Colors}},
     \code{\link{Colors<-}},
     \code{\link{Labels}},
     \code{\link{Labels<-}},
     \code{\link{LabelPositions}},
     \code{\link{LabelPositions<-}},
     \code{\link{Positions}},
     \code{\link{Positions<-}},
     \code{\link{Strata}},
     \code{\link{Strata<-}},
     \code{\link{Indices}},
%    \code{\link{Indices<-}},
%     \code{\link{Visible}},
%     \code{\link{Visible<-}},
     \code{\link{NodeAncestors}},
     \code{\link{NodeAncestors<-}},
     \code{\link{NodeDescendants}}, and
     \code{\link{NodeDescendants<-}} are available.

     The block tree created by \code{\link{setTreeBlocks}} is unlisted by
     \code{\link{blockTreeToList}} such that the above methods
     can be applied.

     The model object of the call of \code{\link{dynamicGraphMain}}
     should have the methods \code{modifyModel}, \code{testEdge},
     \code{graphComponents}, and \code{setGraphComponents}.
     When the graph is modified, by adding or dropping vertices or edge,
     the method \code{modifyModel} is called on the argument \code{object}
     of \code{\link{dynamicGraphMain}}. If an \code{object} is returned in
     the list of the returned value from \code{modifyModel} then
     \code{object} in \code{\link{dynamicGraphMain}} is replaced by this
     object, and the object is also assigned in the top level environment,
     if \code{objectName} was given to \code{\link{dynamicGraphMain}}.

     The method \code{testEdge} of \code{object} should return an object with
     the methods \code{label} and \code{width} for labeling edges,
     see \code{\link{dg.Test-class}}.

     The methods \code{graphComponents} and \code{setGraphComponents}
     are used to communicate the graph components between several views
     of the same model.
     The method \code{graphComponents} of the model object is for
     returning the objects to draw in the view, depending on the viewType.
     The method\code{setGraphComponents} of the model object
     is called on the model object when the model is modified.

     By \code{returnLink} set to TRUE the functions
     \code{drawModel}, \code{redrawView}, and \code{update}, and variables 
     \code{top}, \code{canvas}, \code{viewLabel}, and \code{tags}
     are returned in the object of the returned value.
     The function \code{update} is needed for DynamicGraph to be able to
     update all views when one view is changed, and the other functions
     and variables are needed for overwriting a view.
     When any of these values are present in the objects, R will save
     some environment of the Tcl/tk windows, and the saved workspace
     might be large.

 }

\seealso{See also \code{\link{DynamicGraph}} and all the other functions
	of this package. 

      An example has been divided on the following 4 manual pages:
      \code{\link{dg.Model-class}} and \code{\link{dg.Test-class}}
      gives an example of a model object with test object.
      The pages of \code{\link{drawModel}} show how the user 
      can add menu items with actions that redraws the graph
      after modification of edges.
      Finally, \code{\link{validVertexClasses}} show how to create
      a new vertex class with a new symbol for drawing the vertex 
      and an item added to the pop up menu of the new vertex class.
      The demo \code{demo(Circle.newClass)} of \code{dynamicGraph}
      % The file \code{".../dynamicGraph/Tests/Circle.newClass.R"} of
      % the example catalog of \code{dynamicGraph}
      will do this example collected from these 4 pages.
}

\examples{
require(tcltk)
require(dynamicGraph)

V.Names <- paste(c("Sex", "Age", "Eye", "FEV", "Hair", "Shosize"),
                 1:6, sep ="/")

V.Types <- c("Discrete", "Ordinal", "Discrete",
             "Continuous", "Discrete", "Continuous")

Vertices <- returnVertexList(V.Names, types = V.Types, color = "red")

From <- c(1, 2, 3, 4, 5, 6)
To   <- c(2, 3, 4, 5, 6, 1)

EdgeList <- vector("list", length(To))
for (j in seq(along = To)) EdgeList[[j]] <- c(From[j], To[j])
Edges <- returnEdgeList(EdgeList, Vertices, color = "black")

Z <- dynamicGraphMain(Vertices, edgeList = Edges, w = 4)
}

\keyword{ dplot }
\keyword{ hplot }
\keyword{ iplot }
\keyword{ dynamic }
\keyword{ graphs }
\keyword{ methods }
\keyword{ multivariate }

\eof
\name{modalDialog}

\alias{modalDialog}

\title{Modal dialog window for returning a text string}

\description{
  Ask for a text string in a pop up window.
}

\usage{
modalDialog(title, question, entryInit, top = NULL, entryWidth = 20, 
            returnValOnCancel = "ID_CANCEL", do.grab = FALSE)
}

% Modal Dialog Boxes

% A modal dialog box requires the user to respond to it before changing the focus to other windows within the application. The Tk command tk_dialog is designed for this purpose, and can be called in R TclTk, using the tkdialog function. However, the method illustrated below will use the tktoplevel function and call tkgrab.set and tkgrab.release explicitly, rather than relying on tkdialog to call them automatically.

% Modal Dialog Box Example

% The following example defines a subroutine which creates a simple modal dialog box using tktoplevel to create the dialog window, and using tkgrab.set and tkgrab.release to force the user to respond to this dialog before changing focus to other window(s) in the applciation.


\arguments{
  \item{title}{ Text string for the title bar of the appering window. }
  \item{question}{ Text string for the \code{question}. }
  \item{entryInit}{ Default value of answer. }
  \item{top}{ Text string for the \code{TclTk} \code{top}. }
  \item{entryWidth}{ Integer for the \code{entryWidth}. }
  \item{returnValOnCancel}{ Text
        string for the returned value on \code{Cancel}. }
% \item{graphWindow}{ \code{graphWindow} can be given to give the \code{top}. }
  \item{do.grab}{ Logical. 
        \code{tkgrab.set} resulted in fail for some systems. }
}

\value{
    The text string entered, or \code{returnValOnCancel}.
}

\references{\url{http://bioinf.wehi.edu.au/~wettenhall/RTclTkExamples/modalDialog.html}}

\author{From the examples compiled by James Wettenhall.}

\examples{
Menus <- 
 list(MainUser = 
      list(label = "Test of user drag down menu - Position of \"vertices\"",
           command = function(object, ...) 
             print(Positions(list(...)$Arguments$vertexList))),
      MainUser = 
      list(label = "Test of user drag down menu - modalDialog",
           command = function(object, ...) {
             Args <- list(...)$Arguments
             ReturnVal <- modalDialog("Test modalDialog Entry", "Enter name",
                                      Args$title, graphWindow = Args$graphWindow)
             print(ReturnVal)
             if (ReturnVal == "ID_CANCEL")
               return() } )
     )
}

\keyword{ dynamic }


\eof
\name{nameToVertexIndex}

\alias{nameToVertexIndex}

\title{ The indices of vertices }

\description{
  For each name, find in a list of vertices the index of the vertex with that name.
}

\usage{
nameToVertexIndex(vertexnames, vertices)
}

\arguments{
  \item{vertexnames}{ Vector of text strings of the \code{vertexnames}
      of the vertices for which the indices should be found.  }
  \item{vertices}{ A list of \code{vertices},
                   of each of class containing the class \code{dg.Node}. }
}

\value{
  Integer vector with the indices of the vertices.
}

\author{Jens Henrik Badsberg}

\examples{
Names <- c("Sex", "Age", "Eye", "FEV", "Hair", "Shosize")
Types <- rep("Discrete", 6)
vertices <- returnVertexList(Names, types = Types)
nameToVertexIndex(c("Sex", "Eye"), vertices)
}

\keyword{ dynamic }
\keyword{ graphs }

\eof
\name{returnBlockEdgeList}

\alias{blockEdgeList}
\alias{returnBlockEdgeList}

\alias{dg.BlockEdgeList-class}

\title{Class "dg.BlockEdgeList": The block edge list }

\description{
 Return a list of block edges, each of class \code{dg.BlockEdge}.

 Objects can be created by calls of the form \code{new("dg.BlockEdgeList", ...)}.
}

\usage{
returnBlockEdgeList(edge.list, vertices, blocks, 
                    visibleBlocks = 1:length(blocks), width = 2, 
                    color = "default", oriented = FALSE, type = NULL) 
}

\arguments{
  \item{edge.list}{ A list of vectors identifying the edges (between vertices).
                    Each vector of \code{edge.list} should be a vector of 
                    integers giving the indices of the vertices of an edge, or
                    a vector of text strings with the names of the vertices. }
  \item{vertices}{ The list of \code{vertices}, 
      each of a class containing \code{dg.Vertex}. }
  \item{blocks}{ The list of \code{blocks}, 
      each of a class \code{dg.Block}. }
  \item{visibleBlocks}{ A numeric vector with the indices of the
			\code{visibleBlocks}. The argument is for
			view where some blocks are not visible in the view,
			but the vertices of the blocks are drawn. }
  \item{width}{ A numeric with the initial \code{width} of block edges. }
  \item{color}{
      "default", or list with one or two text strings for colors 
      giving the initial \code{color} of block edges.
      The two colors are used for respectively edges
      between blocks and for edges between blocks 
      and vertices.
    }
  \item{oriented}{ Logical, if TRUE then the edges are \code{oriented}. }
  \item{type}{ A text string giving the \code{type} of block edges. }
}

\value{
    A list of block edges, each of class \code{dg.BlockEdge}.
}

\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"list"}. }
  }
}

\section{Extends}{
Class \code{"dg.EdgeList"}, directly.
Class \code{"dg.NodeList"}, by class \code{"dg.EdgeList"}.
Class \code{"dg.list"}, by class \code{"dg.EdgeList"}.
Class \code{"list"}, by class \code{"dg.EdgeList"}.
}

\section{Methods}{
No methods defined with class "dg.BlockEdgeList" in the signature.
}

\author{Jens Henrik Badsberg}

\note{ 
    The methods of the edge list, \code{\link{returnEdgeList}},
    also applies for block edge lists.
 }

\examples{
Block.tree <- list(label = "W", Vertices = c("country"),
                   X = list(Vertices = c("sex", "race"),
                            A = list(Vertices = c("hair", "eye"),
                                     horizontal = FALSE),
                            B = list(Vertices = c("age"),
                                     C = list(Vertices = c("education")))))
Names <- unlist(Block.tree)
Names <- Names[grep("Vertices", names(Names))]
Types <- rep("Discrete", length(Names))
vertices <- returnVertexList(Names, types = Types)
blocktree <- setTreeBlocks(Block.tree, vertices)
blocks <- blockTreeToList(blocktree$BlockTree)
from <- c("country", "country", "race", "race",       "sex", "sex")
to   <- c(    "sex",    "race", "hair",  "eye", "education", "age")
from <- match(from, Names)
to <- match(to, Names)
edge.list <- vector("list", length(to))
for (j in seq(along = to)) edge.list[[j]] <- c(from[j], to[j])
edges <- returnEdgeList(edge.list, vertices, color = "red", oriented = TRUE)
vertices <- blocktree$Vertices
blockedges <- returnBlockEdgeList(edge.list, vertices, blocks,
                                  color = "red", oriented = TRUE)

Names(blockedges)
Colors(blockedges)
Labels(blockedges)
LabelPositions(blockedges)
# Positions(blockedges)
# Strata(blockedges)
# Indices(blockedges)
str(NodeTypes(blockedges))
str(NodeIndices(blockedges))
Widths(blockedges)
Oriented(blockedges)
Widths(blockedges) <- rep(1,  7)
Widths(blockedges) <- rep(1, 14)
Widths(blockedges)
asDataFrame(blockedges)
}

\keyword{ methods }
\keyword{ dynamic }
\keyword{ graphs }

\eof
\name{returnEdgeList}

\alias{vertexEdgeList}
\alias{returnEdgeList}

\alias{dg.VertexEdgeList-class}

% \alias{NodeTypes}
% % \alias{NodeTypes<-}
% \alias{NodeIndices}
% % \alias{NodeIndices<-}
% \alias{Widths}
% \alias{Widths<-}
% \alias{Dashes}
% \alias{Dashes<-}
% \alias{Oriented}
% \alias{Oriented<-}

% \alias{NodeIndices,dg.list-method}
% \alias{NodeTypes,dg.list-method}
% \alias{Widths,dg.list-method}
% \alias{Widths<-,dg.list-method}
% \alias{Dashes,dg.list-method}
% \alias{Dashes<-,dg.list-method}
% \alias{Oriented,dg.list-method}
% \alias{Oriented<-,dg.list-method}

\title{Class "dg.VertexEdgeList": The edge list }
\description{
 Return a list of edges, each of class containing \code{dg.VertexEdge}.

 Objects can be created by calls of the form \code{new("dg.EdgeList", ...)}.
}
\usage{
returnEdgeList(edge.list, vertices, width = 2, color = "DarkSlateGrey", N = 3, 
               oriented = FALSE, types = NULL, edgeClasses = validEdgeClasses())
}
\arguments{
  \item{edge.list}{ A list of vectors identifying the edges.
                    Each vector of \code{edge.list} should be a vector of 
                    integers giving the indices of the vertices of an edge, or 
                    a vector of text strings with the names of the vertices.}
  \item{vertices}{ The list of \code{vertices}, 
      each of a class containing \code{dg.Vertex}.
     \code{vertices} are used to set the initial labels of the edges.}
  \item{width}{ A single numeric with the initial \code{width} of the edges. }
  \item{color}{ A single text string giving the \code{color} of the edges. }
  \item{oriented}{ Logical, if TRUE then the edges are \code{oriented}. }
  \item{types}{ A vector of text strings giving the \code{types} of the edges,
     identify which classes the edges should be of,
     containing the \code{dg.VertexEdge}.
     } 
  \item{N}{ Integer, \code{N} is the number of coordinates of the vertices. }
  \item{edgeClasses}{ Returned value from \code{\link{validEdgeClasses}},
                      or extension of this matrix. }
}

\value{
     A list of edges, each of class containing \code{dg.VertexEdge}.
}

\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"list"}. }
  }
}

\section{Extends}{
Class \code{"dg.EdgeList"}, directly.
Class \code{"dg.NodeList"}, by class \code{"dg.EdgeList"}.
Class \code{"dg.list"}, by class \code{"dg.EdgeList"}.
Class \code{"list"}, by class \code{"dg.EdgeList"}.
}

\section{Methods}{
No methods defined with class "dg.VertexEdgeList" in the signature.
}

\author{Jens Henrik Badsberg}

\note{
    Beside the methods of the vertex list, \code{\link{vertexList}},
    (except \code{\link{Positions}}, \code{\link{Indices}}
    and \code{\link{Strata}}) the edge list also has the methods
    \code{\link{NodeTypes}},
    % \code{\link{NodeTypes<-}},
    \code{\link{NodeIndices}},
    % \code{\link{NodeIndices<-}},
    \code{\link{Widths}},
    \code{\link{Widths<-}},
    \code{\link{Dashes}},
    \code{\link{Dashes<-}},
    \code{\link{Oriented}}, and
    \code{\link{Oriented<-}}. 
}

\seealso{\code{\link{vertexList}} and \code{\link{newVertexEdge}}.}

\examples{
from <- c("contry", "contry", "race", "race",       "sex",  "sex")
to   <- c(   "sex",   "race", "hair",  "eye", "education",  "age")
vertexnames <- unique(sort(c(from, to)))
vertices <- returnVertexList(vertexnames)
# from <- match(from, vertexnames)
# to <- match(to, vertexnames)
edge.list <- vector("list", length(to))
for (j in seq(along = to)) edge.list[[j]] <- c(from[j], to[j])
edges <- returnEdgeList(edge.list, vertices, color = "red", oriented = TRUE)

Names(edges)
Colors(edges)
Labels(edges)
LabelPositions(edges)
# Positions(edges)
# Strata(edges)
# Indices(edges)
str(NodeTypes(edges))
str(NodeIndices(edges))
Dashes(edges)
Widths(edges)
Oriented(edges)
Widths(edges) <- rep(1, 7)
Widths(edges) <- rep(1, 6)
Widths(edges)
asDataFrame(edges)
}

\keyword{ methods }
\keyword{ dynamic }
\keyword{ graphs }

\eof
\name{returnExtraEdgeList}

\alias{extraEdgeList}
\alias{returnExtraEdgeList}

\alias{dg.ExtraEdgeList-class}

\title{Class "dg.ExtraEdgeList": The extra edge list }

\description{
 Return a list of extra edges, each of class \code{dg.ExtraEdge}.

 Objects can be created by calls of the form
 \code{new("dg.ExtraEdgeList", ...)}.
}

\usage{
returnExtraEdgeList(edge.list, vertices, extravertices = NULL,
                    width = 2, color = "DarkSlateGrey", type = NULL)
}

\arguments{
  \item{edge.list}{ A list of vectors identifying the edges (between vertices).
                    Each vector of \code{edge.list} should be a vector of 
                    integers giving the indices of the vertices of an edge,
		    positive for vertices, negative for extra vertices, or
                    a vector of text strings with the names of the vertices. }
  \item{vertices}{ The list of \code{vertices}, 
      each of a class containing \code{dg.Vertex}. }
  \item{extravertices}{ The list of \code{extravertices}, 
      each of a class containing \code{dg.ExtraVertex}. }
  \item{width}{ A numeric with the initial \code{width} of the extra edges. }
  \item{color}{ A text string giving the initial \code{color}
        of the extra edges. }
  % \item{oriented}{ Logical, if TRUE then the edges are \code{oriented}. }
  \item{type}{ A text string giving the \code{type} of the extra edges. }
}

\value{
    A list of edges, each of class \code{dg.ExtraEdge}.
}

\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"list"}. }
  }
}

\section{Extends}{
Class \code{"dg.EdgeList"}, directly.
Class \code{"dg.NodeList"}, by class \code{"dg.EdgeList"}.
Class \code{"dg.list"}, by class \code{"dg.EdgeList"}.
Class \code{"list"}, by class \code{"dg.EdgeList"}.
}

\section{Methods}{
No methods defined with class "dg.ExtraEdgeList" in the signature.
}

\author{Jens Henrik Badsberg}

\note{ 
    The methods of the edge list, \code{\link{returnEdgeList}},
    also applies for extra edge lists.

    The application is similar to \code{\link{returnBlockEdgeList}},
    see example here.
 }

\seealso{
\code{\link{returnFactorEdgeList}}
 and \code{\link{returnBlockEdgeList}}. }

\keyword{ methods }
\keyword{ dynamic }
\keyword{ graphs }

\eof
\name{returnFactorEdgeList}

\alias{factorEdgeList}
\alias{returnFactorEdgeList}

\alias{dg.FactorEdgeList-class}

\title{Class "dg.FactorEdgeList": The factor edge list }

\description{
 Return a list of factor edges, each of class \code{dg.FactorEdge}.

 Objects can be created by calls of the form
 \code{new("dg.FactorEdgeList", ...)}.
}

\usage{
returnFactorEdgeList(edge.list, vertices, factorvertices = NULL,
                     width = 2, color = "DarkSlateGrey",
                     % oriented = FALSE,
                     type = NULL)
}

\arguments{
  \item{edge.list}{ A list of vectors identifying the edges (between vertices).
                    Each vector of \code{edge.list} should be a vector of 
                    integers giving the indices of the vertices of an edge, or
                    a vector of text strings with the names of the vertices. }
  \item{vertices}{ The list of \code{vertices}, 
      each of a class containing \code{dg.Vertex}. }
  \item{factorvertices}{ The list of \code{factorvertices}, 
      each of a class containing \code{dg.FactorVertex}. }
  \item{width}{ A numeric with the initial \code{width} of the factor edges. }
  \item{color}{ A text string giving the initial \code{color}
        of the factor edges. }
  % \item{oriented}{ Logical, if TRUE then the edges are \code{oriented}. }
  \item{type}{ A text string giving the \code{type} of the factor edges. }
}

\value{
    A list of edges, each of class \code{dg.FactorEdge}.
}

\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"list"}. }
  }
}

\section{Extends}{
Class \code{"dg.EdgeList"}, directly.
Class \code{"dg.NodeList"}, by class \code{"dg.EdgeList"}.
Class \code{"dg.list"}, by class \code{"dg.EdgeList"}.
Class \code{"list"}, by class \code{"dg.EdgeList"}.
}

\section{Methods}{
No methods defined with class "dg.FactorEdgeList" in the signature.
}

\author{Jens Henrik Badsberg}

\note{ 
    The methods of the edge list, \code{\link{returnEdgeList}},
    also applies for factor edge lists.

    No example is given here since the factor edge list usually
    will be returned by the function 
    \code{\link{returnFactorVerticesAndEdges}}.
    The application is similar to \code{\link{returnBlockEdgeList}},
    see example here.

 }

\seealso{
\code{\link{returnFactorVerticesAndEdges}}
 and \code{\link{returnBlockEdgeList}}. }

\keyword{ methods }
\keyword{ dynamic }
\keyword{ graphs }

\eof
\name{returnFactorVerticesAndEdges}

\alias{factorVertexList}
\alias{returnFactorVerticesAndEdges}

\alias{dg.FactorVertexList-class}

\title{Class "dg.FactorVertexList": The factor vertex list}

\description{
 Create factor vertex and factor edge lists.

 Objects can be created by calls of the form
 \code{new("dg.FactorVertexList", ...)}.
}

\usage{
returnFactorVerticesAndEdges(Vertices, factors = NULL, types = "Generator", 
                             factorVertexColor = "default", 
                             factorEdgeColor = "DarkOliveGreen", 
                             % oriented = FALSE, 
                             factorClasses = validFactorClasses())
}

\arguments{
  \item{Vertices}{ The list of \code{Vertices}, each
      containing the class \code{dg.Vertex}. }
  \item{factors}{ The list of vectors identifying the \code{factors}.
          Each item in the list is a vector of the indices of
	  vertices of a factor. 
      }
  \item{types}{ The \code{types} of the factors. Either a single
      \code{type} or a list of the same length as \code{factors}.
      Each item of \code{types} should match the labels of
      \code{\link{factorClasses}}, and is used to set the class of
      the factor vertex.
  }
  \item{factorVertexColor}{ The \code{factorVertexColor}
      of the factor vertices. }
  \item{factorEdgeColor}{ The \code{factorEdgeColor} of the factor edges. }
  % \item{oriented}{ Logical, if TRUE then the edges are \code{oriented}. }
  \item{factorClasses}{ The valid \code{\link{factorClasses}}. }
}

\details{
      The argument \code{factors} is a list of vectors identifying 
      the \code{factors}, or \code{generators}.
      Each item in the list is a vector with of the indices (or names)
      of the vertices of a factor, or variables of a generator.
      A factor vertex is made for each factor, and factor edges
      from this factor vertex to the vertices of the factor
      or added to the factor edge list.
      Also the edges between pairs of the vertices in the factors
      are returned.
}

\value{
  A list with components
  \item{FactorVertices}{The list of factor vertices, each of class containing
        \code{\link{dg.FactorVertex}}. }
  \item{FactorEdges}{The list of factor edge, each of class containing
        \code{\link{dg.FactorEdge}}.}
  \item{PairEdges}{A matrix with the edges of the graph,
        two columns with the indices of the vertices of two ends 
        of the edges. }
}

\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"list"}. }
  }
}

\section{Extends}{
Class \code{"dg.NodeList"}, directly.
Class \code{"dg.list"}, by class \code{"dg.NodeList"}.
Class \code{"list"}, by class \code{"dg.NodeList"}.
}

\section{Methods}{
No methods defined with class "dg.FactorVertexList" in the signature.
}

\author{Jens Henrik Badsberg}

\note{ 
    The methods of the vertex list, \code{\link{returnVertexList}},
    also applies for factor lists,
    and the methods of the edge list, \code{\link{returnEdgeList}},
    also applies for factor edge lists.

    Your \code{\link{modifyModel}} should compute the new factors,
    generators, when modifying the model. See \code{\link{dg.Model-class}}
    for examples on use of \code{returnFactorVerticesAndEdges}.

  % By default only the new factors are computed when a vertex is deleted.
  % Adding and dropping edges requires finding the cliques of the graph
  % from the edges of the graph. Finding the cliques is out of the scope
  % of  \code{\link{dynamicGraph}}.
 }

% \examples{ }

\keyword{ methods }
\keyword{ dynamic }
\keyword{ graphs }

\eof
\name{returnVertexList}

\alias{vertexList}
\alias{returnVertexList}

\alias{dg.VertexList-class}

% \alias{Names}
% \alias{Names<-}
% \alias{Colors}
% \alias{Colors<-}
% \alias{Labels}
% \alias{Labels<-}
% \alias{LabelPositions}
% \alias{LabelPositions<-}
% \alias{Positions}
% \alias{Positions<-}
% \alias{Strata}
% \alias{Strata<-}
% \alias{Blockindices}
% \alias{Blockindices<-}
% \alias{Indices}
% \alias{asDataFrame}
% % \alias{Indices<-}
% \alias{Visible}
% \alias{Visible<-}

% \alias{dg.list-class}

% \alias{show,dg.list-method}
% \alias{Names,dg.list-method}
% \alias{Names<-,dg.list-method}
% \alias{Colors,dg.list-method}
% \alias{Colors<-,dg.list-method}
% \alias{Labels,dg.list-method}
% \alias{Labels<-,dg.list-method}
% \alias{LabelPositions,dg.list-method}
% \alias{LabelPositions<-,dg.list-method}
% \alias{Positions,dg.list-method}
% \alias{Positions<-,dg.list-method}
% \alias{Strata,dg.list-method}
% \alias{Strata<-,dg.list-method}
% \alias{Blockindices,dg.list-method}
% \alias{Blockindices<-,dg.list-method}
% \alias{Indices,dg.list-method}
% \alias{asDataFrame,dg.list-method}
% \alias{Visible,dg.list-method}
% \alias{Visible<-,dg.list-method}

\title{Class "dg.VertexList": The vertex list }

\description{
  Return a list of vertices of classes containing the class \code{dg.Vertex}.

 Objects can be created by calls of the form \code{new("dg.VertexList", ...)}.
}

\usage{
returnVertexList(names, labels = NULL, types = NULL, 
                 strata = NULL, line = FALSE, N = 3, 
                 colors = ifelse(types == "TextVertex", 
                                 "FloralWhite", "DarkRed"), 
                 vertexClasses = validVertexClasses())
}

\arguments{
  \item{names}{ Vector of text strings with the \code{names} of the vertices. }
  \item{labels}{ Vector of text strings with the \code{labels} of the vertices. }
  \item{types}{ Vector of text strings with the \code{types} of the vertices. }
  \item{strata}{ Vector of integers with the \code{strata} of the vertices. }
  \item{line}{ Logical, if TRUE then the vertices are positioned on a
               \code{line}, else in a regular polygone, in a circle. }
  \item{N}{ Integer, \code{N} is the number of coordinates of the vertices. }
  \item{colors}{ Vector of text strings with \code{colors} of the vertices. }
  \item{vertexClasses}{ The valid \code{\link{vertexClasses}}. }
}

\value{
 A list of vertices of classes containing the class \code{dg.Vertex}.
}

\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"list"}. }
  }
}

\section{Extends}{
Class \code{"dg.NodeList"}, directly.
Class \code{"dg.list"}, by class \code{"dg.NodeList"}.
Class \code{"list"}, by class \code{"dg.NodeList"}.
}

\section{Methods}{
No methods defined with class "dg.VertexList" in the signature.
}

\author{Jens Henrik Badsberg}

\note{The methods
      \code{\link{Names}}, 
      \code{\link{Names<-}}, 
      \code{\link{Colors}}, 
      \code{\link{Colors<-}}, 
      \code{\link{Labels}}, 
      \code{\link{Labels<-}}, 
      \code{\link{LabelPositions}}, 
      \code{\link{LabelPositions<-}}, 
      \code{\link{Positions}}, 
      \code{\link{Positions<-}}, 
      \code{\link{Strata}}, 
      \code{\link{Strata<-}},
      \code{\link{Indices}}, and 
      \code{\link{asDataFrame}} 
      are available for vertex lists.
      % \code{\link{Indices<-}}, 
      % \code{\link{Visible}}, 
      % \code{\link{Visible<-}}, 
}

\examples{
vertices <- returnVertexList(c("A", "B", "C", "D"),
                             labels = c("OrdinalVertex", "TextVertex", 
                               "ContinuousVertex", "DiscreteVertex"), 
                             types = c("Ordinal", "TextVertex",
                                       "Continuous", "Discrete"), N = 2)
Names(vertices)
Colors(vertices)
Labels(vertices)
LabelPositions(vertices)
Positions(vertices)
Strata(vertices)
Indices(vertices)
Names(vertices) <- c("a", "b", "c", "d")
Colors(vertices) <- rep("Blue", 4)
Labels(vertices) <- c("A", "B", "C", "D")
LabelPositions(vertices) <- matrix(rep(0, 12), ncol = 3)
Positions(vertices) <- matrix(rep(0, 12), ncol = 3)
Strata(vertices) <- rep(1, 4)
Names(vertices)
Colors(vertices)
Labels(vertices)
LabelPositions(vertices)
Positions(vertices)
Strata(vertices)
Indices(vertices)
asDataFrame(vertices)
}

\keyword{ methods }
\keyword{ dynamic }
\keyword{ graphs }

\eof
\name{selectDialog}
\alias{selectDialog}

\title{Dialog window for selection between items}

\description{
Dialog window for selection between items.
}

\usage{
selectDialog(title = "Selection entry", question = "Select item",
             itemNames, top = NULL, returnValOnCancel = "ID_CANCEL", 
             do.grab = FALSE)
}

\arguments{
  \item{title}{ Text string for the title bar of the appering window. }
  \item{question}{ Text string for the \code{question}. }
  \item{itemNames}{ Default value of answer. }
  \item{top}{ Text string for the \code{TclTk} \code{top}. }
% \item{entryWidth}{ Integer for the \code{entryWidth}. }
  \item{returnValOnCancel}{ Text
        string for the returned value on \code{Cancel}. }
% \item{graphWindow}{ \code{graphWindow} can be given to give the \code{top}. }
  \item{do.grab}{ Logical. 
        \code{tkgrab.set} resulted in fail for some systems. }
}

\value{
    An integer, or \code{returnValOnCancel}.
}

\references{\url{http://bioinf.wehi.edu.au/~wettenhall/RTclTkExamples/modalDialog.html}}

\author{From the examples compiled by James Wettenhall.}

\seealso{ \code{\link{modalDialog}}. }

% \examples{ }

\keyword{ dynamic }


\eof
\name{setBlocks}

\title{Class "dg.BlockList": The block list}

\alias{setBlocks}
\alias{blockList}

\alias{dg.BlockList-class}

\description{
 Create a block list with positioning the vertices in the blocks.    

 Objects can be created by calls of the form \code{new("dg.BlockList", ...)}.
}

\usage{
setBlocks(block.list, vertices, labels = NULL, 
          right.to.left = FALSE, nested.blocks = FALSE,
          % draw.blocks = TRUE,
          blockColors = NULL, color = "Grey", N = 3)
}

\arguments{
  \item{block.list}{ The list of vectors identifying the \code{blocks}.
          Each item in the list is a vector with of the indices (or text
	  strings for names) of vertices of a block. 
      }
  \item{vertices}{ The list of \code{vertices}, each
      containing the class \code{dg.Vertex}. Returned with positions
      set in the interval of the blocks. }
  \item{labels}{ List of text strings with the \code{labels} of the
      blocks. If \code{labels} is set to \code{NULL} then labels are
      found as \code{names(block.list)}.
  }
  \item{right.to.left}{ Logical. If \code{right.to.left} is set to TRUE
        then the explanatory blocks are drawn to the right.
        }
  \item{nested.blocks}{ Logical. If \code{nested.blocks} then the
        blocks are drawn nested. }
  % \item{draw.blocks}{ ~~Describe \code{draw.blocks} here~~ }
  \item{blockColors}{ Vector of text string with the \code{blockColors}
        of the blocks. }
  \item{color}{ Single text string with \code{color} of the blocks.
    Only used when \code{blockColors} is not given. }
  \item{N}{ Integer, \code{N} is the number of coordinates of the
      vertices and block corners. }
}

\value{
  A list with components
  \item{Blocks}{The list of blocks, each of class 
        \code{\link{dg.Block}}.}
  \item{Vertices}{The list of vertices,
        with the positions updated such the vertices
        has positions within the blocks. }
}

\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"list"}. }
  }
}

\section{Extends}{
Class \code{"dg.NodeList"}, directly.
Class \code{"dg.list"}, by class \code{"dg.NodeList"}.
Class \code{"list"}, by class \code{"dg.NodeList"}.
}

\section{Methods}{
No methods defined with class "dg.BlockList" in the signature.
}

\author{Jens Henrik Badsberg}

\examples{
require(tcltk)

require(dynamicGraph)

V.Types <- c("Discrete", "Ordinal", "Discrete",
             "Continuous", "Discrete", "Continuous")

V.Names <- c("Sex", "Age", "Eye", "FEV", "Hair", "Shosize")
V.Names <- paste(V.Names, 1:6, sep ="/")

From <- c(1, 2, 3, 4, 5, 6)
To   <- c(2, 3, 4, 5, 6, 1)

# A block recursive model:

Blocks <- list(Basic = c(2, 1), Intermediate = c(5, 4, 3),  Now = c(6))

V.Names <- paste(V.Names, c(1, 1, 2, 2, 2, 3), sep =":")

Z <- DynamicGraph(V.Names, V.Types, From, To, blocks = Blocks,
                  width = 600, height = 600, drawblocks = TRUE,
                  drawBlockBackground = FALSE, title = "DrawBlocks")

# A block recursiv model, without drawing blocks:

Z <- DynamicGraph(V.Names, V.Types, From, To, blocks = Blocks,
                  width = 600, height = 600, 
                  drawblocks = FALSE, title = "No blocks drawn")

# A block recursive model with nested blocks:

Z <- DynamicGraph(V.Names, V.Types, From, To, blocks = Blocks,
                  width = 600, height = 600, drawblocks = TRUE, 
                  nested.blocks = TRUE, title = "Nested blocks",
                  blockColors = paste("Grey", 100 - 2 * (1:10), sep = ""))

# The block list of the last example:

vertices <- returnVertexList(V.Names, types = V.Types)
blockList <- setBlocks(Blocks, vertices = vertices,  nested.blocks = TRUE,
                       blockColors = paste("Grey", 100 - 2 * (1:10), sep = ""))

names(blockList)
str(blockList$Blocks[[1]])

names(blockList$Blocks)
Names(blockList$Blocks)
Labels(blockList$Blocks)
LabelPositions(blockList$Blocks)
Positions(blockList$Blocks)
Strata(blockList$Blocks)
Colors(blockList$Blocks)
NodeAncestors(blockList$Blocks)
NodeDescendants(blockList$Blocks)
Visible(blockList$Blocks)
Indices(blockList$Blocks)

names(blockList$Vertices)
Names(blockList$Vertices)
Labels(blockList$Vertices)
LabelPositions(blockList$Vertices)
Positions(blockList$Vertices)
Strata(blockList$Vertices)
Colors(blockList$Vertices)
Indices(blockList$Vertices)

asDataFrame(blockList$Vertices)
asDataFrame(blockList$Blocks)

}

\keyword{ methods }
\keyword{ dynamic }
\keyword{ graphs }

\eof
\name{setTreeBlocks}

\alias{setTreeBlocks}
\alias{Closed}
\alias{Closed<-}
\alias{NodeAncestors}
\alias{NodeAncestors<-}
\alias{NodeDescendants}
\alias{NodeDescendants<-}

% \alias{Closed,dg.list-method}
% \alias{Closed<-,dg.list-method}
% \alias{NodeAncestors,dg.list-method}
% \alias{NodeAncestors<-,dg.list-method}
% \alias{NodeDescendants,dg.list-method}
% \alias{NodeDescendants<-,dg.list-method}

\title{The block tree}

\description{
    Create a block tree with positioning the vertices in to blocks.
}

\usage{
setTreeBlocks(block.tree, vertices, root.label = "", N = 3,
              delta = ifelse(overlaying, 1, 0), 
              Delta = ifelse(overlaying, 0, 1.5),
              d = 5, f = 1/4, blockColors = NULL, overlaying = FALSE)
}

%  \itemize{
     % \item{...$Vertices}{ ~~Describe \code{Vertices} here~~ }
     % \item{...$label}{ ~~Describe \code{label} here~~ }
     % \item{...$d}{ ~~Describe \code{d} here~~ }
     % \item{...$g}{ ~~Describe \code{g} here~~ }
     % \item{...$f}{ ~~Describe \code{f} here~~ }
     % \item{...$G}{ ~~Describe \code{G} here~~ }
     % \item{...$F}{ ~~Describe \code{F} here~~ }
     % \item{...$horizontal}{ ~~Describe \code{horizontal} here~~ }
     % \item{...$vertices.last}{ ~~Describe \code{vertices.last} here~~ }
     % \item{...$"block-name" = list(...) }{
     %                ~~Describe \code{"block-name" = list(...)} here~~ }
     % \item{...$common.children}{ ~~Describe \code{common.children}
     %                              here~~ }
%          }

\arguments{
  \item{block.tree}{ A structure with the blocks in a \code{block.tree}.
	See below. }
  \item{vertices}{ The list of \code{vertices}, each
      containing the class \code{dg.Vertex}. Returned with positions
      set in the interval of the blocks. }
  \item{root.label}{ A text string with the \code{root.label}
      of the root block. }
  \item{N}{ Integer, \code{N} is the number of coordinates of the
      vertices and block corners. }
  \item{delta}{ Numeric. Decrement of block size for nested blocks,
	        and space between blocks when \code{overlaying} is TRUE.
                The decrement is  \code{delta} divided by 100, times
                the the size of the window canvas, width or height. } 
  \item{Delta}{ Numeric. Decrement of block size for nested blocks,
	        and space between blocks when \code{overlaying} is FALSE. 
                The decrement is \code{Delta} divided by 100, times
                the the size of the window canvas, width or height. }
  \item{d}{ Numeric. If not \code{d} is given in \code{block.tree}, see below:
            The heading bar (with the label) has a height of
            (\code{d} + 2) divided by 100,
            times height of the window canvas. }
  \item{f}{ Numeric. If not \code{f} or \code{g} is given in \code{block.tree},
            see below:
            The the vertices of the block are placed in an array with a height
            (width if \code{horizontal} is set to FALSE)
            of \code{f} divided by 100,
            times height (width) of the block. 
            Thus this size is relative to the block size. }
  \item{blockColors}{ Vector of text string with the \code{blockColors}
        of the blocks. }
  \item{overlaying}{ Logical. If \code{overlaying} is set to FALSE
        then children blocks of a block are not drawn inside the block. }
}

\details{ 

  A recursive definition:
  \code{Block.tree} is a list with the vertices of the "current" blocks,
  some parameters for controlling the layout,
  and possible some \code{block.trees}:

  \itemize{
      \item{\code{...\$Vertices}}{ The \code{vertices} of the block. }
      \item{\code{...\$label}}{ A text string for
            the \code{label} of the block. 
	    Will overwrite \code{"block-name"} and  \code{root.label}. }
      \item{\code{...\$d}}{ Numeric.
            The heading bar (with the label) has a height of
            (\code{d} + 2) divided by 100,
            times the height of the window canvas. }
      \item{\code{...\$g}}{ Numeric.
            The vertices of the block are placed in an array with a height 
            (width if \code{horizontal} is set to FALSE)
            of \code{g} divided by 100,
            times the height (width) of the window canvas. 
            Thus this size will not decrease with the block size.
            }
      \item{\code{...\$f}}{ Numeric.
            If not \code{g} is given:
            The the vertices of the block are placed in an array with a height
            (width if \code{horizontal} is set to FALSE)
            of \code{f} divided by 100,
            times the height (width) of the block. 
            Thus this size is relative to the block size. }
      \item{\code{...\$G}}{ Numeric.
            (If the height of the block is 100 we are now left with 
             \code{100 - 2 * delta - d - 2 - g} for the blocks. )
            The sub blocks (apart from \code{common.children})
            then have a of height 
            (width, if \code{horizontal} is set to FALSE)
            of \code{G} divided by 100,
            times the height (width) of the window canvas. 
            Thus the sub block size will not decrease with the block size.
            (If the height of the block is 100 we are now left with 
             \code{100 - 2 * delta - d - 2 - g - G}
             for the \code{common.children}. )
            }
      \item{\code{...\$F}}{ Numeric.
            If not \code{G} is given:
            The proportion \code{G} of the remaining space 
            are used for sub blocks (apart from \code{common.children})
            and the proportion \code{G} of the space for blocks
            are used for \code{common.children}.
            }
      \item{\code{...\$horizontal}}{ Logical.
            If \code{horizontal} is set to TRUE,
            then the sub blocks, but \code{common.children},
            are placed side by side,
	    else the blocks are placed vertically. }
      \item{\code{...\$closed}}{ Logical.
            If \code{closed} is set to TRUE,
            then the block is initially drawn "closed",
            and the vertices and sub blocks of the block are not visible. }
      \item{\code{...\$vertices.last}}{  Logical. If \code{vertices.last} then
	    the vertices of the block are placed after the sub blocks. }
      \item{\code{...\$"block-name"}}{\code{= list(...)}
                    Repeated zero, one or more times for sub blocks.
                    \code{"block-name"} is the label of the block,
                    and \code{list(...)} is a \code{Block.tree}. }
      \item{\code{...\$common.children}}{\code{= list(...)}
                    Omitted, or a list with common children
                    of the other sub blocks of the block. 
                    The list is again a \code{Block.tree}. }
  }
}

\value{
  A list with components
  \item{BlockTree}{A tree of blocks, each of class \code{\link{dg.Block}}. }
  \item{Vertices}{The list of vertices,
        with the positions updated such the vertices
        has positions within the blocks. }
}

\author{Jens Henrik Badsberg}

\note{ 

      Ancestors and descendants are set in \code{\link{setTreeBlocks}}.
      Ancestors are used in the function \code{\link{returnBlockEdgeList}} to
      find the edges between blocks and between blocks and vertices.
      Descendants are used in \code{\link{dynamicGraphMain}}
      when closing, opening and moving blocks,
      and when adding or dropping edges from and to blocks.
      The methods \code{\link{NodeAncestors}}, \code{\link{NodeAncestors<-}}
      \code{\link{NodeDescendants}} and \code{\link{NodeDescendants<-}}
      can be used on the block list resulting of 
      \code{\link{blockTreeToList}} on the block tree.

}

\examples{
# Example 1:

Block.tree <- list(label = "W", Vertices = c("country"),
                   X = list(Vertices = c("race", "sex"),
                            A = list(Vertices = c("hair", "eye"),
                                     horizontal = FALSE),
                            B = list(Vertices = c("education"),
                                     C = list(Vertices = c("age")))))
V.Names <- unlist(Block.tree)
vertices <- returnVertexList(V.Names[grep("Vertices", names(V.Names))])
blocktree <- setTreeBlocks(Block.tree, vertices)

Positions(blockTreeToList(blocktree$BlockTree))
Positions(blocktree$Vertices)
NodeAncestors(blockTreeToList(blocktree$BlockTree))
NodeDescendants(blockTreeToList(blocktree$BlockTree))

vertexStrata <- Strata(blocktree$Vertices)
vertexStrata
vertexNames <- Names(blocktree$Vertices)
names(vertexNames) <- NULL
vertexNames

# Indices of the vertices in blocks:

indicesInBlock <- vector("list", max(vertexStrata))
for (i in seq(along = vertexStrata))
  indicesInBlock[[vertexStrata[i]]] <- 
    append(indicesInBlock[[vertexStrata[i]]], i) 
str(indicesInBlock)

# Names of the vertices in blocks:

vertexNamesInblock <- vector("list", max(vertexStrata))
for (i in seq(along = vertexStrata))
  vertexNamesInblock[[vertexStrata[i]]] <- 
    append(vertexNamesInblock[[vertexStrata[i]]], vertexNames[i]) 
str(vertexNamesInblock)

# A useful function, replace "k" (block index k) 
# in block "i" by "x[k]", the content "x[k]" of block "k":

f <- function(A, x) {
  result <- vector("list", length(A))
  names(result) <- names(A)
  for (i in seq(along = A))
    if ((length(A[[i]]) > 0) && (A[[i]] != 0))
      for (k in A[[i]])
        result[[i]] <- append(result[[i]], x[k])
  return(result)
}

# For each block, names of vertices in ancestor blocks:

vertexAncOfBlock <- f(NodeAncestors(blockTreeToList(blocktree$BlockTree)), 
                      vertexNamesInblock)
str(vertexAncOfBlock)

for (i in seq(along = vertexAncOfBlock))
  if (length(vertexAncOfBlock[[i]]) > 0)
    vertexAncOfBlock[[i]] <- unlist(vertexAncOfBlock[[i]])
str(vertexAncOfBlock)

# For each block, names of vertices in descendant blocks:

vertexDesOfBlock <- f(NodeDescendants(blockTreeToList(blocktree$BlockTree)),
                      vertexNamesInblock)
str(vertexDesOfBlock)

for (i in seq(along = vertexDesOfBlock))
  if (length(vertexDesOfBlock[[i]]) > 0)
    vertexDesOfBlock[[i]] <- unlist(vertexDesOfBlock[[i]])
str(vertexDesOfBlock)


# Example 2:

Block.tree <-
  list(g = 0, G = 54, label = "Pedegree.G",
       Male.Side = 
       list(g = 0, G = 33,
            Father = 
            list(g = 0, G = 12,
                 P.G.Father = list(Vertices = c("P.G.Father.1")),
                 P.G.Mother = list(Vertices = c("P.G.Mother.1")),
                 common.children = list(g = 0, label = "Father.1",
                                        Vertices = c("Father.1"))),
            Mother = 
            list(g = 0, G = 12,
                 M.G.Father = list(Vertices = c("M.G.Father.1")),
                 M.G.Mother = list(Vertices = c("M.G.Mother.1")),
                 common.children = list(g = 0, label = "Mother.1",
                                        Vertices = c("Mother.1"))),
            common.children = list(g = 2, Vertices = c("Male"))),
  Female.Side = list(g = 0, G = 12,
    P.G.Father = list(Vertices = c("P.G.Father.2")),
    P.G.Mother = list(Vertices = c("P.G.Mother.2")),
    M.G.Father = list(Vertices = c("M.G.Father.2")),
    M.G.Mother = list(Vertices = c("M.G.Mother.2")),
    common.children = list(g = 0, G = 12, label = "Female",
      Father = list(Vertices = c("Father.2")),
      Mother = list(Vertices = c("Mother.2")),
      common.children = list(g = 2, Vertices = c("Female")))),
  common.children = list(Vertices = c("Marriage"), g = 3, label = "Children",
    Son = list(Vertices = c("Son"), g = 3, 
       P.G.Son = list(Vertices = c("P.G.Son"), g = 2),
       P.G.Dat = list(Vertices = c("P.G.Dat"), g = 1)),
    Dat = list(Vertices = c("Dat"), g = 2,
       M.G.Son = list(Vertices = c("M.G.Son")),
       M.G.Dat = list(Vertices = c("M.G.Dat")))
    )
)

v <- unlist(Block.tree)
V.Names <- v[grep("Vertices", names(v))]
rm(v)

FromTo <- matrix(c("P.G.Father.1", "Father.1", "P.G.Father.2", "Father.2", 
                   "P.G.Mother.1", "Father.1", "P.G.Mother.2", "Father.2", 
                   "M.G.Father.1", "Mother.1", "M.G.Father.2", "Mother.2", 
                   "M.G.Mother.1", "Mother.1", "M.G.Mother.2", "Mother.2", 
                   "Father.1",     "Male",     "Father.2",     "Female",   
                   "Mother.1",     "Male",     "Mother.2",     "Female",   
                   "Male",         "Marriage", "Female",       "Marriage",
                   "Marriage",     "Son",      "Marriage",     "Dat",
                   "Son",          "P.G.Son",  "Dat",          "M.G.Son", 
                   "Son",          "P.G.Dat",  "Dat",          "M.G.Dat"), 
                   byrow = TRUE, ncol = 2)

From <- match(FromTo[,1], V.Names)
To   <- match(FromTo[,2], V.Names)

V.Types <- rep("Discrete", length(V.Names))

Object <- NULL
Z <- DynamicGraph(V.Names, V.Types, From, To, block.tree = Block.tree, 
                  object = Object, width = 600, height = 600, 
                  drawblocks = TRUE, drawBlockFrame = TRUE, 
                  overlaying = TRUE, title = "Pedegree.G")
}

\keyword{ methods }
\keyword{ dynamic }
\keyword{ graphs }

\eof
\name{validEdgeClasses}

\alias{edgeClasses}
\alias{validEdgeClasses}

\title{Valid edge classes}

\description{Return matrix with labels of valid edge classes
and the valid edge classes}

\usage{validEdgeClasses()}

\details{The argument \code{edgeClasses} to
    \code{\link{dynamicGraphMain}} and
    \code{\link{DynamicGraph}}, and to
    \code{\link{newVertexEdge}} and
    \code{\link{returnEdgeList}}
    is by default the returned value of this
    function. If new edge classes are created then \code{edgeClasses}
    should be set to a value with this returned value extended appropriate. }

\value{Matrix of text strings with labels (used in dialog windows) of valid 
       edge classes and the valid edge classes (used to create the edges).}

\author{Jens Henrik Badsberg}

\note{The "draw" method for an edge should return a list with the items 
      "lines", "tags", "from", "to", label" and "label.position".
      "lines" is the "tk"-objects for line objects between pairs of vertices,
      with coordinates at the vertices.
      "tags" is the "tk" -objects for objects between pairs of vertices,
      with coordinates at the middle of the two vertices.
 }

\seealso{ \code{\link{validVertexClasses}}. }

\examples{
require(tcltk)

# Test with new edge class (demo(Circle.newEdge)):

setClass("NewEdge", contains = "dg.VertexEdge")

myEdgeClasses <- rbind(validEdgeClasses(), 
                       NewEdge = c("NewEdge", "NewEdge"))

setMethod("draw", "NewEdge",
          function(object, canvas, position,
                   x = lapply(position, function(e) e[1]),
                   y = lapply(position, function(e) e[2]),
                   stratum = as.vector(rep(0, length(position)),
                                       mode = "list"), 
                   w = 2, color = "green", background = "white")
          {
            f <- function(i, j) {
              dash <- "."
              arrowhead <- "both"
              l <- function(xi, yi, xj, yj)
                tkcreate(canvas, "line", xi, yi, xj, yj, width = w, 
                         arrow = arrowhead, dash = dash,
                         # arrowshape = as.list(c(2, 5, 3) * w),
                         fill = color(object), activefill = "DarkSlateGray")
              lines <- list(l(x[[i]], y[[i]], x[[j]], y[[j]]))
              label.position <- (position[[i]] + position[[j]]) / 2
              pos <- label.position + rep(0, length(label.position))
              label <- tkcreate(canvas, "text", pos[1], pos[2],
                                text = object@label, anchor = "nw", 
				font = "8x16", activefill = "DarkSlateGray")
              tags <- NULL
              x. <- mean(unlist(x))
              y. <- mean(unlist(y))
              s <- 4 * w * sqrt(4 / pi)
              p <- tkcreate(canvas, "rectangle",
			    x. - s, y. - s, x. + s, y. + s, 
                            fill = color(object), activefill = "SeaGreen")
              tags <- list(p)
              return(list(lines = lines, tags = tags,
                          from = object@vertex.indices[i],
                          to = object@vertex.indices[j],
                          label = label, label.position = label.position))
            }
            result <- NULL
            edge <- object@vertex.indices
            m <- length(edge)
            for (j in seq(along = edge))
              if (j < length(edge))
                for (k in (j+1):length(edge))
                    result <- append(result, list(f(j, k)))
            return(result) 
  })

setMethod("addToPopups", "NewEdge",
          function(object, type, nodePopupMenu, i,
			   updateArguments, Args, ...)
          {
               tkadd(nodePopupMenu, "command",
                     label = paste(" --- This is a my new vertex!"),
                     command = function() { print(name(object))})
          })

# Why are these 2 * 7 methods not avaliable from "dg.VertexEdge" ?

setMethod("color", "NewEdge", function(object) object@color)
setReplaceMethod("color", "NewEdge",
                 function(x, value) {x@color <- value; x} )

setMethod("label", "NewEdge", function(object) object@label)
setReplaceMethod("label", "NewEdge",
                 function(x, value) {x@label <- value; x} )

setMethod("name", "NewEdge", function(object) object@label)
setReplaceMethod("name", "NewEdge",
                 function(x, value) {x@label <- value; x} )

setMethod("labelPosition", "NewEdge",
          function(object) object@label.position)
setReplaceMethod("labelPosition", "NewEdge",
                 function(x, value) {x@label.position <- value; x} )

setMethod("nodeIndices", "NewEdge", function(object) object@vextex.indices)
setReplaceMethod("nodeIndices", "NewEdge",
                 function(x, value) {x@vextex.indices <- value; x} )

setMethod("width", "NewEdge", function(object) object@width)
setReplaceMethod("width", "NewEdge",
                 function(x, value) {x@width <- value; x} )

setMethod("dash", "NewEdge", function(object) object@dash)
setReplaceMethod("dash", "NewEdge",
                 function(x, value) {x@dash <- value; x} )

setMethod("propertyDialog", "NewEdge",
          function(object, classes = NULL, title = class(object),
                   sub.title = label(object), name.object = name(object),
                   okReturn = TRUE,
                   fixedSlots = NULL, difficultSlots = NULL,
                   top = NULL, entryWidth = 20, do.grab = FALSE) {
  .propertyDialog(object, classes = classes, title = title,
                  sub.title = sub.title, name.object = name.object,
                  okReturn = okReturn, 
                  fixedSlots = fixedSlots, difficultSlots = difficultSlots,
                  top = top, entryWidth = entryWidth, do.grab = do.grab)
  })


V.Types <- c("Discrete", "Ordinal", "Discrete",
             "Continuous", "Discrete", "Continuous")

V.Names <- c("Sex", "Age", "Eye", "FEV", "Hair", "Shosize")
V.Labels <- paste(V.Names, 1:6, sep ="/")

From <- c(1, 2, 3, 4, 5, 6, 3)
To   <- c(2, 3, 4, 5, 6, 1, 6)

Z <- DynamicGraph(V.Names, V.Types, From, To, texts = c("Gryf", "gaf"),
                  edge.types = c("NewEdge",
                                 "VertexEdge",
                                 "Dashed",
                                 "Dotted",
                                 "DoubleArrow",
                                 "DoubleConnected",
                                 "TripleConnected"),
                  labels = V.Labels, 
                  updateEdgeLabels = FALSE, edgeColor = "green", 
                  vertexColor = "blue", edgeClasses = myEdgeClasses)

}

\keyword{ methods }
\keyword{ dynamic }
\keyword{ graphs }

\eof
\name{validFactorClasses}

\alias{factorClasses}
\alias{validFactorClasses}

\title{Valid factor vertex classes}

\description{Return matrix with labels of valid factor vertex classes
and the valid factor vertex classes. }

\usage{validFactorClasses()}

\details{The argument \code{factorClasses} to \code{\link{DynamicGraph}},
 and to \code{\link{returnFactorVerticesAndEdges}} and \code{\link{newFactor}}
 is by default the returned value of this function.
 If new factor vertex classes are created then \code{factorClasses}
 should be set to a value with this returned value extended appropriate. }
\value{Matrix of text strings with labels (used in dialog windows)
       of valid factor vertex classes and the valid factor 
       vertex classes (used to create the factor vertices).}

\author{Jens Henrik Badsberg}

\seealso{\code{\link{validVertexClasses}}}

\examples{validFactorClasses()}

\keyword{ methods }
\keyword{ dynamic }
\keyword{ graphs }

\eof
\name{validVertexClasses}

\alias{vertexClasses}
\alias{validVertexClasses}

\title{Valid vertex classes}

\description{Return matrix with labels of valid vertex classes and the valid vertex classes}

\usage{validVertexClasses()}

\details{The argument \code{vertexClasses} to
    \code{\link{dynamicGraphMain}} and
    \code{\link{DynamicGraph}}, and to
    \code{\link{newVertex}} and
    \code{\link{returnVertexList}}
    is by default the returned value of this
    function. If new vertex classes are created then \code{vertexClasses}
    should be set to a value with this returned value extended appropriate. }
\value{Matrix of text strings with labels (used in dialog windows)
       of valid vertex classes
       and the valid vertex classes (used to create the vertices).}

\author{Jens Henrik Badsberg}

\seealso{ \code{\link{validEdgeClasses}}. }

\examples{
require(tcltk)

# Test with new vertex class (demo(Circle.newVertex)):

setClass("NewVertex", contains = "dg.Vertex", 
         representation(my.text   = "character",
                        my.number = "numeric"), 
         prototype(my.text    = "",
                   my.number  = 2))

myVertexClasses <- rbind(validVertexClasses(), 
                         NewVertex = c("NewVertex", "NewVertex"))

setMethod("draw", "NewVertex",
          function(object, canvas, position,
                   x = position[1], y = position[2], stratum = 0,
                   w = 2, color = "green", background = "white")
          {
            s <- w * sqrt(4 / pi) / 2
            p1 <- tkcreate(canvas, "oval",
                           x - s - s, y - s, x + s - s, y + s,
                           fill = color(object), activefill = "IndianRed")
            p2 <- tkcreate(canvas, "oval",
                           x - s + s, y - s, x + s + s, y + s, 
                           fill = color(object), activefill = "IndianRed")
            p3 <- tkcreate(canvas, "oval",
                           x - s, y - s - s, x + s, y + s - s, 
                           fill = color(object), activefill = "IndianRed")
            p4 <- tkcreate(canvas, "poly",  x - 1.5 * s, y + 3 * s,
                           x + 1.5 * s, y + 3 * s, x, y, 
                           fill = color(object), activefill = "SteelBlue")
            return(list(dynamic = list(p1, p2, p3, p4), fixed = NULL)) })

setMethod("addToPopups", "NewVertex",
          function(object, type, nodePopupMenu, i,
			   updateArguments, Args, ...)
          {
               tkadd(nodePopupMenu, "command",
                     label = paste(" --- This is a my new vertex!"),
                     command = function() { print(name(object))})
          })

if (!isGeneric("my.text")) {
  if (is.function("my.text"))
    fun <- my.text
  else
    fun <- function(object) standardGeneric("my.text")
  setGeneric("my.text", fun)
}
setGeneric("my.text<-",
           function(x, value) standardGeneric("my.text<-"))

setMethod("my.text", "NewVertex",
          function(object) object@my.text)
setReplaceMethod("my.text", "NewVertex",
                 function(x, value) {x@my.text <- value; x} )

if (!isGeneric("my.number")) {
  if (is.function("my.number"))
    fun <- my.number
  else
    fun <- function(object) standardGeneric("my.number")
  setGeneric("my.number", fun)
}
setGeneric("my.number<-",
           function(x, value) standardGeneric("my.number<-"))

setMethod("my.number", "NewVertex",
          function(object) object@my.number)
setReplaceMethod("my.number", "NewVertex",
                 function(x, value) {x@my.number <- value; x} )

# Why are these 2 * 7 methods not avaliable from "dg.Vertex" ?

setMethod("color", "NewVertex",
          function(object) object@color)
setReplaceMethod("color", "NewVertex",
                 function(x, value) {x@color <- value; x} )

setMethod("label", "NewVertex",
          function(object) object@label)
setReplaceMethod("label", "NewVertex",
                 function(x, value) {x@label <- value; x} )

setMethod("labelPosition", "NewVertex",
          function(object) object@label.position)
setReplaceMethod("labelPosition", "NewVertex",
                 function(x, value) {x@label.position <- value; x} )

setMethod("name", "NewVertex",
          function(object) object@name)
setReplaceMethod("name", "NewVertex",
                 function(x, value) {x@name <- value; x} )

setMethod("index", "NewVertex",
          function(object) object@index)
setReplaceMethod("index", "NewVertex",
                 function(x, value) {x@index <- value; x} )

setMethod("position", "NewVertex", 
          function(object) object@position)
setReplaceMethod("position", "NewVertex",
                 function(x, value) {x@position <- value; x} )

setMethod("stratum", "NewVertex",
          function(object) object@stratum)
setReplaceMethod("stratum", "NewVertex",
                 function(x, value) {x@stratum <- value; x} )

setMethod("propertyDialog", "NewVertex",
          function(object, classes = NULL, title = class(object),
                   sub.title = label(object), name.object = name(object),
                   okReturn = TRUE,
                   fixedSlots = NULL, difficultSlots = NULL,
                   top = NULL, entryWidth = 20, do.grab = FALSE) {
  .propertyDialog(object, classes = classes, title = title,
                  sub.title = sub.title, name.object = name.object,
                  okReturn = okReturn, 
                  fixedSlots = fixedSlots, difficultSlots = difficultSlots,
                  top = top, entryWidth = entryWidth, do.grab = do.grab)
  })

V.Types <- rep("NewVertex", 6)

V.Names <- c("Sex", "Age", "Eye", "FEV", "Hair", "Shosize")
V.Labels <- paste(V.Names, 1:6, sep ="/")

From <- c(1, 2, 3, 4, 5, 6)
To   <- c(2, 3, 4, 5, 6, 1)

Z <- DynamicGraph(V.Names, V.Types, From, To, texts = c("Gryf", "gaf"),
                  labels = V.Labels, 
                  updateEdgeLabels = FALSE, edgeColor = "green", 
                  vertexColor = "blue", vertexClasses = myVertexClasses)
}

\keyword{ methods }
\keyword{ dynamic }
\keyword{ graphs }

\eof
\name{validViewClasses}

\alias{viewClasses}
\alias{validViewClasses}

\title{Valid view classes}

\description{Return matrix with labels of valid view classes
and the valid view classes}

\usage{validViewClasses()}

\details{The argument \code{viewClasses} to \code{\link{dynamicGraphMain}} and
    \code{\link{DynamicGraph}} is by default the returned value of this
    function. If new view classes are created then \code{viewClasses}
    should be set to a value with this returned value extended appropriate. }

\value{Matrix of text strings with labels (used in dialog windows) of valid
       view classes and the valid view classes (used to create the views).}

\author{Jens Henrik Badsberg}

\seealso{ \code{\link{DynamicGraphView-class}}. }

\examples{validViewClasses()}

\keyword{ methods }
\keyword{ dynamic }
\keyword{ graphs }

\eof
